/*
 * JPLOT  -- Java Plotting Interface for any programme or
 *           as an independent GUI
 * 
 * Copyright (C) 1999 Jan van der Lee
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA.
 *
 * Send bugs, suggestions or queries to <jplot@cig.ensmp.fr>
 * The latest releases are found at 
 * http://www.cig.ensmp.fr/~vanderlee/jplot.html
 *
 * Initally developed for use by the Centre d'Informatique Geologique 
 * Ecole des Mines de Paris, Fontainebleau, France.
 */

package jplot;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.text.*;
import javax.swing.table.*;

import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;

/**
 * Builds a panel with Tabs, containing optional parameters
 * for labels, legends, colors, etc. Calling the <code>show</code>
 * method shows a dialog with apply and cancel buttons.
 */
public class AxesPanel extends JPanel {

  private JTabbedPane tabbedPane;
  private JDialog dialog;
  final private String title = "Axes settings";

  private ColorPanel colorPanel;
  private AxesOptions axesPanel;
  private GridOptions gridPanel;
  private BoxOptions boxPanel;
  private GraphPars gp;
  private JPlot jplot;

  public AxesPanel(JPlot jp, GraphPars _gp) {
    gp = _gp;
    jplot = jp;

    tabbedPane = new JTabbedPane();
    //tabbedPane.setTabPlacement(SwingConstants.LEFT);

    // add the options tabs:
    axesPanel = new AxesOptions();
    tabbedPane.add("Axes and tics",axesPanel);
    gridPanel = new GridOptions();
    tabbedPane.add("Grid options",gridPanel);
    boxPanel = new BoxOptions();
    tabbedPane.add("Boxes and fillcolors",boxPanel);
    add(tabbedPane);

    // build the dialog
    JPanel panel = new JPanel(new BorderLayout());
    dialog = new JDialog(jplot.frame,title,false);
    dialog.addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
	dialog.dispose();
      }
    });
    JPanel p = new JPanel(new FlowLayout());
    p.setBorder(BorderFactory.createEtchedBorder());
    JButton b = new JButton("Apply");
    b.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	setValues();
	jplot.updateGraphIfShowing();
	gp.setDataChanged(true);
      }
    });
    p.add(b);
    b = new JButton("Dismiss");
    b.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	dialog.dispose();
      }
    });
    p.add(b);
    panel.add(this,BorderLayout.CENTER);
    panel.add(p,BorderLayout.SOUTH);
    dialog.getContentPane().add(panel);
  }

  public void setValues() {
    axesPanel.setValues();
    gridPanel.setValues();
    boxPanel.setValues();
  }

  public void refresh() {
    axesPanel.refresh();
    gridPanel.refresh();
    boxPanel.refresh();
  }
  
  public void show(int x, int y) {
    refresh();
    dialog.setLocation(x,y);
    dialog.pack();
    dialog.show();
  }

  private Color getColor(Color color) {
    if (colorPanel == null) {
      colorPanel = new ColorPanel(jplot.frame,color);
    }
    else colorPanel.refresh(color);
    return colorPanel.show(jplot.frame,color,100,100);
  }


  public class AxesOptions extends JPanel {
    
    private JCheckBox[] cb_showAxis = new JCheckBox [gp.NAXES];
    private JCheckBox[] cb_mirrorAxis = new JCheckBox [gp.NAXES];
    private JCheckBox[] cb_showTics = new JCheckBox [gp.NAXES];
    private JCheckBox[] cb_mirrorTics = new JCheckBox [gp.NAXES];
    private JCheckBox[] cb_rotateTics = new JCheckBox [gp.NAXES];
    private JCheckBox[] cb_numTics = new JCheckBox [gp.NAXES];
    private JTextField[] numTics = new JTextField [gp.NAXES];
    private JTextField[] tlField = new JTextField [gp.NAXES];

    private LineStyle lineStyle;
    private Color axesColor;
    private JPanel linePanel;

    private JCheckBox cb_autoRatio;
    private JCheckBox cb_manuRatio;
    private JTextField ratio;

    /**
     * Principal constructor. Builds a panel with axes options.
     * @param gp instance of the object containing all graphical parameters
     */
    public AxesOptions () {
      EmptyBorder border = new EmptyBorder(new Insets(0,0,0,10));
      setBorder(new EmptyBorder(new Insets(5,5,5,5)));
      Dimension mediumField = new Dimension(100,20);
      setLayout(new BoxLayout(this,BoxLayout.Y_AXIS));
      JLabel label;
      GriddedPanel p;

      // make the widgets:
      makeRatio();
      for (int k=0; k<gp.NAXES; k++) {
	makeWidgets(k);
	p = new GriddedPanel();
	if (k == 0) p.setBorder(new TitledBorder(new EtchedBorder(),"X-Axis"));
	else p.setBorder(new TitledBorder(new EtchedBorder(),"Y-Axis"));
	label = new JLabel("Axis:");
	label.setBorder(border);
	p.addComponent(label,1,1);
	p.addComponent(cb_showAxis[k],1,2);
	p.addComponent(cb_mirrorAxis[k],1,3);

	label = new JLabel("Tics:");
	label.setBorder(border);
	p.addComponent(label,2,1);
	p.addComponent(cb_showTics[k],2,2);
	p.addComponent(cb_mirrorTics[k],2,3);
	p.addComponent(cb_rotateTics[k],2,4);

	p.addComponent(cb_numTics[k],3,2);
	p.addComponent(numTics[k],3,3);
	JPanel lPanel = new JPanel(new BorderLayout());
	lPanel.add(Utils.makeLabel("length"),BorderLayout.WEST);
	lPanel.add(tlField[k],BorderLayout.CENTER);
	p.addComponent(lPanel,3,4);
	add(p);
      }

      p = new GriddedPanel();
      p.setBorder(new EtchedBorder());
      label = new JLabel("X/Y ratio:");
      label.setBorder(border);
      p.addComponent(label,1,1);
      p.addComponent(cb_autoRatio,1,2);
      p.addComponent(cb_manuRatio,1,3);
      p.addFilledComponent(ratio,1,4);

      JButton b_color = new JButton(jplot.getImageIcon("color.jpg"));
      b_color.addActionListener(new ActionListener() {
	  public void actionPerformed(ActionEvent e) {
	    Color color = getColor(axesColor);
	    if (color != null) {
	      axesColor = color;
	      lineStyle.setColor(axesColor);
	      linePanel.repaint();
	    }
	  }
	});
      b_color.setMargin(new Insets(0,0,0,0));
      b_color.setToolTipText("Set the axes color");

      lineStyle = new LineStyle(new LinePars());
      lineStyle.setColor(axesColor);
      linePanel = new JPanel(new FlowLayout());
      linePanel.add(lineStyle);
      linePanel.add(b_color);
      
      label = new JLabel("Axes color:");
      label.setBorder(border);
      p.addComponent(label,2,1);
      p.addComponent(linePanel,2,2);
      add(p);
    }
    
    private void makeRatio() {
      ButtonGroup bg = new ButtonGroup();
      cb_autoRatio = new JCheckBox("automatic",!gp.useAxesRatio());
      cb_autoRatio.addActionListener(new ActionListener() {
	  public void actionPerformed(ActionEvent e) {	
	    ratio.setEnabled(false);
	  }
	});
      cb_manuRatio = new JCheckBox("manual:",gp.useAxesRatio());
      cb_manuRatio.addActionListener(new ActionListener() {
	  public void actionPerformed(ActionEvent e) {	
	    ratio.setEnabled(true);
	  }
	});
      bg.add(cb_autoRatio);
      bg.add(cb_manuRatio);
      ratio = new JTextField(Float.toString((float)gp.getAxesRatio()));
      ratio.setColumns(7);
      ratio.setEnabled(false);
    }
    
    /**
     * Builds a panel with the scaling options for the axes.
     * @param axis axis for which this panel is built.
     */
    private void makeWidgets(final int axis) {
      cb_showTics[axis] = new JCheckBox("show",gp.drawTics(axis));
      cb_showTics[axis].addActionListener(new ActionListener() {
	  public void actionPerformed(ActionEvent e) {	
	    boolean b = cb_showTics[axis].isSelected()? true : false;
	    tlField[axis].setEnabled(b);
	    cb_numTics[axis].setEnabled(b);
	    numTics[axis].setEnabled(b);
	    cb_mirrorTics[axis].setEnabled(b);
	    cb_rotateTics[axis].setEnabled(b);
	  }
	});

      cb_showAxis[axis] = new JCheckBox("show",gp.drawAxis(axis));
      cb_showAxis[axis].addActionListener(new ActionListener() {
	  public void actionPerformed(ActionEvent e) {	
	    boolean b = cb_showAxis[axis].isSelected()? true : false;
	    cb_mirrorAxis[axis].setEnabled(b);
	  }
	});
      
      tlField[axis] = new JTextField();
      tlField[axis].setColumns(4);
      
      cb_mirrorAxis[axis] = new JCheckBox("mirror",gp.drawMirrorAxis(axis));
      cb_mirrorTics[axis] = new JCheckBox("mirror",gp.drawMirrorTics(axis));
      cb_rotateTics[axis] = new JCheckBox("rotate",gp.rotateTics(axis));
      
      cb_numTics[axis] = new JCheckBox("set number:",gp.useNumberOfTics(axis));
      cb_numTics[axis].addActionListener(new ActionListener() {
	  public void actionPerformed(ActionEvent e) {	
	    numTics[axis].setEnabled(cb_numTics[axis].isSelected());
	  }
	});
      numTics[axis] = new JTextField(Integer.toString(gp.getNumberOfTics(axis)));
      numTics[axis].setColumns(4);
      numTics[axis].setEnabled(cb_numTics[axis].isSelected());
    }

    /**
     * Updates the panel with the current graphical parameters.
     */
    public void refresh() {
      for (int k=0; k<gp.NAXES; k++) {
	cb_showAxis[k].setSelected(gp.drawAxis(k));
	cb_mirrorAxis[k].setSelected(gp.drawMirrorAxis(k));
	cb_showTics[k].setSelected(gp.drawTics(k));
	cb_mirrorTics[k].setSelected(gp.drawMirrorTics(k));
	tlField[k].setText(Float.toString((float)gp.getTicLength(k)));
	cb_numTics[k].setSelected(gp.useNumberOfTics(k));
	numTics[k].setText(Integer.toString(gp.getNumberOfTics(k)));
      }
      cb_manuRatio.setSelected(gp.useAxesRatio());
      cb_autoRatio.setSelected(!gp.useAxesRatio());
      ratio.setText(Float.toString((float)gp.getAxesRatio()));
      axesColor = gp.getAxesColor();
      lineStyle.setColor(axesColor);
    }  
    
    public void setValues() {
      for (int k=0; k<gp.NAXES; k++) {
	gp.setDrawAxis(k,cb_showAxis[k].isSelected());
	gp.setDrawMirrorAxis(k,cb_mirrorAxis[k].isSelected());
	gp.setDrawTics(k,cb_showTics[k].isSelected());
	gp.setDrawMirrorTics(k,cb_mirrorTics[k].isSelected());
	gp.setTicLength(k,Float.parseFloat(tlField[k].getText()));
	gp.setUseNumberOfTics(k,cb_numTics[k].isSelected());
	if (cb_numTics[k].isSelected()) {
	  if (!numTics[k].getText().equals("")) {
	    gp.setNumberOfTics(k,Integer.parseInt(numTics[k].getText()));
	  }	    
	}
      }
      gp.setUseAxesRatio(cb_manuRatio.isSelected());
      if (!ratio.getText().equals("")) {
	gp.setAxesRatio(Double.parseDouble(ratio.getText()));
      }
      gp.setAxesColor(axesColor);
    }
  }
  
  public class GridOptions extends GriddedPanel {

    private JCheckBox cb_hg,cb_vg,cb_front,cb_back;
    private JPanel gridStylePanel, boxFillPanel, innerPanel;
    private LineStyle lineStyle;
    private Color actualColor;

    /**
     * Principal constructor. Builds a panel with axes options.
     * @param gp instance of the object containing all graphical parameters
     */

    public GridOptions () {
      setBorder(new EmptyBorder(new Insets(5,5,5,5)));
      EmptyBorder border = new EmptyBorder(new Insets(0,0,0,10));
      
      cb_hg = new JCheckBox("y-axis",gp.drawGrid(gp.Y_AXIS));
      cb_vg = new JCheckBox("x-axis",gp.drawGrid(gp.X_AXIS));
      cb_back = new JCheckBox("back",!gp.gridToFront());
      cb_front = new JCheckBox("front",gp.gridToFront());
      
      ButtonGroup bg = new ButtonGroup();
      bg.add(cb_back);
      bg.add(cb_front);
      
      // linestyle and color panel for grid:
      lineStyle = new LineStyle(new LinePars());
      lineStyle.setColor(actualColor);
      JButton b_color = new JButton(jplot.getImageIcon("color.jpg"));
      b_color.addActionListener(new ActionListener() {
	  public void actionPerformed(ActionEvent e) {
	    Color color = getColor(actualColor);
	    if (color != null) {
	      actualColor = color;
	      lineStyle.setColor(actualColor);
	      lineStyle.repaint();
	    }
	  }
	});
      b_color.setMargin(new Insets(0,0,0,0));
      b_color.setToolTipText("Gridline color");
      gridStylePanel = new JPanel(new FlowLayout());
      gridStylePanel.add(lineStyle);
      gridStylePanel.add(b_color);
      
      JLabel label = new JLabel("Grid lines:");
      label.setBorder(border);
      addComponent(label,1,1);
      addComponent(cb_hg,1,2);
      addComponent(cb_vg,1,3);
      label = new JLabel("Drawing:");
      label.setBorder(border);
      addComponent(label,2,1);
      addComponent(cb_front,2,2);
      addComponent(cb_back,2,3);
      label = new JLabel("Color:");
      label.setBorder(border);
      addComponent(label,3,1);
      addComponent(gridStylePanel,3,2);
    }
    

    /**
     * Updates the panel with the current graphical parameters.
     */
    public void refresh() {
      cb_hg.setSelected(gp.drawGrid(gp.Y_AXIS));
      cb_vg.setSelected(gp.drawGrid(gp.X_AXIS));
      cb_back.setSelected(!gp.gridToFront());
      cb_front.setSelected(gp.gridToFront());
      actualColor = gp.getGridColor();
      lineStyle.setColor(actualColor);
    }  

    public void setValues() {
      gp.setGridToFront(cb_front.isSelected());
      gp.setGridColor(actualColor);
      gp.setDrawGrid(gp.X_AXIS,cb_vg.isSelected());
      gp.setDrawGrid(gp.Y_AXIS,cb_hg.isSelected());
    }
  }

  public class BoxOptions extends GriddedPanel {

    private SmallColorPreview graphFillColorPreview, triangleFillColorPreview;
    private SmallColorPreview boxFillColorPreview, panelFillColorPreview;
    private Color boxFillColor, triangleFillColor;
    private Color graphFillColor, panelFillColor;
    private Color boxLineColor;
    private JCheckBox cb_showBox, cb_hideBox;
    private JCheckBox cb_showInner, cb_hideInner;

    private JTextField offsetField;

    private JPanel graphFillPanel, boxFillPanel, panelFillPanel;
    private JPanel triangleFillPanel;
    private LineStyle lineStyle;
    
    //private GriddedPanel p;

    /**
     * Principal constructor. Builds a panel with axes options.
     * @param gp instance of the object containing all graphical parameters
     */
    public BoxOptions () {
      setBorder(new EmptyBorder(new Insets(5,5,5,5)));
      EmptyBorder border = new EmptyBorder(new Insets(0,0,0,10));

      JButton b_color = new JButton(jplot.getImageIcon("color.jpg"));
      b_color.addActionListener(new ActionListener() {
	  public void actionPerformed(ActionEvent e) {
	    Color color = getColor(panelFillColor);
	    if (color != null) {
	      panelFillColor = color;
	      panelFillColorPreview.setColor(panelFillColor);
	      panelFillPanel.repaint();
	    }
	  }
	});
      b_color.setMargin(new Insets(0,0,0,0));
      b_color.setToolTipText("Set the overall background fill color");
      panelFillColorPreview = new SmallColorPreview(panelFillColor,50,18);
      panelFillPanel = new JPanel(new FlowLayout());
      panelFillPanel.add(panelFillColorPreview);
      panelFillPanel.add(b_color);

      JLabel label = new JLabel("Panel background:");
      label.setBorder(border);
      addComponent(label,1,1);
      addComponent(panelFillPanel,1,2);

      b_color = new JButton(jplot.getImageIcon("color.jpg"));
      b_color.addActionListener(new ActionListener() {
	  public void actionPerformed(ActionEvent e) {
	    Color color = getColor(graphFillColor);
	    if (color != null) {
	      graphFillColor = color;
	      graphFillColorPreview.setColor(graphFillColor);
	      graphFillPanel.repaint();
	    }
	  }
	});
      b_color.setMargin(new Insets(0,0,0,0));
      b_color.setToolTipText("Set the graph background fill color");
      graphFillColorPreview = new SmallColorPreview(graphFillColor,50,18);
      graphFillPanel = new JPanel(new FlowLayout());
      graphFillPanel.add(graphFillColorPreview);
      graphFillPanel.add(b_color);

      label = new JLabel("Graph background:");
      label.setBorder(border);
      addComponent(label,2,1);
      addComponent(graphFillPanel,2,2);

      b_color = new JButton(jplot.getImageIcon("color.jpg"));
      b_color.addActionListener(new ActionListener() {
	  public void actionPerformed(ActionEvent e) {
	    Color color = getColor(boxFillColor);
	    if (color != null) {
	      boxFillColor = color;
	      boxFillColorPreview.setColor(boxFillColor);
	      boxFillPanel.repaint();
	    }
	  }
	});
      b_color.setMargin(new Insets(0,0,0,0));
      b_color.setToolTipText("Set the box fill color");
      boxFillColorPreview = new SmallColorPreview(boxFillColor,50,18);
      boxFillPanel = new JPanel(new FlowLayout());
      boxFillPanel.add(boxFillColorPreview);
      boxFillPanel.add(b_color);

      b_color = new JButton(jplot.getImageIcon("color.jpg"));
      b_color.addActionListener(new ActionListener() {
	  public void actionPerformed(ActionEvent e) {
	    Color color = getColor(boxLineColor);
	    if (color != null) {
	      boxLineColor = color;
	      lineStyle.setColor(boxLineColor);
	      lineStyle.repaint();
	    }
	  }
	});
      b_color.setMargin(new Insets(0,0,0,0));
      b_color.setToolTipText("Set the axes color");

      lineStyle = new LineStyle(new LinePars());
      lineStyle.setColor(boxLineColor);
      JPanel linePanel = new JPanel(new FlowLayout());
      linePanel.add(lineStyle);
      linePanel.add(b_color);

      offsetField = new JTextField();
      offsetField.setColumns(4);
      JPanel offsetPanel = new JPanel(new BorderLayout());
      offsetPanel.add(Utils.makeLabel("offset"),BorderLayout.WEST);
      offsetPanel.add(offsetField,BorderLayout.CENTER);

      ButtonGroup bg = new ButtonGroup();
      cb_showBox = new JCheckBox("Show");
      cb_hideBox = new JCheckBox("Hide");
      bg.add(cb_showBox);
      bg.add(cb_hideBox);
      label = new JLabel("Bounding box:");
      label.setBorder(border);
      addComponent(label,3,1);
      addComponent(cb_showBox,3,2);
      addComponent(cb_hideBox,3,3);
      addComponent(offsetPanel,4,2);
      addComponent(boxFillPanel,5,2);
      addComponent(linePanel,5,3);

      if (gp.getGraphType() == gp.GRAPHTYPE_PIPER) { 
	b_color = new JButton(jplot.getImageIcon("color.jpg"));
	b_color.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
	      Color color = getColor(triangleFillColor);
	      if (color != null) {
		triangleFillColor = color;
		triangleFillColorPreview.setColor(triangleFillColor);
		triangleFillPanel.repaint();
	      }
	    }
	  });
	b_color.setMargin(new Insets(0,0,0,0));
	b_color.setToolTipText("Set the box fill color");
	triangleFillColorPreview = new SmallColorPreview(triangleFillColor,50,18);
	triangleFillPanel = new JPanel(new FlowLayout());
	triangleFillPanel.add(triangleFillColorPreview);
	triangleFillPanel.add(b_color);
	
	bg = new ButtonGroup();
	cb_showInner = new JCheckBox("Show");
	cb_hideInner = new JCheckBox("Hide");
	bg.add(cb_showInner);
	bg.add(cb_hideInner);
	label = new JLabel("Inner triangles:");
	label.setBorder(border);
	addComponent(label,5,1);
	addComponent(cb_showInner,5,2);
	addComponent(cb_hideInner,5,3);
	addComponent(triangleFillPanel,6,2);
      }
    }

    /**
     * Updates the panel with the current graphical parameters.
     */
    public void refresh() {
      panelFillColor = gp.getBackgroundColor();
      panelFillColorPreview.setColor(panelFillColor);
      graphFillColor = gp.getGraphBackgroundColor();
      graphFillColorPreview.setColor(graphFillColor);
      boxFillColor = gp.getBoxFillColor();
      boxFillColorPreview.setColor(boxFillColor);
      boxLineColor = gp.getBoxColor();
      lineStyle.setColor(boxLineColor);
      cb_showBox.setSelected(gp.showBox());
      cb_hideBox.setSelected(!gp.showBox());
      offsetField.setText(Float.toString(gp.getBoxOffset()));
      if (gp.getGraphType() == gp.GRAPHTYPE_PIPER) {
	triangleFillColor = gp.getInnerColor();
	triangleFillColorPreview.setColor(triangleFillColor);
	cb_showInner.setSelected(gp.showInner());
	cb_hideInner.setSelected(!gp.showInner());
      }
    }  

    public void setValues() {
      gp.setBackgroundColor(panelFillColor);
      gp.setGraphBackgroundColor(graphFillColor);
      gp.setBoxFillColor(boxFillColor);
      gp.setBoxColor(boxLineColor);
      gp.setShowBox(cb_showBox.isSelected());
      gp.setBoxOffset(Float.parseFloat(offsetField.getText()));
      if (gp.getGraphType() == gp.GRAPHTYPE_PIPER) {
	gp.setInnerColor(triangleFillColor);
	gp.setShowInner(cb_showInner.isSelected());
      }
    }
  }
}
