/*
 * JPLOT  -- Java Plotting Interface for any programme or
 *           as an independent GUI
 * 
 * Copyright (C) 1999 Jan van der Lee
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA.
 *
 * Send bugs, suggestions or queries to <jplot@cig.ensmp.fr>
 * The latest releases are found at 
 * http://www.cig.ensmp.fr/~vanderlee/jplot.html
 *
 * Initally developed for use by the Centre d'Informatique Geologique 
 * Ecole des Mines de Paris, Fontainebleau, France.
 */

package jplot;
import java.util.*;
import java.awt.*;

/**
 * The <code>DataArray</code> class is a special-purpose array class
 * for use with the graphical plotting program JPlot. The class
 * stores the data used for the plot in terms of X, Y1, Y2 etc.
 * arrays specifies its plotting style (color, linestyle, pointtype 
 * etc.).
 *
 * @version 24/08/99
 * @author  J. van der Lee
 */
public class DataArray extends LinePars {

  protected Vector points;
  protected double[] maxValue = new double [GraphPars.NAXES];
  protected double[] minValue = new double [GraphPars.NAXES];
  protected double[] lowestNonZeroValue = new double [GraphPars.NAXES];

  /**
   * Default constructor, initializes the class with nothing.
   */
  public DataArray() {
    this(0);
  }

  /**
   * Constructor, allocates memory for the data points.
   * @param s name of this column
   * @param n maximum number of datapoints.
   */
  public DataArray(int n) {
    allocate(n);
  }

  /**
   * Constructor, allocates memory for the data points.
   * @param n maximum number of datapoints.
   * @param s name of this column
   * @param lp line parameters class, contains info of how to draw the line.
   */
  public DataArray(int n, LinePars lp) {
    super(lp);
    allocate(n);
  }

  /**
   * Allocation of memory for the data points.
   * @param n maximum number of datapoints.
   */
  public void allocate(int n) {
    if (n < 10) n = 10;
    points = new Vector(n,10);
    for (int k=0; k<GraphPars.NAXES; k++) {
      maxValue[k] = -GraphPars.INF;
      minValue[k] = GraphPars.INF;
      lowestNonZeroValue[k] =GraphPars.INF;
    }
  }

  /**
   * Updates maximum and minimum values.
   * @param x just entered x-value of the point
   * @param y just entered y-value of the point
   */
  public void updateMinMax(double x, double y) {
    if (x > maxValue[GraphPars.X_AXIS]) maxValue[GraphPars.X_AXIS] = x;
    if (x < minValue[GraphPars.X_AXIS]) minValue[GraphPars.X_AXIS] = x;
    if (x < lowestNonZeroValue[GraphPars.X_AXIS] && x > 0.0) {
      lowestNonZeroValue[GraphPars.X_AXIS] = x;
    }
    if (y > maxValue[GraphPars.Y_AXIS]) maxValue[GraphPars.Y_AXIS] = y;
    if (y < minValue[GraphPars.Y_AXIS]) minValue[GraphPars.Y_AXIS] = y;    
    if (y < lowestNonZeroValue[GraphPars.Y_AXIS] && y > 0.0) {
      lowestNonZeroValue[GraphPars.Y_AXIS] = y;
    }
  }

  /**
   * Adds the values of a plot-point pair (x,y).
   * This point is added at the end of the array.
   * @param i index of the plot-point
   * @param x x-value of the plot-point
   * @param y y-value of the plot-point
   */
  public void addPoint(double x, double y) {
    y *= multiplier;
    y += additioner;
    updateMinMax(x,y);
    points.add(new PlotPoint(x,y));
  }

  /**
   * Adds the values of a plot-point pair (x,y).
   * This point is added at the end of the array.
   * @param i index of the plot-point
   * @param x x-value of the plot-point
   * @param y y-value of the plot-point
   * @param liftPen true if the pen should lift after this point
   */
  public void addPoint(double x, double y, boolean liftPen) {
    y *= multiplier;
    y += additioner;
    updateMinMax(x,y);
    points.add(new PlotPoint(x,y,liftPen));
  }

  /**
   * Sets the values of a plot-point pair (x,y).
   * @param i index of the plot-point
   * @param x x-value of the plot-point
   * @param y y-value of the plot-point
   */
  public void setPoint(int i, double x, double y) {
    if (i >= 0 && i < points.size()) {
      updateMinMax(x,y);
      points.add(i,new PlotPoint(x,y));
    }
  }
  
  /**
   * Returns the plot-point of the specified index.
   * @param i index of the plot-point
   * @return plotpoint at index i
   */
  public PlotPoint getPoint(int i) {
    if (i >= 0 && i < points.size()) return (PlotPoint) points.get(i);
    return null;
  }
  
  /**
   * Defines that, for this plotpoint, the pen should lift.
   * This allows to draw discontinuous graphs.
   * @param i index of the plot-point
   */
  public void setLiftPen(int i) {
    if (i >= 0 && i < points.size()) {
      ((PlotPoint) points.get(i)).setLiftPen(true);
    }
  }

  public boolean liftPen(int i) {
    return ((PlotPoint) points.get(i)).liftPen();
  }
  
  /**
   * Return the length of the data vector.
   * @return length of the PlotPoint vector
   */
  public int size() {
    return points.size();
  }

  /**
   * Return the data vector
   * @return data vector with all the plot points
   */
  public Vector getData() {
    return points;
  }

  /**
   * Return a specific X-value. This function returns POSINF (1e300)
   * if index i falls beyond the valid range.
   * @param i index of the array
   * @return the value of x at index i
   */
  double getX(int i) {
    if (i >= 0 && i < size()) {
      return ((PlotPoint)points.get(i)).getX();
    }
    return GraphPars.INF;
  }

  /**
   * Return a specific Y-value. This function returns POSINF (1e300)
   * if index i falls beyond the valid range.
   * @param i index of the array
   * @return the value of y at index i
   */
  double getY(int i) {
    if (i >= 0 && i < size()) {
      return ((PlotPoint)points.get(i)).getY();
    }
    return GraphPars.INF;
  }
  
  /**
   * Returns the maximum value in the range. Careful, no error
   * checking on the value of axis, which should be less than
   * NAXES, defined in GraphPars.
   * @param axis defines to which axis this function applies.
   * @return the maximum value.
   */
  double getMaxValue(int axis) {
    return maxValue[axis];
  }

  /**
   * Returns the minimum value in the range. Careful, no error
   * checking on the value of axis, which should be less than
   * NAXES, defined in GraphPars.
   * @param axis defines to which axis this function applies.
   * @return the minimum value.
   */
  double getMinValue(int axis) {
    return minValue[axis];
  }
  
  /**
   * Returns the lowest, but non-zero value in the range. This is a
   * a usefull 'minimum' value for e.g. logarithmic ranges.
   * @param axis defines to which axis this function applies.
   * @return the minimum value.
   */
  double getLowestNonZeroValue(int axis) {
    return lowestNonZeroValue[axis];
  }
  
  /**
   * Returns the stroke used to plot these points.
   * By default, a solid line of width 1 is assumed.
   * @return the actual stroke.
   */
  BasicStroke getStroke() {
    float[] dash = {dashLength,dashLength};
    if (dashLength == 0.0f) return new BasicStroke(penWidth);
    return new BasicStroke(penWidth,BasicStroke.CAP_BUTT,
			   BasicStroke.JOIN_BEVEL,0,dash,0);
  }

  /**
   * Sets the drawing style parameters at once.
   * @param l line and point drawing parameters.
   */
  void setLinePars(LinePars l) {
    copy(l);
  }

  /**
   * Set the name of this dataset.
   * @param s string which contains the name of the dataset
   */
  /*
  void setName(String s) {
    name = s;
    }*/
  
  /**
   * Sorts the arrays in ascending or descending order.
   * @param ascending true if we should sort from small to large
   */
  public void sort(boolean ascending) {
    if (ascending) {
      // sort-routing in ascending order
    }
    else {
      // sort-routing in descending order
    }
  }
}

