/*
 * JPLOT  -- Java Plotting Interface for any programme or
 *           as an independent GUI
 * 
 * Copyright (C) 1999 Jan van der Lee
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA.
 *
 * Send bugs, suggestions or queries to <jplot@cig.ensmp.fr>
 * The latest releases are found at 
 * http://www.cig.ensmp.fr/~vanderlee/jplot.html
 *
 * Initally developed for use by the Centre d'Informatique Geologique 
 * Ecole des Mines de Paris, Fontainebleau, France.
 */

package jplot;

import java.io.*;
import java.util.*;

/**
 * This class contains all the info specific for one datafile.
 * The PlotPanel provides the columns to-be-plotted for
 * a specific file, with a line-style class for each column.
 * This class keeps track of this information.
 */
public class DataFile {
  private File file;
  private Vector items;
  private Vector columns;
  //private Vector columnNames;
  private int xColumn;
  private Vector styles;
  private String abbreviatedFilename;
  private String filenameWithoutPath;
  private final int maxLen=45;
  private int index;
  private int graphType;

  private GraphPars gp;
  
  /**
   * Builds the class from a filename. As soon as the datafile is
   * build, read the header of the file and make a list of all the
   * available columns.
   * @param file file for this set
   * @param index index, kind of identifier of this instance, 
   * @param labels vector to store the labels defined in the datafile
   * corresponding to the index of the tabbedPanes.
   */
  DataFile(File file, int index, GraphPars gp) {
    this.file = file;
    this.index = index;
    this.gp = gp;
    columns = new Vector();
    styles = new Vector();
    items = new Vector();
    graphType = gp.GRAPHTYPE_2D;
    int len = file.toString().length();
    if (len > maxLen) {
      abbreviatedFilename = "..." + file.toString().substring(len-maxLen,len);
    }
    else abbreviatedFilename = file.toString();
    int i = abbreviatedFilename.lastIndexOf(JPlot.FS,0);
    len = abbreviatedFilename.length();
    if (i > -1) filenameWithoutPath = abbreviatedFilename.substring(i,len);
    else filenameWithoutPath = abbreviatedFilename;
    loadColumnNames(gp.getLabels());
  }

  /**
   * Reads the datafile for data. The data is put in a specific purpose
   * DataArray object, which contains x- and y values. Note that all
   * blank lines or lines starting with '#' are skipped. One exception:
   * lines starting with '# column 1:' etc. are supposed to provide
   * the name of the column, hence '# column 1: nodes' means that column
   * 1 has the name 'nodes', which will be used by this program in order
   * to easily select columns.
   *
   * If the list of column names existed already, and if the new list
   * of names is identical, return true. Else delete the current
   * selection and return false.
   *
   * @param labels vector to store the labels defined in the datafile
   * @return true if the current selection can be maintained, false otherwise.
   */
  public boolean loadColumnNames(Vector labels) {
    boolean res = false;
    Vector newItems = new Vector();

    // mark the labels eventually present and given by a dataset
    // with CHECK : if they are found in the new dataset, we rename
    // their type to 'DATA' which allows us to keep the settings
    // such as font, color etc.
    Vector newLabels = new Vector();
    for (Enumeration e=labels.elements(); e.hasMoreElements();) {
      GraphLabel g = (GraphLabel) e.nextElement();
      if (g.equals(GraphLabel.DATA) && g.getFile() == file) {
	g.setID(GraphLabel.CHECK);
      }
      newLabels.add(g);
    }

    try {
      BufferedReader in = new BufferedReader(new FileReader(file));
      String s;
      int k = 1;
      while ((s=in.readLine()) != null) {
	s = s.trim();
	if (s.length() == 0) continue;
	StringTokenizer st = new StringTokenizer(s," \t,:");
	String token = st.nextToken();
	if (token.startsWith("#")) {
	  if (!st.hasMoreTokens()) continue;
	  String nextToken = st.nextToken();
	  if (st.hasMoreTokens()) {
	    if (nextToken.startsWith("graph")) {
	      token = st.nextToken();
	      if (token.startsWith("piper")) graphType = gp.GRAPHTYPE_PIPER;
	      else graphType = gp.GRAPHTYPE_2D;
	      continue;
	    }
	    else if (nextToken.equals("column")) {
	      st.nextToken();  // eat column number, e.g. '2:'
	      token = st.nextToken();
	      while (st.hasMoreTokens()) token += " " + st.nextToken();
	      newItems.addElement(token);
	      continue;
	    }
	    else if (nextToken.equals("row")) {
	      st.nextToken();  // eat row number, e.g. '2:'
	      token = st.nextToken();
	      while (st.hasMoreTokens()) token += " " + st.nextToken();
	      newItems.addElement(token);
	      continue;
	    }

	    // get the x-label from the dataset, if any
	    else if (nextToken.startsWith("xlabel") && st.hasMoreTokens()) {
	      GraphLabel gl = 
		new GraphLabel(GraphLabel.XLABEL,getLabel(st));
	      addLabel(gl,newLabels);
	      continue;
	    }

	    // get the y-label from the dataset, if any
	    else if (nextToken.startsWith("ylabel") && st.hasMoreTokens()) {
	      GraphLabel gl = 
		new GraphLabel(GraphLabel.YLABEL,getLabel(st));
	      gl.setRotation(Math.PI*1.5);
	      addLabel(gl,newLabels);
	      continue;
	    }

	    // random labels:
	    else if (nextToken.startsWith("label") && st.hasMoreTokens()) {
	      GraphLabel gl = 
		new GraphLabel(GraphLabel.DATA,getLabel(st));
	      gl.setFile(file);
	      if (st.hasMoreTokens()) {
		double xx,yy;
		xx = Double.parseDouble(st.nextToken());
		if (st.hasMoreTokens()) {
		  yy = Double.parseDouble(st.nextToken());
		  gl.setDataLocation(xx,yy);
		  if (st.hasMoreTokens()) {
		    gl.setDataRotation(Math.PI*Double.parseDouble(st.nextToken())/180.0);
		  }
		}
		else gl.setRotation(Math.PI*xx/180);
	      }
	      addLabel(gl,newLabels);
	      continue;
	    }
	    else continue;
	  }
	}
	else if (newItems.size() == 0) {
	  newItems.addElement("column " + Integer.toString(k++));
	  while (st.hasMoreTokens()) {
	    token = st.nextToken();
	    newItems.addElement("column " + Integer.toString(k++));
	  }
	  break;
	}
      }
      in.close();
    }
    catch (Exception e) {
      Utils.oops(null,"Something's wrong with file '" + getName() + "'!");
      return false;
    }

    if (items.size() > 0) {
      if (items.size() == newItems.size()) {
	res = true;
	Enumeration e1 = items.elements();
	Enumeration e2 = newItems.elements();
	while (e1.hasMoreElements()) {
	  if (!((String)e1.nextElement()).equals((String)e2.nextElement())) {
	    res = false;
	    break;
	  }
	}
      }
      else res = false;
    }
    items = newItems;
    if (!res) {
      if (columns.size() > 0) columns.removeAllElements();
      if (styles.size() > 0) styles.removeAllElements();
    }
    if (items.size() == 2) gp.setDrawLegend(false);

    // if labels read, delete all labels which were marked 
    // 'CHECK' and not found in the current dataset:
    if (labels.size() > 0) labels.removeAllElements();
    if (newLabels.size() > 0) {
      for (Enumeration e=newLabels.elements(); e.hasMoreElements();) {
	GraphLabel g = (GraphLabel) e.nextElement();
	if (!g.equals(GraphLabel.CHECK)) labels.add(g);
      }
      newLabels.removeAllElements();
      newLabels = null;
    }
    return res;
  }  

  private String getLabel(StringTokenizer st) {
    String token = st.nextToken();
    if (token.startsWith("\"")) {  // is a label between " and "
      while (!token.endsWith("\"")) token += " " + st.nextToken();
      token = token.substring(1,token.length()-1);
    }
    return token;
  }

  /*
   * Add a label to the vector of labels.
   * The rule is, for the moment: if the user reloads the
   * datafile (and hence passes through this procedure), the
   * label is declared. But if the label already exists (with,
   * perhaps, another color, font or position), than we keep
   * this label instead BUT we use the position as defined 
   * here in the data set.
   */
  private void addLabel(GraphLabel gl, Vector newLabels) {
    boolean labelFound = false;
    for (Enumeration e=newLabels.elements(); e.hasMoreElements();) {
      GraphLabel g = (GraphLabel) e.nextElement();
      if (gl.equals(g.getText()) && (g.equals(GraphLabel.CHECK))) {
	g.setUsePosition(false);
	g.setDataLocation(gl.getXPos(),gl.getYPos());
	g.setID(gl.getID());
	labelFound = true;
	break;
      }
    }
    if (!labelFound) newLabels.add(gl);
  }


  /**
   * @return the number of items present in the item vector.
   */
  public int getNumberOfItems() {
    return items.size();
  }

  /**
   * @return the item corresponding to index i.
   */
  public String getItem(int i) {
    return (String) items.get(i);
  }

  /**
   * Adds an item to the vector.
   * @param item name of the item added.
   */
  public void addItem(String item) {
    items.add(item);
  }

  /*
   * remove a column at a specified index:
   */
  private void delCol(Integer index) {
    int k=0;
    for (Enumeration e=columns.elements(); e.hasMoreElements(); k++) {
      Integer i = (Integer) e.nextElement();
      if (i.equals(index)) break;
    }
    if (k < columns.size()) {
      columns.remove(k);
      styles.remove(k);
    }
  }
  
  /**
   * adds a column to the vector of columns.
   * The column is added <strong>even</strong> if the column
   * index is already selected (hence selecting several times the
   * same column is allowed). This is useful if the user wants
   * to plot the same data several times but in a different way.
   * (histogram + lines, for example).
   * @param columnIndex index of the column
   * @param lp parameters used to draw the line
   */
  public void addColumn(int columnIndex, LinePars lp) {
    Integer index = new Integer(columnIndex);
    columns.add(index);
    styles.add(lp);
  }

  /**
   * sets the index of the column used for the X-axis
   * @param columnIndex index of the column
   */
  public void setXColumn(int columnIndex) {
    xColumn = columnIndex;
  }

  /**
   * returns the index of the column used for the X-axis
   * @return index of the column
   */
  public int getXColumn() {
    return xColumn;
  }

  /**
   * returns the index of the column used for the Y-axis
   * @param i index of the column item
   * @return index of the column
   */
  public int getYColumn(int i) {
    return ((Integer)columns.get(i)).intValue();
  }

  /**
   * returns the linestyle of one of the vector items.
   * @param index index of the column
   */
  public LinePars getLinePars(int index) {
    return (LinePars)styles.get(index);
  }

  /**
   * sets the linestyle of one of the vector items.
   * @param index index of the column
   * @param lp linepars instance (can't be null!)
   */
  public void setLinePars(int index, LinePars lp) {
    styles.set(index,lp);
  }

  /**
   * sets the linestyle of all the vector items.
   * @param lp linepars instance
   */
  public void setLinePars(LinePars lp) {
    if (styles.size() > 0) {
      styles.removeAllElements();
    }
    for (int k=0; k<columns.size(); k++) {
      lp = new LinePars(lp);
      if (lp.slideColor) lp.nextColor(columns.size());
      styles.add(lp);
    }
  }

  public String getLegend(int index) {
    return getLinePars(index).getName();
  }

  /**
   * remove a column from the vector of columns.
   * @param columnIndex index of the column
   */
  public void removeColumn(int columnIndex) {
    if (columnIndex < columns.size()) {
      columns.remove(columnIndex);
      styles.remove(columnIndex);
    }
  }

  /**
   * Return the number of columns (selected items)
   * @return number of Y-columns
   */
  public int getNumberOfColumns() {
    return columns.size();
  }

  /**
   * Return the file
   * @return file
   */
  public File getFile() {
    return file;
  }

  /**
   * Return the abbreviated filename.
   * This name is different from the file itselves if it exceeds a
   * a certain number of characters (about 45). Otherwise, the name
   * is too long to fit in decent textboxes...
   * @return an abbreviated version of the filename, if the latter is too long.
   */
  public String getAbbreviatedFilename() {
    return abbreviatedFilename;
  }

  /**
   * Return the filename with the preceeding path stripped off.
   * @return an abbreviated version of the filename, without an eventual path.
   */
  public String getFilenameWithoutPath() {
    return filenameWithoutPath;
  }

  /**
   * Return the filename as such.
   * @return the filename.
   */
  public String getName() {
    return file.toString();
  }

  /**
   * Checks whether the argument is a column selected for output.
   * @param index index which is sought for.
   */
  public int isSelected(int index) {
    int k = 0;
    for (Enumeration e=columns.elements(); e.hasMoreElements(); k++) {
      if (((Integer)e.nextElement()).intValue() == index) {
	//System.out.print("... " + index + " is found ...");
	return k;
      }
    }
    return -1;
  }

  /**
   * Returns the name of one of the columns present here.
   * @return name of column with index i
   */
  public String getColumnName(int i) {
    int index = ((Integer)columns.get(i)).intValue();
    return (String)items.get(index);
  }

  /**
   * Sets the current datafile index to another value.
   * @param i new index
   */
  public void setIndex(int i) {
    index = i;
  }

  /**
   * @return the graphstyle of the current dataset
   */
  public int getGraphType() {
    return graphType;
  }

  /**
   * Reads the current file and returns the data arrays.
   * @param data arrays of X,Y data
   * @return false if some weird error might occur during file reading.
   */
  public boolean getDataArrays(Vector dataArrays, boolean isPiper) {
    int nCol = (isPiper)? getNumberOfItems() : getNumberOfColumns();
    double x=0.0;
    double[] y = new double [nCol];
    boolean newDataSet = true;
    boolean isFirstLabel = true;
    String s;

    try {
      BufferedReader in = new BufferedReader(new FileReader(getFile()));
      DataArray[] data = new DataArray [nCol];
      int k=0;
      while ((s=in.readLine()) != null) {
	s = s.trim();
	if (s.length() == 0) {
	  newDataSet = true;
	  continue;
	}
	else if (s.startsWith("#")) continue;
	StringTokenizer st = new StringTokenizer(s," \t,");
	if (st.countTokens() < 2) continue;

	// start new dataset here
	if (!isPiper) {
	  if (newDataSet) {
	    if (JPlot.debug) System.out.println("starting a new dataset with " + nCol + " columns...");
	    for (int i=0; i<nCol; i++) {
	      data[i] = new DataArray(101,getLinePars(i));
	    }
	    newDataSet = false;
	  }
	  boolean liftPen = false;
	  for (int i=0; st.hasMoreTokens(); i++) {
	    String t = st.nextToken();
	    if (t.equals("*")) liftPen = true;
	    if (i == getXColumn()) x = Double.parseDouble(t);
	    for (k=0; k<nCol; k++) {
	      if (getYColumn(k) == i) y[k] = Double.parseDouble(t);
	    }
	  }
	  for (int i=0; i<nCol; i++) data[i].addPoint(x,y[i],liftPen);
	}
	else {
	  //System.out.println("So far, so fine for s = " + s);
	  data[0] = new DataArray(3,getLinePars(k++));
	  if (st.countTokens() < 4) {
	    Utils.oops(null,"Incorrect format of Piper diagram datafile '" + getAbbreviatedFilename() + "'!");
	    return false;
	  }
	  else {
	    data[0].addPoint(Double.parseDouble(st.nextToken()),
			     Double.parseDouble(st.nextToken()));
	    data[0].addPoint(Double.parseDouble(st.nextToken()),
			     Double.parseDouble(st.nextToken()));
	    
	    // if there's one more point, it must be the TDS:
	    if (st.hasMoreTokens()) {
	      data[0].addPoint(Double.parseDouble(st.nextToken()),0.0);
	    }
	    dataArrays.add(data[0]);
	  }
	}
      }
      if (!isPiper) for (int i=0; i<nCol; i++) dataArrays.add(data[i]);
      in.close();
    }
    catch (Exception exception) {
      Utils.oops(null,"Something's wrong with file '" + 
		 getAbbreviatedFilename() + 
		 "'.\nCheck the file's format and its content.");
      return false;
    }
    return true;
  }
}
