/*
 * JPLOT  -- Java Plotting Interface for any programme or
 *           as an independent GUI
 * 
 * Copyright (C) 1999 Jan van der Lee
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA.
 *
 * Send bugs, suggestions or queries to <jplot@cig.ensmp.fr>
 * The latest releases are found at 
 * http://www.cig.ensmp.fr/~vanderlee/jplot.html
 *
 * Initally developed for use by the Centre d'Informatique Geologique 
 * Ecole des Mines de Paris, Fontainebleau, France.
 */

package jplot;

import javax.swing.*;
import java.awt.*;
import java.awt.image.*;
import java.text.*;
import java.math.*;
import java.awt.geom.*;
import java.awt.event.*;
import javax.swing.border.*;
import java.util.*;
import java.awt.print.*;

/**
 * The <code>Graph</code> class builds a panel which displays a
 * graph according to the data stored in the {@link DataArray}.
 * Extends JPanel since the graph is returned in a panel (it can
 * be used in other applications or embedded in a frame. Implements
 * Printable, so we can print the graph, and ComponentListener, needed
 * to listen to a resize event.
 *
 * This class should be subclassed in order to build a specific
 * graph type (i.e. a 2D graph, piper diagram or 3D, later)
 *
 * @version 24/08/99
 * @author  J. van der Lee
 */
public abstract class Graph extends JPanel implements Printable {

  // vector of DataArray objects, containing
  // the arrays of datapoints forming the graph:
  Vector data;

  // axes and tic-label stuff:
  final protected int X = GraphPars.X_AXIS;
  final protected int Y = GraphPars.Y_AXIS;
  final protected int Na = GraphPars.NAXES;
  protected double[] axisLength = new double [Na];
  protected double leftMargin;
  protected double rightMargin;
  protected double bottomMargin;
  protected double topMargin;

  // variables used to calculate tic-seperation distances:
  protected String[][] ticLabel = new String[Na][];
  protected double[] ticLength = new double [Na];
  protected double[] diff = new double [Na];
  protected double[] inv = new double [Na];
  protected int[] numberOfTics = new int [Na];

  // separation between tic-labels and the axis:
  protected double sep[] = new double [Na];

  // legend stuff:
  protected boolean legendActive = false;
  protected boolean somethingActive = false;
  protected final float xSep=5.0f;
  protected final float ySep=3.0f;

  // this class contains all the parameters
  // which are used to plot the graph.
  protected GraphPars gp;

  // legend:
  protected double ldy;    // distance between two legend labels
  protected double keyLen; // length of the key (drawing style, e.g. a line)
  protected double legendWidth;
  protected double legendHeight;

  // size of the graph panel
  protected Dimension panelSize;
  protected Dimension panelSizeForPrinter;

  // dimension of the panel:
  protected int width;
  protected int height;

  // colors of the label drag'n drop box:
  private final Color boxColor = Color.gray;

  // the value of Ln(10), need it a few times for logscales:
  private final static double LNTEN = Math.log(10.0);

  protected Line2D.Double line = new Line2D.Double();
  protected Rectangle2D.Double rect = new Rectangle2D.Double();

  // coordinates used for dragging and zooming.
  protected int x1, y1, boxWidth, boxHeight;
  protected int xStart, yStart;

  protected BufferedImage bi;
  private boolean initializing;

  // variables used by a piper diagram
  protected double piperSep;           // separation gap between rectangles
  protected double normalSep;        // (vertical) normal of the gap
  protected double triangleBottom;   // length of bottom of a triangle
  protected double triangleSide;     // length of the sides of a triangle
  protected double triangleHeight;   // height of a triangle
  protected double borderWidth;      // white space between panel and border
  protected double arc;              // angle of the triangle (lower corners)
  protected double tan, sin, cos;    // geometrical coefficients

  private JPlot jplot;

  /**
   * Main constructor.
   * @param gp objects containing the parameters used to draw this graph.
   * @param d dimension of the current graph.
   */
  public Graph(JPlot jp, GraphPars gp) {
    jplot = jp;
    if (JPlot.debug) System.out.print("Initializing Graph(gp)...");
    initializing = true;
    setLayout(new FlowLayout(FlowLayout.RIGHT));
    setBorder(new BevelBorder(BevelBorder.LOWERED));
    addMouseListener(new GraphMouseListener());
    addMouseMotionListener(new DragListener());
    this.gp = gp;
    height = gp.getPanelHeight();
    width = gp.getPanelWidth();
    leftMargin = gp.getLeftMargin();
    rightMargin = gp.getRightMargin();
    bottomMargin = gp.getBottomMargin();
    topMargin = gp.getTopMargin();
    panelSize = gp.getDimension();
    keyLen = 25.0f;
    for (int k=0; k<Na; k++) {
      ticLabel[k] = new String [gp.getMaxNumberOfTics()];
    }
    bi = new BufferedImage(width,height,BufferedImage.TYPE_INT_RGB);    
    if (JPlot.debug) System.out.println("done");
  }
  
  /** 
   * Get the minimum size of this component.
   * This is the panel size defined by width and height.
   * @return the minimum size.
   */
  public Dimension getMinimumSize() {
    return new Dimension(width,height);
  }  
  
  /** 
   * Get the preferred size of this component.
   * This is the panel size defined by width and height
   * @return the preferred size.
   */
  public Dimension getPreferredSize() {
    return new Dimension(width,height);
  }
  
  /**
   * Sets the new location of the top-left corner (x,y) and the size
   * in terms of the width (w) and height (h).
   * @param x the new x-coordinate.
   * @param y the new y-coordinate.
   * @param w the new width.
   * @param h the new height.
   */
  /*
  public void setBounds(int x, int y, int w, int h) {
    width = w;
    height = h;
    super.setBounds(x,y,width,height);
    //updateGraph();
    //System.out.println("@@@@@@@@@@@@@ resized");
  }
  */

  /** 
   * Set the size of the plot.
   * @param w the width, in pixels.
   * @param h the height, in pixels.
   */
  /*
  public void setSize(int w, int h) {
    width = w;
    height = h;
    super.setSize(width,height);
  }
  */

  /*
   * This method is called as the user is zooming and releases the
   * mousebutton. It rescales the plot according to the selected
   * box boundaries.
   * @param x The final x position.
   * @param y The final y position.
   */
  private synchronized void finishZoom(int x1, int y1, int x2, int y2) {
    //Graphics g = getGraphics();
  }

  /*
   * @param x x position.
   * @param y y position.
   */
  private synchronized void zoomRect(int x, int y) {
    Graphics g = getGraphics();
  }

  /**
   * Returns the log10 of a double value.
   * @param x value 
   * @return log10(x)
   */
  public static double log10(double x) {
    return Math.log(x)/LNTEN;
  }

  /**
   * Calculation of the separation length between tics.
   * Some smart rounding algorithms are needed to get the
   * scaling properly in case of data going to 10.3 or so...
   * Usefull stuff stolen from the Gnuplot sources, thank you guys!!
   */
  protected double calculateTicSep(double min, double max)
  {
    double xnorm, tic, posns;
    double lrange = log10(Math.abs(min-max));
    double fl = Math.floor(lrange);
    xnorm = Math.pow(10.0,lrange-fl);
    posns = gp.getMaxNumberOfTics()/xnorm;

    if (posns > 40) tic = 0.05;       // eg 0, .05, .10, ...
    else if (posns > 20) tic = 0.1;   // eg 0, .1, .2, ...
    else if (posns > 10) tic = 0.2;   // eg 0,0.2,0.4,...
    else if (posns > 4) tic = 0.5;    // 0,0.5,1,
    else if (posns > 1) tic = 1;      // 0,1,2,....
    else if (posns > 0.5) tic = 2;    // 0, 2, 4, 6
    else tic = Math.ceil(xnorm);
    tic *= Math.pow(10.0,fl);
    /*
    if (JPlot.debug) {
      System.out.print("calculated tic-length = " + tic);
      System.out.println(" for min = " + min + ", max = " + max);
    }
    */
    return tic;
  }                

  /**
   * Return the number of digits required to display the
   * given number.  If more than 15 digits are required, 
   * 15 is returned). 
   */
  public static int getNumDigits(double num) {
    int numDigits = 0;
    if (num == 0.0) return 0;
    while (numDigits <= 15 && Math.abs(Math.floor(num)/num-1.0) > 1e-10) {
    //while (numDigits <= 15 && num != Math.floor(num)) {
      num *= 10.0;
      numDigits++;
    }
    return numDigits;
  }

  /**
   * formats a double precision number such that it is
   * correctly rounded for output. Kind of pre-processor for
   * all number for output.
   * @param num number to be formatted
   * @param n number of digits (accuracy) after the decimal point.
   * @return the formatted number in a string. 
   */ 
  public static String formatNumber(double num, int n) {
    int maxFloat = 6;
    if (num != 0.0) {
      int exp = (int)Math.floor(log10(num));
      int x = Math.abs(exp) + n;
      BigDecimal bd = new BigDecimal(num);
      if (x > maxFloat) {
	if (exp > 0) bd = bd.movePointLeft(exp);
	else bd = bd.movePointRight(Math.abs(exp));
      }
      else {
	if (exp < 0) n = x; 
	else n = x - Math.abs(exp);
      }
      bd = bd.setScale(n,BigDecimal.ROUND_HALF_EVEN);
      int nn = getNumDigits(bd.doubleValue());
      if (nn < n) bd = bd.setScale(nn);
      String res = bd.toString();
      if (x > maxFloat) res += "e" + exp;
      return res;
    }
    else return "0";
  }

  /**
   * Determine the axis labels, if needed. These are numbers
   * which are translated in Strings. The length is evalulated
   * and used to set the left and bottom margins.
   */
  abstract void makeTicLabels();

  /**
   * This method updates the margins as a function of the label 
   * widths and the axes texts. Also sets the default positions
   * of the text labels.
   */
  protected void updateMargins() {
    double xlHeight = 0.0;
    double ylWidth = 0.0;
    double tHeight = 0.0;

    if (JPlot.debug) System.out.print("updating margins...");

    // we have to find the height of the X- and Y labels and title
    // in order to get the margins right:
    for (Enumeration e=(gp.getLabels()).elements(); e.hasMoreElements();) {
      GraphLabel gl = (GraphLabel) e.nextElement();
      if (gl.equals(GraphLabel.XLABEL)) xlHeight = gl.getHeight()+ySep;
      else if (gl.equals(GraphLabel.YLABEL)) ylWidth = gl.getWidth()+xSep;
      else if (gl.equals(GraphLabel.TITLE)) tHeight = gl.getHeight()+ySep;
      if (gl.equals(GraphLabel.PIPER_X1)) xlHeight = gl.getHeight()-3.0;
    }

    double maxYLabel = 0.0;

    // initialize the margins with the default (minimum) values:
    borderWidth = gp.getBoxOffset();
    leftMargin = gp.getLeftMargin() + borderWidth;
    bottomMargin = gp.getBottomMargin() + xlHeight + borderWidth;
    rightMargin = gp.getRightMargin() + borderWidth;
    topMargin = gp.getTopMargin() + tHeight + borderWidth;

    // add some extra space to the bottom-margin for the tic-labels:
    if (gp.drawTicLabels(X)) bottomMargin += ySep*2;

    // add some extra space to the left-margin in order to account
    // for the y-labels (take the label with the maximum length):
    if (gp.drawTicLabels(Y)) {
      FontMetrics fm = getFontMetrics(gp.getTicFont(Y));
      for (int i=0; i<numberOfTics[Y]; i++) {
	double yl = fm.stringWidth(ticLabel[Y][i]);
	if (yl > maxYLabel) maxYLabel = yl;
	yl += xSep*2 + borderWidth;
	if (yl > leftMargin) leftMargin = yl;
      }
    }
    leftMargin += ylWidth;

    // axes lengths must be something like this:
    axisLength[X] = width - leftMargin - rightMargin;
    axisLength[Y] = height - topMargin - bottomMargin;

    // ... except if someone has fixed the axes ratio:
    double r = axisLength[Y]/axisLength[X];
    if (gp.useAxesRatio()) {
      
      // at this stage, both axes are at maximum length. Let 'ratio' 
      // be the ratio specified by the user. If r > ratio, then we
      // must adapt the Y-axis-length:
      if (r > gp.getAxesRatio()) {
	axisLength[Y] = gp.getAxesRatio()*axisLength[X];
	topMargin = height - bottomMargin - axisLength[Y];
      }
      else if (r < gp.getAxesRatio()) {
	axisLength[X] = gp.getAxesRatio()*axisLength[Y];
	rightMargin = width - leftMargin - axisLength[X];
      }
    }
    else gp.setAxesRatio(r);
    //gp.setAxisLength(X,axisLength[X]);
    //gp.setAxisLength(Y,axisLength[Y]);


    if (gp.getGraphType() == gp.GRAPHTYPE_PIPER) {
      if (!gp.useLegendPos()) {
	gp.setLegendPos(borderWidth,borderWidth+3.0f);
      }

      // needed to convert data scale to graphical pixel scaling
      tan = 2.0*axisLength[Y]/axisLength[X];
      arc = Math.atan(tan);
      sin = Math.sin(arc);
      cos = Math.cos(arc);
      normalSep = piperSep*sin;
      
      triangleBottom = (axisLength[X]-piperSep)/2.0;
      triangleHeight = (axisLength[Y]-normalSep)/2.0;
      triangleSide   = triangleHeight/sin;
      sep[X] = triangleBottom/(numberOfTics[X]-1);
      sep[Y] = triangleHeight/(numberOfTics[Y]-1);
      if (tHeight > 0.0) tHeight += 10.0;
    }
    else {
      if (!gp.useLegendPos()) {
	gp.setLegendPos(leftMargin + ticLength[Y] + 7.0f,
			topMargin + 7.0f);
      }
      sep[X] = axisLength[X]/(numberOfTics[X]-1);
      sep[Y] = axisLength[Y]/(numberOfTics[Y]-1);
    }
    ticLength[X] = gp.getTicLength(X)*width;
    ticLength[Y] = gp.getTicLength(Y)*width;
    setDefaultLabelPositions(maxYLabel);
    if (JPlot.debug) System.out.println("done.");
  }

  /*
   * Set the label positions to default values.
   */
  private void setDefaultLabelPositions(double m) {
    int numOfOtherLabels=0;
    if (gp.getGraphType() == gp.GRAPHTYPE_PIPER) m *= 0.7;
    for (Enumeration e=gp.getLabels().elements(); e.hasMoreElements();) {
      GraphLabel gl = (GraphLabel) e.nextElement();
      if (!gl.usePosition()) {
	double x=0.0;
	double y=0.0;
	if (gl.equals(GraphLabel.XLABEL)) {
	  x = leftMargin+axisLength[X]/2;
	  y = topMargin+axisLength[Y]+(bottomMargin-borderWidth)/2+ySep;
	}
	else if (gl.equals(GraphLabel.YLABEL)) {
	  x = borderWidth + (leftMargin-borderWidth-m)/2;
	  y = topMargin+axisLength[Y]/2;
	}
	else if (gl.equals(GraphLabel.TITLE)) {
	  x = leftMargin+axisLength[X]/2;
	  y = topMargin/2;
	}
	else if (gl.equals(GraphLabel.OTHER)) {
	  x = leftMargin + 10.0f + gl.getWidth()/2;
	  y = topMargin + axisLength[Y] - 14.0f*(1+numOfOtherLabels);
	  numOfOtherLabels++;
	}
	else if (gl.equals(GraphLabel.PIPER_X1)) {
	  x = leftMargin+triangleBottom/2;
	  y = topMargin+axisLength[Y]+(bottomMargin-borderWidth)/2+ySep;
	}
	else if (gl.equals(GraphLabel.PIPER_X2)) {
	  x = leftMargin+axisLength[X]-triangleBottom/2;
	  y = topMargin+axisLength[Y]+(bottomMargin-borderWidth)/2+ySep;
	}
	else if (gl.equals(GraphLabel.PIPER_Y1)) {
	  gl.setRotation(-arc);
	  double xx = m+sin*(xSep+gl.getTextHeight());
	  x = leftMargin+triangleBottom/4-xx;
	  y = topMargin+axisLength[Y]-triangleHeight/2-xx/tan;
	}
	else if (gl.equals(GraphLabel.PIPER_Y2)) {
	  gl.setRotation(arc);
	  double xx = m+cos*(xSep+gl.getTextHeight());
	  x = leftMargin+axisLength[X]-triangleBottom/4+xx;
	  y = topMargin+axisLength[Y]-triangleHeight/2-xx/tan;
	}
	else if (gl.equals(GraphLabel.PIPER_Y3)) {
	  gl.setRotation(-arc);
	  double xx = m+sin*(xSep+gl.getTextHeight());
	  x = leftMargin+axisLength[X]/2-triangleBottom/4-xx;
	  y = topMargin+triangleHeight/2-xx/tan;
	}
	else if (gl.equals(GraphLabel.PIPER_Y4)) {
	  gl.setRotation(arc);
	  double xx = m+cos*(xSep+gl.getTextHeight());
	  x = leftMargin+axisLength[X]/2+triangleBottom/4+xx;
	  y = topMargin+triangleHeight/2-xx/tan;
	}
	else if (gl.equals(GraphLabel.DATA)) {
	  x = toX(gl.getXPos());
	  y = toY(gl.getYPos());
	  gl.setUsePosition(true);
	  //gl.setID(GraphLabel.OTHER);
	}
	gl.setLocation(x-gl.getWidth()/2,y-gl.getHeight()/2);
	gl.setRotation(gl.getRotation());
      }
    }
  }

  /**
   * Find the minimum and maximum values for X- and Y axes.
   * @param axis axis for which the min and max are needed.
   * @param vector array containing the data set.
   */
  abstract boolean setMinMax(int axis, Vector data);
  
  /*
   * Plot the labels, ALL the labels on the graph.
   * @param g2 a Graphics2D instance (canvas).
   */
  protected void plotLabels(Graphics2D g2) {
    for (Enumeration e=gp.getLabels().elements(); e.hasMoreElements();) {
      GraphLabel gl = (GraphLabel) e.nextElement();
      if (!gl.getText().equals("")) {
	g2.setColor(gl.getColor());
	float x = (float)gl.getXPos();
	float y = (float)gl.getYPos();
	g2.rotate(gl.getRotation(),x,y);
	g2.drawString(gl.getCharIterator(),x,y);
	g2.rotate(-gl.getRotation(),x,y);
      }
    }
  }

  /*
   * Finds the height and width of the box which envelops the legend.
   * This box is used for dragging the legend.
   * @param data vector with data arrays.
   */
  protected void getLegendBox(Vector data) {
    String s = "";
    int k=0;
    for (Enumeration e=data.elements(); e.hasMoreElements();) {
      DataArray da = (DataArray)e.nextElement();
      if (da.drawLegend()) {
	if (da.getName().length() > s.length()) s = da.getName();
	k++;
      }
    }
    FontMetrics fm = getFontMetrics(gp.getLegendFont());
    legendWidth = fm.stringWidth(s) + keyLen + xSep;
    ldy = fm.getHeight()*gp.getLegendSpacing();
    legendHeight = ldy*(k-1) + fm.getHeight() + 2;
  }

  /*
   * Writes the legend in the upper-left corner of the graph, or
   * whereever the user has pinned it. Note that the legend is drawn
   * one 'ldy' (separation in the y-direction) lower than expected,
   * this is a poor-man's trick to get the bounding-box right as we
   * start to drag'n drop.
   */
  protected void drawLegend(Graphics2D g2, DataArray da, int i) {
    float y = (float)(gp.getLegendPos(Y) + ldy*(i+1));
    float x = (float)(gp.getLegendPos(X));
    float yy = y-(float)ldy/4;
    if (da.drawLine()) {
      line.setLine(x,yy,x+keyLen,yy);
      g2.draw(line);
    }
    if (da.drawSymbol()) {
      drawPointType(da.getSymbol(),g2,x+keyLen/2,yy,da.getSymbolSize());
    }
    g2.drawString(da.getName(),(float)(x+keyLen+xSep),y);
  }

  /**
   * Returns the X-value scaled to the pixel-availability.
   * This function takes the X-value and returns the corresponding
   * coordinates for the panel.
   * @param x real x-value (as introduced by the data)
   * @return x-value scaled to the actual panel coordinates
   */
  abstract double toX(double x);

  /**
   * Returns the Y-value scaled to the pixel-availability.
   * This function takes the Y-value and returns the corresponding
   * coordinates for the panel.
   * @param x real x-value (as introduced by the data)
   * @return x-value scaled to the actual panel coordinates
   */
  abstract double toY(double x);

  /**
   * Fills the graph area with a background color.
   * @param g2 Graphics canvas
   */
  abstract void fillGraphArea(Graphics2D g2);

  /**
   * Draws the graph on the graphics canvas of this panel.
   * Performs a test on the minimum and maximum values. If
   * they are different by more than 10 % with respect to their
   * previous values, release all fixed positions of legend, labels
   * etc and restore the default positions. Otherwise, they may fall
   * far beyond the screen, leading to a coredump.
   * @param data vector of Data Array vectors with the data points
   */
  protected void show(Vector data) {
    this.data = data;
    if (data.size() < 1) return;
    if (gp.drawLegend()) getLegendBox(data);

    // find the minimum and maximum values of the current datasets:
    for (int k=0; k<Na; k++) {
      if (!setMinMax(k,data)) {
	Utils.bummer(null,"Invalid data, no way to show a decent graph");
	return;
      }
    }
    makeTicLabels();
    updateMargins();
    updateGraph();
  }

  public void resetLabelPositions(double w0, double h0, 
				  double w1, double h1) {
    if (gp.useLegendPos()) {
      gp.setLegendPos(w1*gp.getLegendPos(gp.X_AXIS)/w0,
		      h1*gp.getLegendPos(gp.Y_AXIS)/h0);
    }
    for (Enumeration e=gp.getLabels().elements(); e.hasMoreElements();) {
      GraphLabel gl = (GraphLabel) e.nextElement();
      if (gl.usePosition()) gl.setLocation(w1*gl.getX()/w0,h1*gl.getY()/h0);
      if (gl.equals(GraphLabel.PIPER_Y1)) gl.setRotation(-arc);
      else if (gl.equals(GraphLabel.PIPER_Y2)) gl.setRotation(arc);
    }
  }

  /**
   * Creates the graph and returns the graph in a buffer.
   * The graph is drawn using the specified dimension. This class
   * stores the current widht/height of the graph and changes the
   * margins (computation-intensive) only if the size is actually
   * different from the current settings.
   *
   * @param size size of the image
   * @return image in an image buffer, ready to be drawn.
   */
  public BufferedImage getBufferedImage(Dimension size) {
    if (width != size.getWidth() || height != size.getHeight()) {
      double w=width;
      double h=height;
      width = size.width;
      height = size.height;
      bi = new BufferedImage(width,height,BufferedImage.TYPE_INT_RGB);
      updateMargins();
      resetLabelPositions(w,h,size.width,size.height);
      updateGraph();
    }
    return bi;
  }

  /**
   * Overloaded paintComponent, is called at each repaint.
   * If the panel size changed then the user probably
   * resized the window with the mouse and we have to reset the
   * margins of the graph.
   * @param g graphics context
   */
  public void paintComponent(Graphics g) {
    super.paintComponent(g);
    Graphics2D g2 = (Graphics2D) g;
    panelSize = getSize(panelSize);
    g2.drawImage(getBufferedImage(panelSize),0,0,this);
  }

  /**
   * This function builds the graph in a double-buffered image zone.
   */
  protected Graphics2D createGraphics() {
    Graphics2D g2  = bi.createGraphics();
    g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, 
			RenderingHints.VALUE_ANTIALIAS_ON);
    g2.setColor(gp.getBackgroundColor());
    g2.fill(new Rectangle(0,0,width,height));
    if (gp.showBox()) {
      rect.setRect(borderWidth,borderWidth,
		   width-2*borderWidth,height-2*borderWidth);
      g2.setColor(gp.getBoxFillColor());
      g2.fill(rect);
      g2.setColor(gp.getBoxColor());
      g2.draw(rect);
    }
    fillGraphArea(g2);
    return g2;
  }

  /**
   * This function builds the graph in a double-buffered image zone.
   */
  abstract void updateGraph();

  /**
   * Draws a point-type corresponding to a symbolic constant,
   * yielding values from 0 to gp.NO_SYMBOL (==13).
   * @param index symbolic constant
   * @param g2 graphics 2D instance
   * @param x x-coordinate of the midpoint of the point
   * @param y y-coordinate of the midpoint of the point
   * @param ps size of the point in pixels
   */
  static void drawPointType(int index, Graphics2D g2, double _x, double _y, float ps) {
    if (index > 12 || ps <= 0.0f) return;
    float x = (float)(_x)-ps/2.0f;
    float y = (float)(_y)-ps/2.0f;
    if (index == 0) g2.draw(new Ellipse2D.Float(x,y,ps,ps));
    else if (index == 1) g2.draw(new Rectangle2D.Float(x,y,ps,ps));
    else if (index == 2) {
      GeneralPath gp = new GeneralPath();
      gp.moveTo(x,y+ps/2.0f);
      gp.lineTo(x+ps/2.0f,y+ps);
      gp.lineTo(x+ps,y+ps/2.0f);
      gp.lineTo(x+ps/2.0f,y);
      gp.lineTo(x,y+ps/2.0f);
      g2.draw(gp);
    }
    else if (index == 3) {
      GeneralPath gp = new GeneralPath();
      gp.moveTo(x,y+ps);
      gp.lineTo(x+ps,y+ps);
      gp.lineTo(x+ps/2.0f,y);
      gp.lineTo(x,y+ps);
      g2.draw(gp);
    }
    if (index == 4) {
      g2.draw(new Ellipse2D.Float(x,y,ps,ps));
      g2.fill(new Ellipse2D.Float(x,y,ps,ps));
    }
    else if (index == 5) {
      g2.draw(new Rectangle2D.Float(x,y,ps,ps));
      g2.fill(new Rectangle2D.Float(x,y,ps,ps));
    }
    else if (index == 6) {
      GeneralPath gp = new GeneralPath();
      gp.moveTo(x,y+ps/2.0f);
      gp.lineTo(x+ps/2.0f,y+ps);
      gp.lineTo(x+ps,y+ps/2.0f);
      gp.lineTo(x+ps/2.0f,y);
      gp.lineTo(x,y+ps/2.0f);
      g2.draw(gp);
      g2.fill(gp);
    }
    else if (index == 7) {
      GeneralPath gp = new GeneralPath();
      gp.moveTo(x,y+ps);
      gp.lineTo(x+ps,y+ps);
      gp.lineTo(x+ps/2.0f,y);
      gp.lineTo(x,y+ps);
      g2.draw(gp);
      g2.fill(gp);
    }
    else if (index == 8) {
      g2.draw(new Line2D.Float(x,y+ps/2.0f,x+ps,y+ps/2.0f));
      g2.draw(new Line2D.Float(x+ps/2.0f,y,x+ps/2.0f,y+ps));
    }
    else if (index == 9) {
      g2.draw(new Line2D.Float(x,y,x+ps,y+ps));
      g2.draw(new Line2D.Float(x+ps,y,x,y+ps));
    }
    else if (index == 10) {
      g2.draw(new Line2D.Float(x,y+ps/2.0f,x+ps,y+ps/2.0f));
      g2.draw(new Line2D.Float(x+ps/2.0f,y,x+ps/2.0f,y+ps));
      g2.draw(new Line2D.Float(x,y,x+ps,y+ps));
      g2.draw(new Line2D.Float(x+ps,y,x,y+ps));
    }
    else if (index == 11) {
      g2.draw(new Line2D.Float(x+ps/2.0f,y+ps/2.0f,x+ps/2.0f,y+ps/2.0f));
    }
    else if (index == 12) {
      g2.draw(new Line2D.Float(x,y+ps/3.0f,x+ps,y+ps/3.0f));
      g2.draw(new Line2D.Float(x,y+2.0f*ps/3.0f,x+ps,y+2.0f*ps/3.0f));
      g2.draw(new Line2D.Float(x+ps/3.0f,y,x+ps/3.0f,y+ps));
      g2.draw(new Line2D.Float(x+2.0f*ps/3.0f,y,x+2.0f*ps/3.0f,y+ps));
    }
  }

  public int print(Graphics g, PageFormat pageFormat, int pageIndex) 
    throws PrinterException {
    g.translate((int)pageFormat.getImageableX(),
		(int)pageFormat.getImageableY());
    if (pageIndex >= 1) return Printable.NO_SUCH_PAGE;
    //int wPage = (int)pageFormat.getImageableWidth();
    //int hPage = (int)pageFormat.getImageableHeight();
    //paintComponent(g);
    Color bgColor = gp.getBackgroundColor();
    gp.setBackgroundColor(Color.white);
    width += 1;  // trick to rebuild the bufferedImage
    paintComponent(g);
    gp.setBackgroundColor(bgColor);
    System.gc();
    return PAGE_EXISTS;
  }

  public class GraphMouseListener implements MouseListener {

    public void mouseClicked(MouseEvent e) {
      double xx,yy;
      if (e.getClickCount() == 2) {
	//int a = (int)JPlot.plotFrame.getLocation().getX() + 20;
	//int b = (int)JPlot.plotFrame.getLocation().getY() + 20;
	int a = e.getX() + 20;
	int b = e.getY() + 20;
	// show the legend parameter dialog if the legend is hit:
	xx = gp.getLegendPos(X);
	yy = gp.getLegendPos(Y);
	if (gp.drawLegend() && xStart > xx && xStart < xx+legendWidth &&
	    yStart > yy && yStart < yy+legendHeight) {
	  jplot.legendPanel.show(jplot.frame,a,b);
	}
	else {
	  // show the label parameters dialog if a label is hit:
	  for (Enumeration el=gp.getLabels().elements(); 
	       el.hasMoreElements();) {
	    GraphLabel gl = (GraphLabel) el.nextElement();
	    if (gl.contains(xStart,yStart)) {
	      jplot.labelPanel.show(jplot.frame,a,b);
	      break;
	    }
	  }
	}
      }
    }
    public void mouseEntered(MouseEvent e) {}
    public void mouseExited(MouseEvent e) {}
    public boolean startDrag(boolean active, int x, int y, int w, int h) {
      if (!active) {
	active = true;
	x1 = x;
	y1 = y;
	boxWidth = w;
	boxHeight = h;
	Graphics g = getGraphics();

	// draws a box with the upper-left corner at x1,y1 and with the
	// specified width (to the right) and height (direct to the bottom):
	//------------------------------------------------------------------
	g.drawRect(x1,y1,boxWidth,boxHeight);
      }
      else active = false;
      return active;
    }
    public void mousePressed(MouseEvent e) {
      double xx, yy;
      xStart = e.getX();
      yStart = e.getY();
      if (JPlot.debug) {
	System.out.println("mouse at (x,y) = " + xStart + "," + yStart);
      }

      // If the mouse pointer hits the legend box,
      // start dragging of the legend:
      //------------------------------------------
      xx = gp.getLegendPos(X);
      yy = gp.getLegendPos(Y);
      if (gp.drawLegend() && xStart > xx && xStart < xx+legendWidth &&
	  yStart > yy && yStart < yy+legendHeight) {
	legendActive = startDrag(legendActive,(int)xx,(int)yy,
				 (int)(legendWidth),(int)(legendHeight));
	somethingActive = legendActive;
      }

      // Check whether the mouse hits one of the current labels
      //-------------------------------------------------------
      else {
	for (Enumeration el=gp.getLabels().elements(); el.hasMoreElements();) {
	  GraphLabel gl = (GraphLabel) el.nextElement();
	  if (gl.contains(xStart,yStart)) {
	    gl.setActive(startDrag(gl.isActive(),(int)gl.getX(),(int)gl.getY(),
				   (int)gl.getWidth(),(int)gl.getHeight()));
	    somethingActive = gl.isActive();
	    break;
	  }
	}
      }
    }
    public void mouseReleased(MouseEvent e) {
      if (!somethingActive) return;
      else if (legendActive) {
	legendActive = false;
	gp.setLegendPos((double)x1,(double)y1);
	gp.setUseLegendPos(true);
	gp.setDataChanged(true);
	updateGraph();
      }
      else {
	for (Enumeration el=gp.getLabels().elements(); el.hasMoreElements();) {
	  GraphLabel gl = (GraphLabel) el.nextElement();
	  if (gl.isActive()) {
	    gl.setActive(false);
	    if (x1 != (int)gl.getX() || y1 != (int)gl.getY()) {
	      gp.setDataChanged(true);
	      gl.setLocation((double)x1,(double)y1);
	      gl.setUsePosition(true);
	      updateGraph();
	      if (JPlot.debug) {
		System.out.println("fixing label " + gl.getText() + 
				   ", bb = " + x1 + "," + y1 + "," +
				   (int)gl.getWidth() + "," + (int)gl.getHeight() + 
				   ",  text at " + (int)gl.getXPos() + "," + 
				   (int)gl.getYPos());
	      }
	    }
	    else repaint();
	    break;
	  }
	}
      }
      somethingActive = false;
    }
  }
  
  public class DragListener implements MouseMotionListener {
    public void mouseDragged(MouseEvent e) {
      if (somethingActive) {
	int x = e.getX();
	int y = e.getY();

	// erase the previous box:
	Graphics g = getGraphics();
	g.setXORMode(boxColor);
	g.drawRect(x1,y1,boxWidth,boxHeight);
	
	// draw the new box:
	x1 += (x - xStart);
	y1 += (y - yStart);
	g.drawRect(x1,y1,boxWidth,boxHeight);
	g.setPaintMode();
	xStart = x;
	yStart = y;
      }
    }
    public void mouseMoved(MouseEvent e) {}
  }
}
