/*
 * JPLOT  -- Java Plotting Interface for any programme or
 *           as an independent GUI
 * 
 * Copyright (C) 1999-2000 Jan van der Lee
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA.
 *
 * Send bugs, suggestions or queries to <jplot@cig.ensmp.fr>
 * The latest releases are found at 
 * http://www.cig.ensmp.fr/~vanderlee/jplot.html
 *
 * Initally developed for use by the Centre d'Informatique Geologique 
 * Ecole des Mines de Paris, Fontainebleau, France.
 */

package jplot;

import java.awt.font.*;
import java.awt.*;
import java.text.*;
import javax.swing.*;
import java.util.*;
import java.io.*;

/**
 * This class defines a label in terms of the actual text, font, color
 * and position but also stuff like rotation, whether to hide or show etc.
 * Some special treatment for standard X, Y labels and a title. Random labels
 * are considered to be of type OTHER, which places them, by default, in the
 * lower-left corner of the graph. 
 *
 * Extends Rectangle, since a label has a rectangular bounding box,
 * used to select, drag and drop the label. Note that the bounding box
 * is defined by the upper-left corner, width (to the right) and
 * height (directed towards the bottom).  
 */
public class GraphLabel {

  static final int UNKNOWN=-1;
  static final int XLABEL=0;    // X label for a 2D graph
  static final int YLABEL=1;    // Y label for a 2D graph
  static final int TITLE=2;     // title for a 2D graph
  static final int PIPER_X1=3;  // first X label for a piper diagram
  static final int PIPER_X2=4;  // second X label for a piper diagram
  static final int PIPER_Y1=5;  // lower-left Y label for a piper diagram
  static final int PIPER_Y2=6;  // lower-right Y label for a piper diagram
  static final int PIPER_Y3=7;  // upper-left Y label for a piper diagram
  static final int PIPER_Y4=8;  // upper-right Y label for a piper diagram
  static final int DATA=9;      // labels defined by the data x,y coordinates
  static final int OTHER=10;    // random labels
  static final int CHECK=11;    // DATA labels which must be checked


  private String text;
  private Font font;
  private Color color;
  private boolean active;
  private boolean usePos;
  private FontMetrics fm;
  private double rotation;
  private int whoAmI;
  private double xPos;   // x-position of the text
  private double yPos;   // y-position of the text
  private double bbX;    // x-position of the bounding box
  private double bbY;    // x-position of the bounding box
  private double textWidth;
  private double textHeight;
  private double bbWidth;
  private double bbHeight;
  private double sin,cos;
  private AttributedString attributedString;
  private File file;    // label belongs to this file

  private final String lf = System.getProperty("line.separator");

  /**
   * Constructor, sets the text of the label.
   * @param t type of this label
   * @param s text of the label
   * @param f font of the label
   * @param c text color of the label
   */
  public GraphLabel(int t, String s, Font f, Color c) {
    font = f;
    color = c;
    usePos = false;
    active = false;
    rotation = 0.0;
    sin = 0.0;
    cos = 1.0;
    xPos = yPos = 0.0;
    bbX = bbY = 0.0;
    bbWidth = bbHeight = 0.0;
    file = null;
    setID(t);    // might change rotate if it is an YLABEL
    setFont(f);  
    setText(s); // should be at the end
  }

  /*
   * Sets the size of the bounding box.
   * Note that this doesn't work perfectly well since rotated text 
   * is set closer together, at least in the current jvm (1.3).
   * Hence we have to introduce some ugly empirical corrections.
   */
  private void setBoundingBox() {
    if (rotation != 0.0) {
      setSize(textWidth*cos + textHeight*sin,
	      textHeight*cos + textWidth*sin);
    }
    else setSize(textWidth,textHeight);
  }

  /*
   * Determine the font metrics of the current text.
   * Wouldn't it be nice if I could get the font-metrics of 
   * the text *without* this ugly memory allocation???
   */
  private void setTextMetrics() {
    if (text != null && !text.equals("")) {
      attributedString = new AttributedString(text);
      attributedString.addAttribute(TextAttribute.FONT,font);
      if (fm == null) fm = (new JPanel()).getFontMetrics(font);
      textWidth = fm.stringWidth(text);
      textHeight = fm.getHeight();
      setBoundingBox();
    }
    else textWidth = textHeight = 0.0;
  }

  /**
   * Constructor, sets the text of the label.
   * @param s new text of the label
   */
  public GraphLabel(int t, String s) {
    this(t,s,Utils.getDefaultFont(),Color.black);
  }

  /**
   * Constructor, sets the text of the label.
   * @param s new text of the label
   */
  public GraphLabel(String s) {
    this(UNKNOWN,s);
  }

  /**
   * Constructor, sets the ID of the label but no text.
   * @param t ID of the label
   */
  public GraphLabel(int t) {
    this(t,"");
  }

  /**
   * Constructor, does nothing but initializing.
   */
  public GraphLabel() {
    this("");
  }

  /**
   * Constructor, builds the class with another graph label.
   */
  public GraphLabel(GraphLabel gl) {
    copy(gl);
  }

 /**
   * Constructor, builds the class with another graph label.
   */
  public void copy(GraphLabel gl) {
    bbX = gl.getX();
    bbY = gl.getY();
    bbWidth = gl.getWidth();
    bbHeight = gl.getHeight();
    active = gl.isActive();
    usePos = gl.usePosition();
    setRotation(gl.getRotation());
    setID(gl.getID());
    xPos = gl.getXPos();
    yPos = gl.getYPos();
    textWidth = gl.getTextWidth();
    textHeight = gl.getTextHeight();
    file = gl.getFile();
    setFont(new Font(gl.getFont().getName(),
		     gl.getFont().getStyle(),gl.getFont().getSize()));
    color = new Color(gl.getColor().getRed(),
		      gl.getColor().getGreen(),gl.getColor().getBlue());
    setText(gl.getText());
  }

  /**
   * Sets the text of the label to something new.
   * @param s new text for the label
   */
  public void setText(String s) {
    text = s;
    setTextMetrics();
    setLocation(bbX,bbY);
  }

  /**
   * @returns the current text of the label
   */
  public String getText() {
    return text;
  }

  /**
   * @returns an attributedCharacterIterator instance of the
   * current text
   */
  public AttributedCharacterIterator getCharIterator() {
    return attributedString.getIterator();
  }

  /**
   * Sets the color to a specific value
   * @param c color used to draw the label
   */
  public void setColor(Color c) {
    color = c;
  }

  /**
   * @return the color used to draw the label
   */
  public Color getColor() {
    return color;
  }

  /**
   * Sets the font to a specific value
   * @param f font used to draw the label
   */
  public void setFont(Font f) {
    font = f;
    fm = (new JPanel()).getFontMetrics(font);
    setTextMetrics();
  }

  /**
   * Sets the font to a specific value
   * @return the font used to draw the label
   */
  public Font getFont() {
    return font;
  }

  /**
   * @param b true if the user starts to draw the label
   */
  public void setActive(boolean b) {
    active = b;
  }

  /**
   * @return true if the label is actually being drawn by the mouse
   */
  public boolean isActive() {
    return active;
  }

  /**
   * Allows the user to used the current x- and y-positions,
   * will use default positions otherwise
   * @param b true if the positions should be used
   */
  public void setUsePosition(boolean b) {
    usePos = b;
  }

  /**
   * @return true if the positions should be used
   */
  public boolean usePosition() {
    return usePos;
  }

  /**
   * returns true if the current label corresponds to one of
   * the pre-defined label types (X-label, Y-label etc).
   * @param type type to compare with (must be an int)
   * @return true if 'type' corresponds to the current 'whoAmI'
   */
  public boolean equals(int type) {
    return (type == whoAmI);
  }

  /**
   * returns true if the current label corresponds to one of
   * the pre-defined label types (X-label, Y-label etc).
   * @param name name to compare with (must be a string)
   * @return true if 'name' corresponds to the current label text
   */
  public boolean equals(String name) {
    return text.equals(name);
  }

  /**
   * Sets the current identity of the label to some specific value
   * The idientity must be an int and, in principle, something predefined 
   * such as GraphLabel.XLABEL).
   * @param type type of the current label
   */
  public void setID(int type) {
    whoAmI = type;
  }

  /**
   * @return the current identity of the label to some specific value.
   */
  public int getID() {
    return whoAmI;
  }

  /**
   * Sets the rotation angle.
   * @param r angle for this label
   */
  public void setDataRotation(double r) {
    rotation = r;
  }

  /**
   * Sets the rotation angle. Although all kind of angles (in PI-units)
   * are allowed, internally we only use the interval 0-2pi hence we
   * take care of all other cases.
   * @param r angle for this label
   */
  public void setRotation(double r) {
    rotation = r;
    while (rotation < 0.0) rotation += 2.0*Math.PI;
    while (rotation > 2.0*Math.PI) rotation -= 2.0*Math.PI;
    if (rotation != 0.0) {
      sin = Math.abs(Math.sin(rotation));
      cos = Math.abs(Math.cos(rotation));
    }
    else {
      sin = 0.0;
      cos = 1.0;
    }
    setBoundingBox();
    setLocation(bbX,bbY);
  }

  /**
   * Returns the rotation angle.
   * @return angle for this label
   */
  public double getRotation() {
    return rotation;
  }

  /**
   * This function also sets the text location of the label text
   * itself, slightly different from the x,y position of the box.
   * Optimize this function since it is used for drag'n drop.
   *
   * @param x x-position of the lower-left corner of the text
   * @param y y-position of the lower-left corner of the text
   */
  public void setLocation(double x, double y) {
    bbX = x;
    bbY = y;
    if (rotation <= Math.PI/2.0) {
      xPos = x;
      yPos = y + textHeight*cos;
    }
    else if (rotation <= Math.PI) {
      xPos = x + getWidth() - textHeight*sin;
      yPos = y;
    }
    else if (rotation <= 3.0*Math.PI/2.0) {
      xPos = x + getWidth() - textHeight*sin;
      yPos = y + getHeight() - textHeight*cos;
    }
    else if (rotation <= 2.0*Math.PI) {
      xPos = x + textHeight*sin;
      yPos = y + getHeight();
    }
  }

  /**
   * Sets the location of this label in data coordinates.
   * Ones set this way, you cannot plot the label, it must first
   * define the location in pixel-coordinates (setLocation(x,y));
   * @param x x-position of the lower-left corner of the text
   * @param y y-position of the lower-left corner of the text
   */
  public void setDataLocation(double x, double y) {
    xPos = x;
    yPos = y;
  }

  /**
   * Returns the X position of the text.
   * @return the x-position of the text
   */
  public double getXPos() {
    return xPos;
  }

  /**
   * Returns the Y position of the text.
   * @return the y-position of the text
   */
  public double getYPos() {
    return yPos;
  }

  /**
   * Returns the X position of the bounding box.
   * @return the x-position of the bounding box
   */
  public double getX() {
    return bbX;
  }

  /**
   * Returns the Y position of the bounding box.
   * @return the y-position of the bounding box
   */
  public double getY() {
    return bbY;
  }

  /**
   * Sets the size of the bounding box:
   * @param w width of the bb
   * @param h height of the bb
   */
  public void setSize(double w, double h) {
    bbWidth = w;
    bbHeight = h;
  }

  /**
   * Sets the size of the text-label:
   * @param w width of the label
   * @param h height of the label
   */
  public void setTextSize(double w, double h) {
    textWidth = w;
    textHeight = h;
  }

  /**
   * Returns the width of the bounding box of the label
   * @return the width of the bounding box
   */
  public double getWidth() {
    return bbWidth;
  }

  /**
   * Returns the height of the bounding box of the label
   * @return the height of the bounding box
   */
  public double getHeight() {
    return bbHeight;
  }

  /**
   * Returns the width of the text of the label
   * @return the width of the text
   */
  public double getTextWidth() {
    return textWidth;
  }

  /**
   * Returns the height of the text of the label
   * @return the height of the text
   */
  public double getTextHeight() {
    return textHeight;
  }

  /**
   * @return the file to which the label belongs
   */
  public File getFile() {
    return file;
  }

  /**
   * Sets the file to which the label belongs. Can be null.
   * @param f file to which the label belongs
   */
  public void setFile(File f) {
    file = f;
  }

  /**
   * @return a string representation of this label, the text.
   */
  public String toString() {
    return text;
  }

  /**
   * Checks whether x,y falls within the bounding box of the
   * current text.
   * @param x x-position
   * @param y y-position
   * @return true if the point x,y falls withing the bounding box.
   */
  public boolean contains(double x, double y) {
    return (x>bbX && x<bbX+bbWidth &&  y>bbY && y<bbY+bbHeight);
  }

  /**
   * Returns the settings of the current label in a string.
   * Calling this method creates a string containing the name
   * and settings of the current label in string-format, formatted
   * such that it can directly be used for a script file.
   * @return the settings of the current label in a string.
   */
  public String getSettings() {
    StringBuffer sb = new StringBuffer(200);
    sb.append(lf);
    sb.append("    label \"" + text + "\" {");
    sb.append(lf);
    sb.append("      whoami = ").append(whoAmI);
    sb.append(lf);
    sb.append("      color = ").append(color.getRed()).append(",");
    sb.append(color.getGreen()).append(",").append(color.getBlue());
    sb.append(lf);
    sb.append("      font ").append(" = \"").append(font.getName()).append("\",");
    sb.append(font.getStyle()).append(",").append(font.getSize());
    if (usePos) {
      sb.append(lf);
      sb.append("      location = ").append((int)bbX).append(",");
      sb.append((int)bbY);
    }
    sb.append(lf);
    sb.append("      rotation = ").append(rotation/Math.PI*180.0);
    sb.append(lf).append("    }");
    return sb.toString();
  }
  
}
