/*
 * JPLOT  -- Java Plotting Interface for any programme or
 *           as an independent GUI
 * 
 * Copyright (C) 1999 Jan van der Lee
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA.
 *
 * Send bugs, suggestions or queries to <jplot@cig.ensmp.fr>
 * The latest releases are found at 
 * http://www.cig.ensmp.fr/~vanderlee/jplot.html
 *
 * Initally developed for use by the Centre d'Informatique Geologique 
 * Ecole des Mines de Paris, Fontainebleau, France.
 */

package jplot;

import javax.swing.*;
import java.awt.*;
import java.awt.image.*;
import java.text.*;
import java.math.*;
import java.awt.geom.*;
import java.awt.event.*;
import javax.swing.border.*;
import java.util.*;
import java.awt.print.*;

/**
 * The <code>Graph_2D</code> class builds a panel which displays a
 * 2Dgraph according to the data stored in the {@link DataArray}.
 * All general graph stuff and initialization is done in class
 * <code>Graph</code>.
 *
 * @version 24/08/99
 * @author  J. van der Lee
 */
public class Graph_2D extends Graph {

  // width of one histo-bar:
  private double histoWidth;
  
  /**
   * Main constructor.
   * @param jp jplot instance
   * @param gp objects containing the parameters used to draw this graph.
   */
  public Graph_2D(JPlot jp, GraphPars gp) {
    super(jp,gp);
    histoWidth = 10.0;
  }

  /*
   * Find the boundary values for an X- or Y range in the case 
   * of logarithmic scaling.
   * @param axis defines the axis we are working on
   * @param v value which will be converted to log-scale
   * @return new boundary value, now for the log scale.
   */
  private double getLogBoundary(int axis, double v) {
    double k = 0.0;
    double x = 10.0;
    if (v > 1.0) {
      while (v >= 1.0 && k < 299) {
	v /= 10.0;
	k += 1.0;
      }
    }
    else {
      x = 0.1;
      while (v <= 0.1 && k < 299) {
	v *= 10.0;
	k += 1.0;
      }
    }
    return Math.pow(x,k);
  }

  /**
   * Determines the minimum and maximum values of the X-range
   * This function sets the Xmin, Xmax values
   * to the minimum X, maximum X etc. as found in the data-arrays,
   * <em>unless</em> specified explicitely by the user.
   *
   * @param axis symbolic constant, saying which axis we are.
   * @param data vector of plotting points (curves)
   */
  protected boolean setMinMax(int axis, Vector data) {
    double min = gp.getMinValue(axis);
    double max = gp.getMaxValue(axis);
    //System.out.println("starting with min, max = " + min + "," + max);

    // for automatic scaling, determine the max-
    // and min values of the different all the data arrays:
    //-----------------------------------------------------
    if (gp.useAutoRange(axis)) {
      Enumeration e=data.elements();
      DataArray da = (DataArray)e.nextElement();
      double minVal;
      if (gp.useLogScale(axis)) minVal = da.getLowestNonZeroValue(axis);
      else minVal = da.getMinValue(axis);
      //System.out.println("minVal = " + minVal);
      min = minVal;
      max = da.getMaxValue(axis);
      while (e.hasMoreElements()) {
	da = (DataArray)e.nextElement();
	if (gp.useLogScale(axis)) minVal = da.getLowestNonZeroValue(axis);
	else minVal = da.getMinValue(axis);
	if (minVal < min) min = minVal;
	if (da.getMaxValue(axis) > max) max = da.getMaxValue(axis);
      }

      if (min == max) {
	min -= 0.1;
	max += 0.1;
      }

      // some special treatment for logarithmic axes:
      //---------------------------------------------
      if (gp.useLogScale(axis)) {
	min = getLogBoundary(axis,min)/10.0;
	max = getLogBoundary(axis,max);
      }

      if (JPlot.debug) {
	System.out.print("min and max values for ");
	if (axis == X) System.out.print("X-axis data: ");
	else System.out.print("Y-axis data: ");
	System.out.println("min = " + min + ", max = " + max);
      }
    }

    if (gp.useLogScale(axis)) {
      max = log10(max);
      min = log10(min);
    }


    // some smart rounding algorithms here! Note the ridiculus
    // "/(1.0/tic)" instead of *tic, an experimental trick to
    // avoid the java bug (0.1999999999999999 instead of 0.2,
    // I known they say it's a feature but it the most useless
    // feature I've ever seen in my live  ---a bug for me:-().
    //--------------------------------------------------------
    double tic = calculateTicSep(min,max);
    if (gp.useAutoRange(axis)) {
      if (min < max) {
	min = (double)tic*Math.floor(min/tic);
	max = (double)tic*Math.ceil(max/tic);
      }
      else {
	min = (double)tic*Math.ceil(min/tic);
	max = (double)tic*Math.floor(max/tic);
      }
    }
    if (gp.useNumberOfTics(axis)) {
      numberOfTics[axis] = gp.getNumberOfTics(axis);
    }
    else {
      numberOfTics[axis] = (int)Math.ceil(Math.abs(min-max)/tic-1e-10)+1;
      gp.setNumberOfTics(axis,numberOfTics[axis]);
    }
    diff[axis] = (double)Math.abs(min - max);

    if (gp.useLogScale(axis)) {
      if (numberOfTics[axis] > diff[axis]) {

	// the following (1.0+1e-10) trick is introduced, again
	// to avoid the rounding behaviour ('feature') of java.
	//-----------------------------------------------------
	numberOfTics[axis] = (int)(diff[axis]*(1.0+1e-10))+1;
	gp.setNumberOfTics(axis,numberOfTics[axis]);
      }
      min = Math.pow(10,min);
      max = Math.pow(10,max);
    }

    inv[axis] = (min < max)? 1.0 : -1.0;

    if (JPlot.debug) {
      System.out.println("number of tics for axis " + 
			 axis + " = " + numberOfTics[axis]);
      System.out.print(", min and max values after smoothing: ");
      System.out.println("min = " + min + ", max = " + max);
      System.out.println("diff[X] = " + diff[X] + ", diff[Y] = " + diff[Y]);
    }
    gp.setMinValue(axis,min);
    gp.setMaxValue(axis,max);
    return true;
  }

  /*
   * Determine the axis labels, if needed. These are numbers
   * which are translated in Strings. The length is evalulated
   * and used to set the left and bottom margins.
   * @param 
   */
  protected void makeTicLabels() {
    double f, Xn, Yn;
    final int numDigits = 4;

    // make the X-axis labels:
    //------------------------
    if (gp.drawTicLabels(X)) {
      if (gp.useLogScale(X)) Xn = inv[X]*diff[X];
      else Xn = (gp.getMaxValue(X)-gp.getMinValue(X));
      Xn /= (numberOfTics[X]-1);
      if (JPlot.debug) System.out.print("\nXlabels:");
      for (int i=0; i<numberOfTics[X]; i++) {
	if (gp.useLogScale(X)) f = gp.getMinValue(X)*Math.pow(10.0,Xn*i);
	else f = gp.getMinValue(X)+i*Xn;
	if (JPlot.debug) System.out.print(" " + f);
	ticLabel[X][i] = formatNumber(f,numDigits);
      }
    }

    // make the Y axes labels:
    //------------------------    
    if (gp.drawTicLabels(Y)) {
      if (gp.useLogScale(Y)) Yn = inv[Y]*diff[Y];
      else Yn = (gp.getMaxValue(Y)-gp.getMinValue(Y));
      Yn /= (numberOfTics[Y]-1);
      if (JPlot.debug) System.out.print("\nYlabels:");
      for (int i=0; i<numberOfTics[Y]; i++) {
	if (gp.useLogScale(Y)) f = gp.getMinValue(Y)*Math.pow(10.0,Yn*i);
	else f = gp.getMinValue(Y)+Yn*i;
	if (JPlot.debug) System.out.print(" " + f);
	ticLabel[Y][i] = formatNumber(f,numDigits);
      }
      if (JPlot.debug) System.out.println();
    }    
  }

  /*
   * This function draws the grid.
   * @param g2 instance of the graphical canvas
   */
  private void drawGrid(Graphics2D g2) {
    double x = leftMargin + axisLength[X];
    double y = topMargin + axisLength[Y];
    g2.setStroke(new BasicStroke());
    g2.setColor(gp.getGridColor());

    // draw the X-grid lines:
    //-----------------------
    if (gp.drawGrid(X)) {
      for (int i=1; i<numberOfTics[X]-1; i++) {
	double d = leftMargin + sep[X]*i;
	g2.draw(new Line2D.Double(d,topMargin,d,y));
      }
    }

    // draw the Y-grid lines:
    //-----------------------
    if (gp.drawGrid(Y)) {
      for (int i=1; i<numberOfTics[Y]-1; i++) {
	double d = topMargin + sep[Y]*i;
	g2.draw(new Line2D.Double(leftMargin,d,x,d));
      }
    }
  }

  /*
   * This function draws the axes (actually X,Y and mirror axes) and 
   * the tics on the axes.
   * @param g2 instance of a graphical canvas on which to draw the axes
   */
  private void plotAxes(Graphics2D g2) {
    rect.setRect(leftMargin,topMargin,axisLength[X],axisLength[Y]);
    g2.setColor(gp.getAxesColor());
    g2.draw(rect);

    double x = leftMargin + axisLength[X];
    double y = topMargin + axisLength[Y];

    // draw the X-axis tics:
    //----------------------
    if (gp.drawTics(X)) {
      for (int i=0; i<numberOfTics[X]; i++) {
	double d = leftMargin + sep[X]*i;
	g2.draw(new Line2D.Double(d,y-ticLength[X],d,y));
	if (gp.drawMirrorTics(X)) {
	  g2.draw(new Line2D.Double(d,topMargin+ticLength[X],d,topMargin));
	}
      }
    }

    // draw the Y axes tics:
    //----------------------
    if (gp.drawTics(Y)) {
      for (int i=0; i<numberOfTics[Y]; i++) {
	double d = topMargin+sep[Y]*i;
	g2.draw(new Line2D.Double(leftMargin,d,leftMargin+ticLength[Y],d));
	if (gp.drawMirrorTics(Y)) {
	  g2.draw(new Line2D.Double(x,d,x-ticLength[Y],d));
	}
      }
    }
  }

  /*
   * Draw the labels around the x- and y-tics. The xLabelHeight is determined
   * as a function of one label-height (assuming the height is the same for
   * all the labels).
   * @param g2 instance of a 2D graphical canvas
   */
  private void plotTicLabels(Graphics2D g2) {
    float x, y;
    double a = topMargin + axisLength[Y];
    double f;

    // draw the X-axis labels:
    //------------------------
    if (gp.drawTicLabels(X)) {
      FontMetrics fm = getFontMetrics(gp.getTicFont(X));
      //double xLabelHeight = 3*fm.getHeight()/4;
      double xLabelHeight = fm.getHeight();
      //System.out.println("a = " + a + ", height = " + xLabelHeight);
      g2.setColor(gp.getTicColor(X));
      g2.setFont(gp.getTicFont(X));
      for (int i=0; i<numberOfTics[X]; i++) {
	x = (float)(leftMargin+sep[X]*i-fm.stringWidth(ticLabel[X][i])/2);
	y = (float)(a+xLabelHeight);
	//System.out.println("setting y of " + ticLabel[X][i] + " at " + y);
	g2.drawString(ticLabel[X][i],x,y);
      }
    }
    
    // draw the Y axes labels:
    //------------------------
    if (gp.drawTicLabels(Y)) {
      FontMetrics fm = getFontMetrics(gp.getTicFont(Y));
      double yLabelHeight = fm.getHeight();
      g2.setColor(gp.getTicColor(Y));
      g2.setFont(gp.getTicFont(Y));
      for (int i=0; i<numberOfTics[Y]; i++) {
	x = (float)(leftMargin-fm.stringWidth(ticLabel[Y][i])-xSep);
	y = (float)(a-sep[Y]*i+yLabelHeight/4);
	g2.drawString(ticLabel[Y][i],x,y);
      }
    }
  }
  
  /*
   * Returns the X-value scaled to the pixel-availability.
   * This function takes the X-value and returns the corresponding
   * coordinates for the panel.
   * @param x real x-value (as introduced by the data)
   * @return x-value scaled to the actual panel coordinates
   */
  protected double toX(double x) {
    double d;
    if (gp.useLogScale(X)) d = log10(x/gp.getMinValue(X));
    else d = x - gp.getMinValue(X);
    return leftMargin + inv[X]*d*axisLength[X]/diff[X];
  }

  /*
   * Returns the Y-value scaled to the pixel-availability.
   * This function takes the Y-value and returns the corresponding
   * coordinates for the panel.
   * @param y real y-value (as introduced by the data)
   * @return y-value scaled to the actual panel coordinates
   */
  protected double toY(double y) {
    double d;
    if (gp.useLogScale(Y)) d = log10(y/gp.getMinValue(Y));
    else d = y - gp.getMinValue(Y);
    return topMargin + axisLength[Y]*(1.0 - inv[Y]*d/diff[Y]);
  }

  /**
   * Fills the graph area with a background color. The
   * area is the area between the axes.
   * @param g2 graphics canvas
   */
  protected void fillGraphArea(Graphics2D g2) {
    rect.setRect(leftMargin,topMargin,axisLength[X],axisLength[Y]);
    g2.setColor(gp.getGraphBackgroundColor());
    g2.fill(rect);
  }
  
  /**
   * This function builds the graph in a double-buffered image zone.
   * Make sure that the size of the graph (width x height) is set
   * before calling this function.
   */
  protected void updateGraph() {

    // create graphics using the createGraphics method of the 
    // super class, which creates it in a BufferedImage:
    Graphics2D g2 = createGraphics();

    double x1, x2, y1, y2;
    double a = topMargin + axisLength[Y];
    double zero = toY(0.0);
    boolean liftPen = false;

    // draw the on the background, if this is what they want:
    if (!gp.gridToFront()) drawGrid(g2);

    int i;
    for (Enumeration e=data.elements(); e.hasMoreElements();) {
      
      // get the next data array from the vector:
      DataArray da = (DataArray)e.nextElement();
      int every = da.getPointFrequency();

      // if histogram steps are asked for, find the maximum width:
      if (da.getGraphStyle() == da.HISTO) {
	for (i=1; i<da.size(); i++) {
	  double w = toX(da.getX(i)) - toX(da.getX(i-1)) - 1.0;
	  if (w < histoWidth) histoWidth = w;
	}
      }

      // set color and penwidth:
      g2.setColor(da.getColor());
      if (da.drawLine()) g2.setStroke(da.getStroke());

      // make sure we start with values which fall inside the axes system:
      x1 = leftMargin-1;
      y1 = topMargin+1;
      for (i=0; i<da.size() && !inRange(x1,y1); i++) {
        x1 = toX(da.getX(i));
        y1 = toY(da.getY(i));
      }
      if (da.drawSymbol() && every < 2) {
	drawPointType(da.getSymbol(),g2,x1,y1,da.getSymbolSize());
      }
      if (da.getGraphStyle() == da.STICKS) {
	line.setLine(x1,y1,x1,zero);
	g2.draw(line);
      }
      else if (da.getGraphStyle() == da.HISTO) {
	//if (da.size() > i) histoWidth = toX(da.getX(i))-x1;
	rect.setRect(x1-histoWidth/2,y1,histoWidth,a-y1);
	g2.draw(rect);
      }
      for (; i<da.size(); i++) {
	PlotPoint pp = da.getPoint(i);
	x2 = toX(pp.getX());
        y2 = toY(pp.getY());
	
	// if dashes are used, calculate the minimum distance between
	// two plotting points in order to display the dash correctly:
	if (da.getDashLength() > 0) {
	  double minDist = Math.sqrt((x2-x1)*(x2-x1) + (y2-y1)*(y2-y1));
	  if (minDist < da.getDashLength()*2) continue;
	}
        if (inRange(x1,y1) && inRange(x2,y2)) {
	  if (da.getGraphStyle() == da.HISTO) {
	    rect.setRect(x2-histoWidth/2,y2,histoWidth,a-y2);
	    g2.draw(rect);
	    //histoWidth = x2-x1-histoWidth/2;
	  }
	  else {
	    if (da.drawLine() && !liftPen) {
	      line.setLine(x1,y1,x2,y2);
	      g2.draw(line);
	    }
	    if (da.getGraphStyle() == da.STICKS) {
	      line.setLine(x2,y2,x2,zero);
	      g2.draw(line);
	    }
	  }
	  if (da.drawSymbol() && (i%every)==0) {
	    drawPointType(da.getSymbol(),g2,x2,y2,da.getSymbolSize());
	  }
        }
        x1 = x2;
        y1 = y2;
	liftPen = pp.liftPen();
      }
    }

    // draw the on the foreground, if this is what they want:
    if (gp.gridToFront()) drawGrid(g2);

    g2.setStroke(new BasicStroke());

    // draw the axes and tics:
    plotAxes(g2);
    
    // draw the tic labels:
    plotTicLabels(g2);
    
    // draw the title:
    // It's now a (special) label, this is not needed anymore.
    //if (!gp.getTitle().equals("")) plotTitle(g2);
    plotLabels(g2);
    
    // draw the legend. Do this at the end because it must be
    // drawn on the top of all other drawing stuff (e.g. grid...)
    if (gp.drawLegend()) {
      int k=0;
      g2.setFont(gp.getLegendFont());
      for (Enumeration e=data.elements(); e.hasMoreElements(); k++) {
	DataArray da = (DataArray)e.nextElement();
	if (!da.drawLegend()) k--;
	else {
	  g2.setColor(da.getColor());
	  if (da.drawLine()) g2.setStroke(da.getStroke());
	  drawLegend(g2,da,k);
	}
      }
    }
    g2.setStroke(new BasicStroke());
    repaint();
  }

  /**
   * Checks whether x and y are within the ranges. The ranges are
   * defined by the axes system.
   * @param x x-point
   * @param y y-point
   */
  private boolean inRange(double x, double y) {
    if (x < leftMargin || x > leftMargin + axisLength[X] ||
	y < topMargin || y > topMargin + axisLength[Y]) return false;
    return true;
  }
}
