/*
 * JPLOT  -- Java Plotting Interface for any programme or
 *           as an independent GUI
 * 
 * Copyright (C) 1999 Jan van der Lee
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA.
 *
 * Send bugs, suggestions or queries to <jplot@cig.ensmp.fr>
 * The latest releases are found at 
 * http://www.cig.ensmp.fr/~vanderlee/jplot.html
 *
 * Initally developed for use by the Centre d'Informatique Geologique 
 * Ecole des Mines de Paris, Fontainebleau, France.
 */

package jplot;

import javax.swing.*;
import java.awt.*;
import java.awt.image.*;
import java.text.*;
import java.math.*;
import java.awt.geom.*;
import java.awt.event.*;
import javax.swing.border.*;
import java.util.*;
import java.awt.print.*;

/**
 * The <code>Graph_Piper</code> class builds a panel which displays a
 * Piper-diagram (ask geochemists what this is) according to the data 
 * stored in the {@link DataArray}.
 * All general graph stuff and initialization is done in class
 * <code>Graph</code>.
 *
 * @version 24/08/99
 * @author  J. van der Lee
 */
public class Graph_Piper extends Graph {

  // handy variables
  private final double sin30 = Math.sin(Math.PI/6.0);

  /**
   * Main constructor.
   * @param jp current jplot instance
   * @param gp objects containing the parameters used to draw this graph.
   */
  public Graph_Piper(JPlot jp, GraphPars gp) {
    super(jp,gp);
    if (JPlot.debug) System.out.print("Initializing piper diagram...");
    piperSep = 20.0;
    gp.setBoxOffset(10.0f);
    gp.setShowBox(true);
    normalSep = piperSep*Math.sin(Math.PI/3);
    gp.setMaxValue(X,100);
    gp.setMinValue(X,0);
    gp.setMaxValue(Y,100);
    gp.setMinValue(Y,0);
    numberOfTics[X] = 6;
    numberOfTics[Y] = 6;
    GraphLabel gl = new GraphLabel(GraphLabel.PIPER_X1,"Ca[2+]",
				   gp.getTicFont(X),Color.black);
    gp.addLabel(gl);
    gl = new GraphLabel(GraphLabel.PIPER_X2,"Cl[-]",
			gp.getTicFont(X),Color.black);
    gp.addLabel(gl);
    gl = new GraphLabel(GraphLabel.PIPER_Y1,"Mg[2+]",
			gp.getTicFont(Y),Color.black);
    gp.addLabel(gl);
    gl = new GraphLabel(GraphLabel.PIPER_Y2,"SO4[2-]",
			gp.getTicFont(Y),Color.black);
    gp.addLabel(gl);
    gl = new GraphLabel(GraphLabel.PIPER_Y3,"Cl[-] + SO4[2-]",
			gp.getTicFont(Y),Color.black);
    gp.addLabel(gl);
    gl = new GraphLabel(GraphLabel.PIPER_Y4,"Ca[2+] + Mg[2+]",
			gp.getTicFont(Y),Color.black);
    gp.addLabel(gl);
    if (JPlot.debug) System.out.println("done.");
  }

  /*
   * Makes a set of defaults labels which are set around the axes.
   */
  private void makeDefaultLabels() {
  }

  /**
   * Determine the axis labels, if needed. These are numbers
   * which are translated in Strings. The length is evalulated
   * and used to set the left and bottom margins.
   */
  protected void makeTicLabels() {
    double f, Xn, Yn;
    final int numDigits = 4;

    // make the X-axis labels:
    //------------------------
    if (gp.drawTicLabels(X)) {
      Xn = (gp.getMaxValue(X)-gp.getMinValue(X))/(numberOfTics[X]-1);
      if (JPlot.debug) System.out.print("\nXlabels:");
      for (int i=0; i<numberOfTics[X]; i++) {
	f = gp.getMinValue(X)+i*Xn;
	if (JPlot.debug) System.out.print(" " + f);
	ticLabel[X][i] = formatNumber(f,numDigits);
      }
    }

    // make the Y axes labels:
    //------------------------    
    if (gp.drawTicLabels(Y)) {
      Yn = (gp.getMaxValue(Y)-gp.getMinValue(Y))/(numberOfTics[Y]-1);
      if (JPlot.debug) System.out.print("\nYlabels:");
      for (int i=0; i<numberOfTics[Y]; i++) {
	f = gp.getMinValue(Y)+Yn*i;
	if (JPlot.debug) System.out.print(" " + f);
	ticLabel[Y][i] = formatNumber(f,numDigits);
      }
      if (JPlot.debug) System.out.println();
    }    
  }

  /**
   * This function draws the tics on the axis.
   * Quite complex job for piper diagrams...
   * @param g2 instance of the graphical canvas
   */
  private void drawTics(Graphics2D g2) {
    double x2,y2,x1, y1 = topMargin + axisLength[Y];
    
    // draw the X-tics in the lower two triangles:
    if (gp.drawTics(X)) {
      for (int i=1; i<numberOfTics[X]-1; i++) {
	x1 = leftMargin + sep[X]*i;
	x2 = x1-cos*ticLength[X];
	y2 = y1-sin*ticLength[X];
	line.setLine(x1,y1,x2,y2);
	g2.draw(line);
	x1 += triangleBottom+piperSep;
	x2 = x1+cos*ticLength[X];
	y2 = y1-sin*ticLength[X];
	line.setLine(x1,y1,x2,y2);
	g2.draw(line);
      }
    }
    // draw the Y-tics in the lower two triangles:
    if (gp.drawTics(Y)) {
      y1 = topMargin + axisLength[Y];
      for (int i=1; i<numberOfTics[Y]-1; i++) {
	x1 = leftMargin + sep[X]*i/2.0;
	x2 = x1 + ticLength[Y];
	y1 -= sep[Y];
	line.setLine(x1,y1,x2,y1);
	g2.draw(line);
	x1 = leftMargin + axisLength[X] - sep[X]*i/2.0;
	x2 = x1 - ticLength[Y];
	line.setLine(x1,y1,x2,y1);
	g2.draw(line);
      }
    }
  }

  /**
   * This function draws the grid.
   * @param g2 instance of the graphical canvas
   */
  private void drawGrid(Graphics2D g2) {
    double x2,y2,x1, y1 = topMargin + axisLength[Y];
    g2.setColor(gp.getGridColor());
    
    // draw the X-grid lines in the lower two triangles:
    if (gp.drawGrid(X) || gp.drawGrid(Y)) {
      for (int j=0; j<2; j++) {	
	double offset = (triangleBottom+piperSep)*j + leftMargin;
	for (int i=1; i<numberOfTics[X]-1; i++) {
	  x1 = offset + sep[X]*i;
	  x2 = x1+(triangleBottom-sep[X]*i)/2.0;
	  y2 = y1-(triangleHeight-sep[Y]*i);
	  if (gp.drawGrid(X)) {
	    line.setLine(x1,y1,x2,y2);
	    g2.draw(line);
	  }
	  x1 = x2-sep[X]*i;
	  if (gp.drawGrid(Y)) {
	    line.setLine(x2,y2,x1,y2);
	    g2.draw(line);
	  }
	  x2 = offset + triangleBottom - sep[X]*i;
	  if (gp.drawGrid(X)) {
	    line.setLine(x1,y2,x2,y1);
	    g2.draw(line);
	  }
	}
      }
    }

    if (gp.drawGrid(X)) {
      // draw the grid in the upper area:
      double sign=1.0;
      for (int j=0; j<2; j++) {
	y1 = topMargin + axisLength[Y] - normalSep;
	if (j == 1) sign = -1.0;
	for (int i=1; i<numberOfTics[X]-1; i++) {
	  x1 = leftMargin + triangleBottom + piperSep/2 - sign*sin30*sep[X]*i;
	  //x1 = leftMargin + triangleBottom + piperSep/2 - sign*sin*sep[X]*i;
	  y1 -= sep[Y];
	  x2 = x1 + sign*triangleBottom/2;
	  y2 = y1-triangleHeight;
	  line.setLine(x1,y1,x2,y2);
	  g2.draw(line);
	}
      }
    }
  }

  /*
   * Draws a colored, filled triangle in each triangle.
   * Used to quickly check whether the points are centered.
   */
  private void drawFilledTriangles(Graphics2D g2) {
    double xOffset;
    // first we build the two triangles at the bottom
    g2.setColor(gp.getInnerColor());
    for (int i=0; i<2; i++) {
      xOffset = leftMargin+i*(axisLength[X]+piperSep)/2.0;
      Polygon p = new Polygon();
      p.addPoint((int)(xOffset+triangleBottom/4.0),
		 (int)(topMargin+axisLength[Y]-triangleHeight/2.0));
      p.addPoint((int)(xOffset+3.0*triangleBottom/4.0),
		 (int)(topMargin+axisLength[Y]-triangleHeight/2.0));
      p.addPoint((int)(xOffset+triangleBottom/2.0),
		 (int)(topMargin+axisLength[Y]));
      g2.fill(p);
    }
    // next, the triangles in the upper one
    xOffset = leftMargin+(axisLength[X]-triangleBottom/2.0)/2.0;
    Polygon p = new Polygon();
    p.addPoint((int)(xOffset),
	       (int)(topMargin+axisLength[Y]-triangleHeight/2.0-normalSep));
    p.addPoint((int)(xOffset+triangleBottom/2.0),
	       (int)(topMargin+axisLength[Y]-triangleHeight/2.0-normalSep));
    p.addPoint((int)(xOffset),
	       (int)(topMargin+triangleHeight/2.0));
    p.addPoint((int)(xOffset+triangleBottom/2.0),
	       (int)(topMargin+triangleHeight/2.0));
    g2.fill(p);
  }

  protected boolean setMinMax(int axis, Vector data) {
    return true;
  }  

  /*
   * This function draws the axes (actually X,Y and mirror axes) and 
   * the tics on the axes.
   * @param g2 instance of a graphical canvas on which to draw the axes
   * @param background true if this function should draw only the
   * background color of the axis system.
   */
  private void plotAxes(Graphics2D g2, boolean background) {
    // draw the bounding box:
    double x = leftMargin;
    double y = axisLength[Y] + topMargin;
    
    // draw triangles:
    for (int i=0; i<2; i++) {
      Polygon p = new Polygon();
      int x1 = (int)(x + (triangleBottom + piperSep)*i);
      p.addPoint(x1,(int)y);
      p.addPoint(x1 + (int)triangleBottom,(int)y);
      p.addPoint(x1 + (int)triangleBottom/2,(int)(y-triangleHeight));
      if (background) {
	g2.setColor(gp.getGraphBackgroundColor());
	g2.fill(p);
      }
      else {
	g2.setColor(gp.getAxesColor());
	g2.draw(p);
      }
    }
    Polygon p = new Polygon();
    p.addPoint((int)(x+triangleBottom+piperSep/2),(int)(y-normalSep));
    p.addPoint((int)(x+(triangleBottom+piperSep)/2),
	       (int)(y-triangleHeight-normalSep));
    p.addPoint((int)(x+triangleBottom+piperSep/2),
	       (int)(topMargin));
    p.addPoint((int)(x+(3*triangleBottom+piperSep)/2),
	       (int)(y-triangleHeight-normalSep));
    if (background) {
      g2.setColor(gp.getGraphBackgroundColor());
      g2.fill(p);
    }
    else {
      g2.setColor(gp.getAxesColor());
      g2.draw(p);
      drawTics(g2);
    }
  }

  /*
   * Draw the labels around the x- and y-tics. The xLabelHeight is determined
   * as a function of one label-height (assuming the height is the same for
   * all the labels).
   * @param g2 instance of a 2D graphical canvas
   */
  private void plotTicLabels(Graphics2D g2) {
    float x, y;
    double a = topMargin + axisLength[Y];
    double f;
    
    // draw the X-axis labels:
    double xOffset = leftMargin;
    int k;
    if (gp.drawTicLabels(X)) {
      FontMetrics fm = getFontMetrics(gp.getTicFont(X));
      double xLabelHeight = fm.getHeight();
      g2.setColor(gp.getTicColor(X));
      g2.setFont(gp.getTicFont(X));
      for (int j=0; j<2; j++) {
	if (j>0) xOffset += triangleBottom + piperSep;
	for (int i=0; i<numberOfTics[X]; i++) {
	  if (j > 0) k = i;
	  else k = numberOfTics[X]-1 - i;
	  x = (float)(xOffset+sep[X]*i-fm.stringWidth(ticLabel[X][k])/2);
	  y = (float)(a+xLabelHeight);
	  g2.drawString(ticLabel[X][k],x,y);
	}
      }
    }
    
    // draw the Y axes labels:
    if (gp.drawTicLabels(Y)) {
      FontMetrics fm = getFontMetrics(gp.getTicFont(Y));
      double yLabelHeight = fm.getHeight();
      g2.setColor(gp.getTicColor(Y));
      g2.setFont(gp.getTicFont(Y));
      for (k=0; k<2; k++) {
	double yOffset=k*(triangleHeight + normalSep);
	xOffset = leftMargin + k*(triangleBottom+piperSep)/2.0;
	double sign=1.0;
	for (int j=0; j<2; j++) {
	  if (j == 1) {
	    sign = -1.0;
	    xOffset = leftMargin + axisLength[X] + 
	      k*sign*(triangleBottom+piperSep)/2.0;
	  }
	  for (int i=0; i<numberOfTics[Y]; i++) {
	    x = (float)(xOffset+(j-1)*
			fm.stringWidth(ticLabel[Y][i])-sign*xSep);
	    x += sign*sep[X]*i/2;
	    y = (float)(a-sep[Y]*i+yLabelHeight/4 - yOffset);
	    g2.drawString(ticLabel[Y][i],x,y);
	  }
	}
      }
    }
  }

  // dummy instance
  protected double toX(double x) {
    return x;
  }
  
  /**
   * Returns the X-value scaled to the pixel-availability.
   * This function takes the X-value and returns the corresponding
   * coordinates for the panel.
   * @param x real x-value (as introduced by the data)
   * @param y y-value in pixels
   * @return x-value scaled to the actual panel coordinates
   */
  protected double toX(double x, double y) {
    return leftMargin-triangleHeight*(y/gp.getMaxValue(Y))/tan +
      (gp.getMaxValue(X)-x)*triangleBottom/gp.getMaxValue(X);
  }

  /**
   * Returns the X-value scaled to the pixel-availability.
   * This function takes the X-value and returns the corresponding
   * coordinates for the panel.
   * @param x real x-value (as introduced by the data)
   * @param y y-value in pixels
   * @return x-value scaled to the actual panel coordinates
   */
  protected double toX2(double x, double y) {
    return leftMargin + axisLength[X] -
      triangleBottom + triangleHeight*(y/gp.getMaxValue(Y))/tan +
      x*triangleBottom/gp.getMaxValue(X);
  }

  /**
   * Returns the X-value scaled to the pixel-availability.
   * This function takes the X-value and returns the corresponding
   * coordinates the upper triangles.
   * @param x real x-value (as introduced by the data)
   * @param y y-value in pixels
   * @return x-value scaled to the actual panel coordinates
   */
  protected double toX3(double x, double y) {
    return leftMargin + axisLength[X]/2.0 + 
      cos*(y-x)*triangleSide/gp.getMaxValue(X);
  }

  /**
   * Returns the Y-value scaled to the pixel-availability.
   * This function takes the Y-value and returns the corresponding
   * coordinates for the lower triangles.
   * @param y real y-value (as introduced by the data)
   * @return y-value scaled to the actual panel coordinates
   */
  protected double toY(double y) {
    return topMargin + axisLength[Y] - triangleHeight*(y/gp.getMaxValue(Y));
  }

  /**
   * Returns the Y-value scaled to the pixel-availability.
   * This function takes the Y-value and returns the corresponding
   * coordinates for the upper part of the graph.
   * @param y real y-value (as introduced by the data)
   * @return y-value scaled to the actual panel coordinates
   */
  protected double toY3(double x, double y) {
    return topMargin + axisLength[Y] - normalSep -
      sin*(x+y)*triangleSide/gp.getMaxValue(Y);
  }

  /**
   * Fills the graph area with a background color. The
   * area is the area between the axes.
   * @param g2 graphics canvas
   */
  protected void fillGraphArea(Graphics2D g2) {
    plotAxes(g2,true);
  }

  /**
   * This function builds the graph in a double-buffered image zone.
   */
  protected void updateGraph() {
    Graphics2D g2  = createGraphics();
    double x, y;
    double a = topMargin + axisLength[Y];

    // draw the on the background, if this is what they want:
    if (gp.showInner()) drawFilledTriangles(g2);
    g2.setStroke(new BasicStroke());
    if (!gp.gridToFront()) drawGrid(g2);

    int i;
    for (Enumeration e=data.elements(); e.hasMoreElements();) {
      
      // get the next data array from the vector:
      DataArray da = (DataArray)e.nextElement();

      // set color and penwidth:
      g2.setColor(da.getColor());
      g2.setStroke(da.getStroke());
      x = toX(da.getX(0),da.getY(0));
      y = toY(da.getY(0));
      drawPointType(da.getSymbol(),g2,x,y,da.getSymbolSize());
      x = toX2(da.getX(1),da.getY(1));
      y = toY(da.getY(1));
      drawPointType(da.getSymbol(),g2,x,y,da.getSymbolSize());
      x = toX3(da.getX(0)+da.getY(0),da.getX(1)+da.getY(1));
      y = toY3(da.getX(0)+da.getY(0),da.getX(1)+da.getY(1));
      drawPointType(da.getSymbol(),g2,x,y,da.getSymbolSize());
      if (gp.drawTds() && da.size() > 2) {
	g2.setStroke(new BasicStroke());
	drawPointType(0,g2,x,y,(float)(gp.getTdsFactor()*
				       da.getSymbolSize()*da.getX(2)));
      }
    }
    g2.setStroke(new BasicStroke());

    // draw the on the foreground, if this is what they want:
    if (gp.gridToFront()) drawGrid(g2);

    // draw the axes and tics:
    plotAxes(g2,false);
    
    // draw the tic labels:
    plotTicLabels(g2);
    
    // plot the labels:
    plotLabels(g2);

    // draw the legend. Do this at the end because it must be
    // drawn on the top of all other drawing stuff (e.g. grid...)
    if (gp.drawLegend()) {
      int k=0;
      g2.setFont(gp.getLegendFont());
      for (Enumeration e=data.elements(); e.hasMoreElements(); k++) {
	DataArray da = (DataArray)e.nextElement();
	if (!da.drawLegend()) k--;
	else {
	  g2.setColor(da.getColor());
	  if (da.drawLine()) g2.setStroke(da.getStroke());
	  drawLegend(g2,da,k);
	}
      }
    }
    g2.setStroke(new BasicStroke());
    repaint();
  }

  /**
   * Checks whether x and y are within the ranges. The ranges are
   * defined by the axes system.
   * @param x x-point
   * @param y y-point
   */
  private boolean inRange(double x, double y) {
    if (x < leftMargin || x > leftMargin + axisLength[X] ||
	y < topMargin || y > topMargin + axisLength[Y]) return false;
    return true;
  }
}
