/*
 * JPLOT  -- Jan's Plotting Program for any programme or
 *           as an independent GUI
 *
 * JPLOT stands for Jan's Plotting Program.
 * 
 * Copyright (C) 1999-2001 by Jan van der Lee
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA.
 *
 * Send bugs, suggestions or queries to <jplot@cig.ensmp.fr>
 * The latest releases are found at 
 * http://www.cig.ensmp.fr/~vanderlee/jplot.html
 *
 * Initally developed for use by the Centre d'Informatique Geologique 
 * Ecole des Mines de Paris, Fontainebleau, France.
 */

package jplot;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.filechooser.*;
import javax.swing.event.*;

import java.io.File;
import java.io.PrintWriter;
import java.io.FileWriter;
import java.io.BufferedReader;
import java.io.FileReader;
import java.util.*;
import java.awt.*;
import java.awt.print.*;
import java.awt.event.*;
import java.io.IOException;
import java.net.*;

import javax.swing.plaf.metal.DefaultMetalTheme;
import javax.swing.plaf.metal.MetalLookAndFeel;

/**
 * JPLOT: a Jan's Plotting Program.
 * Developped for JCHESS, but extended for general use.
 *
 * BUGS/TODO:
 * - rescale the X-scale as well with some value
 * - put the different graph-styles in separate functions. Otherwise
 *   it is impossible to use different graph-styles together (e.g.,
 *   a histogram AND a line connecting the points).
 */
public class JPlot extends JPanel {

  public final static boolean debug = false;
  
   // dimensions of the app:
  public static int WIDTH = 520;
  public static int HEIGHT = 400;
  // the name of the app
  public final static String AppName = "JPlot";

  public JFrame frame;
  public JFrame plotFrame;

  public JMenuBar menubar;
  private static JPlot jp;
  private JPlot jplot;
  private JTabbedPane tabbedPane;
  private Vector plotPanels;
  private JToolBar toolBar;

  // vector with classes containing the data
  private Vector dataArrays = new Vector();

  // the graph
  private Graph graph;

  //Options graphOptions;
  private AxesPanel axesPanel;
  private ScalingPanel scalingPanel;
  private PiperOptions piperPanel;
  private SavePanel export;

  // the following panels are public since they may be
  // activated from the graph, which is displayed in an
  // external frame.
  public LabelPanel labelPanel;
  public LegendPanel legendPanel;

  private StyleChooser globalStyleChooser;

  private File scriptFile;   // filename of the script

  // list of font family names available on this platform
  // (must be initialized elsewhere):
  public static String[] fontNames = {"no fonts found"};

  // the following flag yields 'true' when all the classes
  // are fully loaded (they are on a seperate thread):
  public boolean loaded;
  private boolean hideGraph;

  private GraphPars gp;
  private JPanel mainPanel;

  // declared here, since these items may be dynamically
  // removed/inserted:
  private JMenu plotMenu;
  private JMenuItem scalingMenuItem, piperMenuItem;
  private JCheckBoxMenuItem mi_2D, mi_piper;
  private JCheckBoxMenuItem mi_java, mi_motif, mi_mac, mi_windows;

  private JFileChooser dataChooser;

  public static final String FS = System.getProperty("file.separator");

  // stand alone is true if JPlot runs on itself (started by main())
  private boolean isStandAlone=false;

  // insets of an empty panel:
  private final Insets insets = new Insets(0,0,0,0);

  // Possible Look & Feels
  private static final String mac = 
    "com.sun.java.swing.plaf.mac.MacLookAndFeel";
  private static final String metal = 
    "javax.swing.plaf.metal.MetalLookAndFeel";
  private static final String motif = 
    "com.sun.java.swing.plaf.motif.MotifLookAndFeel";
  private static final String windows = 
    "com.sun.java.swing.plaf.windows.WindowsLookAndFeel";
  static private String currentLookAndFeel;

  private int xPos = 450;
  private int yPos = 20;

  /**
   * Principal constructor.
   * Builds the panel with the plotting commands
   *
   * @param files vector of files containing the data arrays. This array
   * is the result of command-line filenames, and may be empty.
   */
  public JPlot(JFrame parent, Vector files, File script, boolean lonely) {
    setLayout(new BorderLayout());
    isStandAlone = lonely;
    loaded = false;
    plotFrame = null;
    hideGraph = false;
    frame = parent;
    jplot = this;
    scriptFile = script;

    // make the item in a item bar, but only if we
    // are running as a stand-alone application:
    //--------------------------------------------
    if (isStandAlone) {
      setBorder(new EtchedBorder());
      menubar = makeMenuBar();      
      add(menubar,BorderLayout.NORTH);
      gp = new GraphPars(parent);
      if (scriptFile != null) gp.setTitleName(scriptFile.toString());
      gp.setTitleString();
    }
    else gp = new GraphPars(null);

    mainPanel = new JPanel(new BorderLayout());
    toolBar = get2DToolBar();
    mainPanel.add(toolBar,BorderLayout.NORTH);
    graph = new Graph_2D(jplot,gp);

    // create the tabbed pane
    //-----------------------
    tabbedPane = new JTabbedPane();
    plotPanels = new Vector();
    tabbedPane.setFont(new Font("serif", Font.PLAIN, 12));
    mainPanel.add(tabbedPane, BorderLayout.CENTER);

    // add a Tab for the general plot pane
    //------------------------------------
    for (Enumeration e=files.elements(); e.hasMoreElements();) {
      File f = (File)e.nextElement();
      if (f != null && !f.toString().equals("")) insertDatafile(f,true);
    }

    // make the buttons show, reload etc.
    //-----------------------------------
    add(mainPanel,BorderLayout.CENTER);

    // pre-build a number of time-consuming instances
    // of dialogs using separate threads:
    //-----------------------------------------------
    Thread t = new Thread() {
      public void run() {
	GraphicsEnvironment env = GraphicsEnvironment.getLocalGraphicsEnvironment();
	if (fontNames.length == 1) {
	  fontNames = env.getAvailableFontFamilyNames();
	}
	legendPanel = new LegendPanel(jplot,gp);
	axesPanel = new AxesPanel(jplot,gp);
	labelPanel = new LabelPanel(jplot,gp);
	scalingPanel = new ScalingPanel(jplot,gp);
	piperPanel = new PiperOptions(jplot,gp);
	if (debug) System.out.println("Class loading ready.");
	loaded = true;
      }
    };
    t.start();
  }

  /**
   * Another constructor.
   * Builds the panel without initial files.
   */
  public JPlot(JFrame parent) {
    this(parent,new Vector(),null,false);
  }

  /**
   * Another constructor.
   * Builds the panel with files
   */
  public JPlot(JFrame parent, Vector files) {
    this(parent,files,null,false);
  }

  /**
   * Builds the main menubar
   *
   * @return an instance of a JMenuBar class
   */
  JMenuBar makeMenuBar() {
    JMenuItem mi;
    JMenuBar menuBar = new JMenuBar();

    // make the file-menu:
    JMenu file = (JMenu) menuBar.add(new JMenu("File"));
    file.setMnemonic('F');
    ImageIcon icon = getImageIcon("New16.gif");
    mi = (JMenuItem) file.add(new JMenuItem("New project",icon));
    mi.setMnemonic('N');
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	clear();
      }
    });
    icon = getImageIcon("Open16.gif");
    mi = (JMenuItem) file.add(new JMenuItem("Open project...",icon));
    mi.setMnemonic('O');
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	openScript();
      }
    });
    icon = getImageIcon("Print16.gif");
    mi = (JMenuItem) file.add(new JMenuItem("Print project...",icon));
    mi.setMnemonic('P');
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	printText();
      }
    });
    file.addSeparator();
    icon = getImageIcon("Export16.gif");
    mi = (JMenuItem) file.add(new JMenuItem("Export image...",icon));
    mi.setMnemonic('E');
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	exportImage();
      }
    });
    file.addSeparator();
    icon = getImageIcon("Save16.gif");
    mi = (JMenuItem) file.add(new JMenuItem("Save project",icon));
    mi.setMnemonic('S');
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	saveScript();
      }
    });
    icon = getImageIcon("SaveAs16.gif");
    mi = (JMenuItem) file.add(new JMenuItem("Save project as...",icon));
    mi.setMnemonic('A');
    file.addSeparator();
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	saveAsScript();
      }
    });

    file.addSeparator();
    mi = (JMenuItem) file.add(new JMenuItem("Quit"));
    mi.setMnemonic('Q');
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	quitDialog();
      }
    });

    // make the data-menu:
    //--------------------
    JMenu data = (JMenu) menuBar.add(new JMenu("Data"));
    data.setMnemonic('D');
    icon = getImageIcon("Add16.gif");
    mi = (JMenuItem) data.add(new JMenuItem("Load datafile",icon));
    mi.setMnemonic('L');
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	chooseDatafile();
      }
    });
    icon = getImageIcon("Remove16.gif");
    mi = (JMenuItem) data.add(new JMenuItem("Close datafile",icon));
    mi.setMnemonic('C');
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	removeDatafile();
      }
    });
    icon = getImageIcon("Preferences16.gif");
    mi = (JMenuItem) data.add(new JMenuItem("Global plotstyle",icon));
    mi.setMnemonic('G');
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	showGlobal();
      }
    });
    
    // make the Options menu:
    //-----------------------
    JMenu optionsMenu = (JMenu) menuBar.add(new JMenu("Options")); 
    optionsMenu.setMnemonic('O');
    ButtonGroup group = new ButtonGroup();
    mi_2D = new JCheckBoxMenuItem("2D graph");
    mi_2D.setSelected(true);
    optionsMenu.add(mi_2D);
    mi_2D.setMnemonic('2');
    mi_2D.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	switchGraphType(gp.GRAPHTYPE_2D);
      }
    });
    group.add(mi_2D);

    mi_piper = new JCheckBoxMenuItem("Piper graph");
    optionsMenu.add(mi_piper);
    //mi_diagram.setMnemonic('i');
    mi_piper.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	switchGraphType(gp.GRAPHTYPE_PIPER);
      }
    });
    group.add(mi_piper);

    // make the plot-menu:
    //--------------------
    plotMenu = (JMenu) menuBar.add(new JMenu("Plot")); 
    plotMenu.setMnemonic('P');
    mi = (JMenuItem) plotMenu.add(new JMenuItem("Show graph"));
    mi.setMnemonic('S');
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	showGraph(true);
      }
    });
    mi = (JMenuItem) plotMenu.add(new JMenuItem("Dismiss graph"));
    mi.setMnemonic('D');
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	dismiss();
      }
    });
    plotMenu.addSeparator();
    icon = getImageIcon("Print16.gif");
    mi = (JMenuItem) plotMenu.add(new JMenuItem("Print graph",icon));
    mi.setMnemonic('r');
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	printGraph();
      }
    });
    plotMenu.addSeparator();
    icon = getImageIcon("Axes16.png");
    mi = (JMenuItem) plotMenu.add(new JMenuItem("Axes",icon));
    mi.setMnemonic('A');
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	optionsAction(0);
      }
    });
    icon = getImageIcon("Scaling16.png");
    scalingMenuItem = (JMenuItem) plotMenu.add(new JMenuItem("Scaling",icon));
    scalingMenuItem.setMnemonic('c');
    scalingMenuItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	optionsAction(1);
      }
    });
    piperMenuItem = new JMenuItem("Piper");
    piperMenuItem.setMnemonic('i');
    piperMenuItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	optionsAction(4);
      }
    });
    icon = getImageIcon("Labels16.png");
    mi = (JMenuItem) plotMenu.add(new JMenuItem("Labels",icon));
    mi.setMnemonic('L');
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	optionsAction(2);
      }
    });
    icon = getImageIcon("Legend16.png");
    mi = (JMenuItem) plotMenu.add(new JMenuItem("Legend",icon));
    mi.setMnemonic('g');
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	optionsAction(3);
      }
    });

    // make the Options menu:
    //-----------------------
    JMenu prefMenu = (JMenu) menuBar.add(new JMenu("Preferences")); 
    prefMenu.setMnemonic('r');
    group = new ButtonGroup();
    mi_java = new JCheckBoxMenuItem("Java Look & Feel");
    prefMenu.add(mi_java);
    mi_java.setMnemonic('J');
    mi_java.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	updateLookAndFeel(metal);
      }
    });
    if (currentLookAndFeel.equals(metal)) mi_java.setSelected(true);
    mi_java.setEnabled(isAvailableLookAndFeel(metal));
    group.add(mi_java);

    mi_mac = new JCheckBoxMenuItem("Macintosh Look & Feel");
    prefMenu.add(mi_mac);
    mi_mac.setMnemonic('M');
    mi_mac.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	updateLookAndFeel(mac);
      }
    });
    mi_mac.setEnabled(isAvailableLookAndFeel(mac));
    group.add(mi_mac);

    mi_motif = new JCheckBoxMenuItem("Motif Look & Feel");
    prefMenu.add(mi_motif);
    mi_motif.setMnemonic('o');
    mi_motif.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	updateLookAndFeel(motif);
      }
    });
    mi_motif.setEnabled(isAvailableLookAndFeel(motif));
    group.add(mi_motif);

    mi_windows = new JCheckBoxMenuItem("Windows Look & Feel");
    prefMenu.add(mi_windows);
    mi_windows.setMnemonic('n');
    mi_windows.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	updateLookAndFeel(windows);
      }
    });
    mi_windows.setEnabled(isAvailableLookAndFeel(windows));
    if (currentLookAndFeel.equals(windows)) mi_windows.setSelected(true);
    group.add(mi_windows);

    return menuBar;
  }  

  /**
   * Update the graph if the graph is actually showing.
   * Also repains the plot panel because it shows linestyles.
   */
  public void updateGraphIfShowing() {
    if (plotFrame != null && plotPanels.size() > 0) {
      showGraph(true);
      int i = 0;
      if (plotPanels.size() > 1) i = tabbedPane.getSelectedIndex();
      ((PlotPanel)plotPanels.get(i)).update();
    }
    else showGraph(false);
  }

  /*
   * Action performed when the user selects the graph-options.
   * test whether the graphOptions instance is ready.
   */
  private void optionsAction(int k) {
    if (loaded) {
      int x = (int)frame.getLocation().getX() + 20;
      int y = (int)frame.getLocation().getY() + 20;
      if (k == 0) axesPanel.show(x,y);
      else if (k == 1) scalingPanel.show(frame,x,y);
      else if (k == 2) labelPanel.show(frame,x,y);
      else if (k == 3) legendPanel.show(frame,x,y);
      else if (k == 4) piperPanel.show(frame,x,y);
      else if (k == 5) showGlobal();
    }
  }

  /**
   * Returns an image which is found in a valid image URL. The basis
   * of the url is where 'jplot' is created.
   * @param name name of the image
   * @return an image or icon */
  public ImageIcon getImageIcon(String name) {
    ImageIcon im=null;
    try {
      URL imageURL = jplot.getClass().getResource("/images/" + name);
      Toolkit tk = Toolkit.getDefaultToolkit();
      im = new ImageIcon(tk.createImage(imageURL));
    }
    catch(Exception e) {
      Utils.oops(frame,"Impossible to load JPlot's icon '" + name + "'.\nSomething's wrong with the installation.");
    }
    return im;
  }

  /**
   * Switch from one graph type to another.  Calling this function
   * switches the GUI in a mode which is specific for a graph. It
   * should (dynamically) change the menu items, options, toolbar
   * etc.. The argument is a graph type, a symbolic constant
   * statically defined in gp.
   *
   * @param graphType type of the graph (i.e. PIPER, 2D...)  
   */
  public void switchGraphType(int graphType) {
    if (gp.getGraphType() == graphType) return;
    else {
      if (gp.dataChanged()) {
	int result = JOptionPane.showConfirmDialog(frame,"The current data set corresponds to a different graph-\ntype.  Loading the data will reset all the settings to\ntheir default values and they have not been saved.\n\nDo you want to save the settings now?","Warning",JOptionPane.YES_NO_CANCEL_OPTION,JOptionPane.INFORMATION_MESSAGE);
	if ((result == JOptionPane.YES_OPTION && !saveScript()) ||
	    result == JOptionPane.CANCEL_OPTION) return;
      }
      if (debug) System.out.println("switching graph type to " + graphType);
      gp.reset();
      gp.setGraphType(graphType);
      axesPanel = new AxesPanel(jplot,gp);
      labelPanel = new LabelPanel(jplot,gp);
      scalingPanel = new ScalingPanel(jplot,gp);
      if (toolBar != null) mainPanel.remove(toolBar);
      if (graphType == gp.GRAPHTYPE_2D) {
	toolBar = get2DToolBar();
	if (isStandAlone) {
	  plotMenu.remove(piperMenuItem);
	  plotMenu.add(scalingMenuItem);
	}
	graph = new Graph_2D(jplot,gp);
      }
      else if (graphType == gp.GRAPHTYPE_PIPER) {
	toolBar = getPiperToolBar();
	if (isStandAlone) {
	  plotMenu.remove(scalingMenuItem);
	  plotMenu.add(piperMenuItem);
	}
	graph = new Graph_Piper(jplot,gp);
      }
      mainPanel.add(toolBar,BorderLayout.NORTH);
      mainPanel.repaint();
    }
    if (isStandAlone) {
      mi_2D.setSelected(graphType == gp.GRAPHTYPE_2D);
      mi_piper.setSelected(graphType == gp.GRAPHTYPE_PIPER);
    }
  }

 /**
   * Builds the toolbar
   *
   * @return an instance of a JToolBar
   */
  JToolBar getToolBar() {
    JToolBar toolbar = new JToolBar();

    ImageIcon icon = getImageIcon("New24.gif");
    Action action = new AbstractAction("New",icon) {
	public void actionPerformed(ActionEvent e) {
	  if (JOptionPane.showConfirmDialog(jplot,"Clear the current system?\nAll settings will go to default!","",JOptionPane.YES_NO_OPTION) == 
	      JOptionPane.YES_OPTION) clear();
	}
      };
    SmallButton sb = new SmallButton(action,"Back to default settings");
    toolbar.add(sb);

    //ImageIcon icon = getImageIcon("newdata.png");
    icon = getImageIcon("Add24.gif");
    action = new AbstractAction("Load",icon) {
      public void actionPerformed(ActionEvent e) {
	chooseDatafile();
      }
    };
    sb = new SmallButton(action,"Load a new datafile");
    toolbar.add(sb);

    //icon = getImageIcon("jp_close.jpg");
    //icon = getImageIcon("fileclose.png");
    icon = getImageIcon("Remove24.png");
    action = new AbstractAction("Close",icon) {
      public void actionPerformed(ActionEvent e) {
	removeDatafile();
      }
    };
    sb = new SmallButton(action,"Close the current datafile");
    toolbar.add(sb);
    toolbar.addSeparator();
    return toolbar;
  }

  /*
   * Extends the default toolbar with buttons used by
   * the (default) 2D graph stuff
   * @param toolbar current toolbar
   */
  private JToolBar get2DToolBar() {
    JToolBar toolbar = getToolBar();
    ImageIcon icon = getImageIcon("graph.png");
    Action action = new AbstractAction("Show",icon) {
      public void actionPerformed(ActionEvent e) {
	showGraph(true);
      }
    };
    SmallButton sb = new SmallButton(action,"Show the current graph");
    toolbar.add(sb);
    toolbar.addSeparator();

    icon = getImageIcon("Axes24.png");
    action = new AbstractAction("Axes",icon) {
      public void actionPerformed(ActionEvent e) {
	optionsAction(0);
      }
    };
    sb = new SmallButton(action,"Axes parameters");
    toolbar.add(sb);

    icon = getImageIcon("Scaling24.png");
    action = new AbstractAction("Scaling",icon) {
      public void actionPerformed(ActionEvent e) {
	optionsAction(1);
      }
    };
    sb = new SmallButton(action,"Scaling");
    toolbar.add(sb);

    icon = getImageIcon("Labels24.png");
    action = new AbstractAction("Labels",icon) {
      public void actionPerformed(ActionEvent e) {
	optionsAction(2);
      }
    };
    sb = new SmallButton(action,"Labels");
    toolbar.add(sb);

    icon = getImageIcon("Legend24.png");
    action = new AbstractAction("Legend",icon) {
      public void actionPerformed(ActionEvent e) {
	optionsAction(3);
      }
    };
    sb = new SmallButton(action,"Legend options");
    toolbar.add(sb);

    toolbar.addSeparator();
    icon = getImageIcon("Preferences24.gif");
    action = new AbstractAction("Linestyle",icon) {
      public void actionPerformed(ActionEvent e) {
	optionsAction(5);
      }
    };
    sb = new SmallButton(action,"Global style preferences");
    toolbar.add(sb);
    return toolbar;
  }
  
  /*
   * Extends the default toolbar with buttons used by
   * the more exotic Piper diagram
   * @param toolbar current toolbar
   */
  private JToolBar getPiperToolBar() {
    JToolBar toolbar = getToolBar();
    ImageIcon icon = getImageIcon("graph_piper.png");
    Action action = new AbstractAction("Show",icon) {
      public void actionPerformed(ActionEvent e) {
	showGraph(true);
      }
    };
    SmallButton sb = new SmallButton(action,"Show the current graph");
    toolbar.add(sb);

    toolbar.addSeparator();
    icon = getImageIcon("Axes24.png");
    action = new AbstractAction("Axes",icon) {
      public void actionPerformed(ActionEvent e) {
	optionsAction(0);
      }
    };
    sb = new SmallButton(action,"Axes parameters");
    toolbar.add(sb);

    icon = getImageIcon("Scaling24.png");
    action = new AbstractAction("Axes",icon) {
      public void actionPerformed(ActionEvent e) {
	optionsAction(4);
      }
    };
    sb = new SmallButton(action,"Piper diagram options");
    toolbar.add(sb);

    icon = getImageIcon("Labels24.png");
    action = new AbstractAction("Labels",icon) {
      public void actionPerformed(ActionEvent e) {
	optionsAction(2);
      }
    };
    sb = new SmallButton(action,"Labels");
    toolbar.add(sb);

    icon = getImageIcon("Legend24.png");
    action = new AbstractAction("Legend",icon) {
      public void actionPerformed(ActionEvent e) {
	optionsAction(3);
      }
    };
    sb = new SmallButton(action,"Legend options");
    toolbar.add(sb);

    toolbar.addSeparator();
    icon = getImageIcon("Preferences24.gif");
    action = new AbstractAction("Linestyle",icon) {
      public void actionPerformed(ActionEvent e) {
	optionsAction(5);
      }
    };
    sb = new SmallButton(action,"Global style preferences");
    toolbar.add(sb);
    return toolbar;
  }
  
  /**
   * Reads a data file in memory.
   * We're supposing that the file is structured according to
   * X, Y1, Y2, ... etc. All lines starting with # are ignored.
   */
  private boolean parseDatafile() {
    boolean isPiper=(gp.getGraphType() == gp.GRAPHTYPE_PIPER);
    if (plotPanels.size() == 0) {
      Utils.oops(frame,"No datafile loaded yet, open one first.");
      return false;
    }
    double x=0.0;
    double[] y;
    dataArrays.removeAllElements();
    for (Enumeration e=plotPanels.elements(); e.hasMoreElements();) {
      DataFile df = ((PlotPanel)e.nextElement()).getDataFile();
      if (debug) System.out.println(df.getName() + "...");
      if (!df.getDataArrays(dataArrays,isPiper)) {
	return false;
      }
    }
    return true;
  }

  private void showGlobal() {
    if (plotPanels.size() > 0) {
      if (globalStyleChooser == null) {
	globalStyleChooser = new StyleChooser(jplot, "Global style-chooser",
					      true);
      }
      int i = tabbedPane.getSelectedIndex();
      PlotPanel pp = (PlotPanel)plotPanels.get(i);
      LinePars init = pp.getDataFile().getLinePars(0);
      LinePars lp = globalStyleChooser.show(350,100,init);
      if (lp != null) {
	gp.setDataChanged(lp.dataChanged());
	pp.getDataFile().setLinePars(lp);
	updateGraphIfShowing();
	pp.update();
      }
    }
    else {
      Utils.oops(frame,"Select global settings of what?\nLoad a dataset and make a selection first.");
    }
  }

  /**
   * Display the graph in a JFrame, using a separate thread.
   * The method has an argument which can be set to false if the 
   * graph shouldn't be shown in a frame. This is useful to make
   * the graph before hand, e.g. to pre-define the graph-parameters
   * or to print the graph.
   * @param b flag, true if the graph must be shown.
   * @return true if the graph could be drawn.
   */
  public boolean showGraph(boolean b) {
    if (debug) System.out.print("parsing datafile ");
    if (parseDatafile()) {
      if (dataArrays.size() > 0) {
	if (plotFrame == null) {
	  if (debug) {
	    System.out.println("going for " + dataArrays.size() + 
			       " data arrays");
	  }
	  if (b) {
	    plotFrame = new JFrame("JPlot");
	    plotFrame.setIconImage(getImageIcon("jplot16.png").getImage());
	    JPanel p = new JPanel(new BorderLayout());
	    p.add(graph,BorderLayout.CENTER);
	    JToolBar tb = new JToolBar();
	    ImageIcon icon;
	    Action action;
	    SmallButton sb;
	    
	    icon = getImageIcon("exit.jpg");
	    action = new AbstractAction("Close",icon) {
		public void actionPerformed(ActionEvent e) {
		  dismiss();
		}
	      };
	    sb = new SmallButton(action,"Close the graph");
	    tb.add(sb);
	    
	    icon = getImageIcon("Refresh24.gif");
	    action = new AbstractAction("Refresh",icon) {
		public void actionPerformed(ActionEvent e) {
		  graph.show(dataArrays);
		}
	      };
	    sb = new SmallButton(action,"Refresh the graph");
	    tb.add(sb);
	    tb.addSeparator();
	    
	    icon = getImageIcon("Export24.gif");
	    action = new AbstractAction("Export",icon) {
		public void actionPerformed(ActionEvent e) {
		  exportImage();
		}
	      };
	    sb = new SmallButton(action,"Export the graph");
	    tb.add(sb);

	    icon = getImageIcon("Print24.gif");
	    action = new AbstractAction("Print",icon) {
		public void actionPerformed(ActionEvent e) {
		  printGraph();
		}
	      };
	    sb = new SmallButton(action,"Print the current graph");
	    tb.add(sb);
	    
	    p.add(tb,BorderLayout.NORTH);
	    plotFrame.getContentPane().add(p);
	    plotFrame.addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent e) {
		  dismiss();
		}
	      });
	    plotFrame.setLocation(xPos,yPos);
	    plotFrame.pack();
	    plotFrame.show();
	    plotFrame.toFront();
	  }
	  graph.show(dataArrays);
	}
	else {
	  if (b) {
	    plotFrame.setState(Frame.NORMAL);
	    plotFrame.show();  // brings frame to the front
	  }
	  graph.show(dataArrays);
	}
	gp.setUpdateGraph(false);
      }
    }
    else return false;
    return true;
  }

  /**
   * Quits the application.
   * Asks for saving the script, if not yet done and allows
   * to cancel this drastic action.
   */
   public void quitDialog() {
    if (gp.dataChanged()) {
      int result = JOptionPane.showConfirmDialog(jplot,"The settings have changed, save project first?","",JOptionPane.YES_NO_CANCEL_OPTION);
      if (result == JOptionPane.YES_OPTION) {
	if (saveScript()) quit();
      }
      else if (result == JOptionPane.NO_OPTION) quit();
    }
    else quit();
  }

  /**
   * Quit. Exists if this is a stand alone instance. Otherwise we
   * clear the graph and dispose the frame (can be popped up later,
   * but without data and with default settings).
   */
  public void quit() {
    clear();
    frame.dispose();
    jplot = null;
    if (isStandAlone) System.exit(1);
  }

  /**
   * kills the graph frame, stops showing anything
   */
   public void dismiss() {
    if (plotFrame != null) {
      plotFrame.dispose();
      plotFrame = null;
    }
  }

  private void printText() {
    if (scriptFile == null) return;
    setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
    Thread t = new Thread() {
	public void run() {
	  PrinterJob job = PrinterJob.getPrinterJob();
	  if (job != null) {
	    PageFormat pf = new PageFormat();
	    PrintFilePainter pfp = new PrintFilePainter(scriptFile.toString());
	    job.setPrintable(pfp,pf);
	    job.setCopies(1);
	    if (job.printDialog()) {
	      try {job.print();}
	      catch (Exception e) {}
	    }
	  }
	}
      };
    t.start();
    setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
  }

  /*
   * Opens a print-dialog for printing the graph on a printer.
   */
  private void printGraph() {
    setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
    Thread t = new Thread() {
	public void run() {
	  try {
	    PrinterJob prnJob = PrinterJob.getPrinterJob();
	    prnJob.setPrintable(graph);
	    if (prnJob.printDialog()) {
	      prnJob.print();
	    }
	  }
	  catch (PrinterException e) {
	    e.printStackTrace();
	  }
	}
      };
    t.start();
    setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
  }

  /**
   * Read a new datafile in.
   * This function pops up a dialog and asks for a filename. If
   * the file is exists (and contains data), then we have to make
   * a new 'plot' tabbedPane, with this new data in it.
   */
  private void chooseDatafile() {
    if (dataChooser == null) {
      dataChooser = new JFileChooser();
      try {
	String path = new File(".").getCanonicalPath();
	dataChooser.setCurrentDirectory(new File(path));
      }
      catch (IOException ex) {}
    }
    FileFilter ff1 = new DataFileFilter1();
    FileFilter ff2 = new DataFileFilter2();
    dataChooser.addChoosableFileFilter(ff1);
    dataChooser.addChoosableFileFilter(ff2);
    dataChooser.setFileFilter(ff1);
    if (dataChooser.showDialog(jplot,"Select") == 0) {      
      insertDatafile(dataChooser.getSelectedFile(),true);
    }
  }

  private class DataFileFilter1 extends FileFilter {
    public boolean accept(File pathname) {
      return pathname.isDirectory() || 
	pathname.getName().endsWith(".res") ||
	pathname.getName().endsWith(".dat");
    }
    public String getDescription() {
      return "data files (*.res, *.dat)";
    }
  }

  private class DataFileFilter2 extends FileFilter {
    public boolean accept(File pathname) {
      return pathname.isDirectory() || 
	pathname.getName().endsWith(".ppr");
    }
    public String getDescription() {
      return "piper files (*.ppr)";
    }
  }

  /*
   * Read a new datafile in.
   * This function pops up a dialog and asks for a filename. If
   * the file is exists (and contains data), then we have to make
   * a new 'plot' tabbedPane, with this new data in it.
   */
  private boolean insertDatafile(File file, boolean initialize) {
    boolean res = true;
    if (!file.exists()) {
      Utils.oops(frame,"Impossible to open file " + file.toString() + 
		 " for reading!");
      res = false;
    }
    else {
      DataFile df = new DataFile(file,plotPanels.size(),gp);
      switchGraphType(df.getGraphType());
      String s = "data(" + (plotPanels.size()+1) + ")";
      PlotPanel pp = new PlotPanel(jplot,df,gp,initialize);
      tabbedPane.insertTab(s,null,pp,null,plotPanels.size());
      tabbedPane.setSelectedIndex(plotPanels.size());
      plotPanels.add(pp);
      
      // launch ones the graph-initialization stuff so all the
      // option panels are initialized with the current data:
      showGraph(false);
    }
    return res;
  }

  /**
   * Remove a datafile from the memory and the GUI.
   * Once you can insert datafiles to the GUI, you must be able to
   * remove them as well, i.e. remove from memory but also remove the tab
   * with the selection. And ask for confirmation before actually doing it.
   */
  void removeDatafile() {
    boolean ret = true;
    int nFiles = plotPanels.size();
    int i = tabbedPane.getSelectedIndex();
    if (i == -1) return;
    if (i > nFiles) {
      if (nFiles > 0) {
	Utils.oops(frame,"Select first a data panel for removal");
      }
      else Utils.oops(frame,"Nothing to remove, no data loaded");
    }
    else {
      String txt = "Close " + tabbedPane.getTitleAt(i) + ", " + 
	((PlotPanel)plotPanels.get(i)).getDataFile().getFilenameWithoutPath() + "?";
      int res = JOptionPane.showConfirmDialog(jplot,txt,"",
					      JOptionPane.YES_NO_OPTION);
      if (res == JOptionPane.YES_OPTION) {
	
	// erase the data from the system:
	//--------------------------------
	tabbedPane.removeTabAt(i);
	plotPanels.removeElementAt(i);
	for (int k=0; k<nFiles-1; k++) {
	  tabbedPane.setTitleAt(k,"data(" + (k+1) + ")");
	  ((PlotPanel)plotPanels.get(k)).setDataFileIndex(k);
	}
	if (plotFrame != null) {
	  if (nFiles-1 == 0) dismiss();
	  else updateGraphIfShowing();
	}
      }
    }
  }

  /**
   * Exports the image to some graphic format.
   */
  private void exportImage() {
    // TODO: check first if there is any graph around
    if (export == null) export = new SavePanel(graph);
    int x = (int)frame.getLocation().getX() + 20;
    int y = (int)frame.getLocation().getY() + 20;
    export.show(frame,x,y);
  }
  
  /**
   * Saves the script. If no script file is specified, it pops up
   * a file-chooser to ask for a valid file
   * @return false if something went wrong, true otherwise.
   */
  private boolean saveScript() {
    boolean res = true;
    if (scriptFile == null) {
      if (!saveAsScript()) res = false;
    }
    else if (!writeScript(scriptFile)) {
      res = false;
      scriptFile = null;
    }
    if (res) {
      gp.setDataChanged(false);
      if (scriptFile != null) gp.setTitleName(scriptFile.toString());
    }
    return res;
  }

  private class InputFileFilter extends FileFilter
  {
    public boolean accept(File pathname) {
      return pathname.isDirectory() || 
	pathname.getName().endsWith(".jplot");
    }
    public String getDescription() {
      return "JPlot scripts (*.jplot)";
    }
  }
    
  /**
   * Saves-as dialog. Pops up a dialog and asks for a valid
   * filename in order to save the current script.
   * @return false if the user canceled, true otherwise.
   */
  private boolean saveAsScript() {
    boolean ret = true;
    JFileChooser chooser = new JFileChooser(new File("."));
    FileFilter ff = new InputFileFilter();
    chooser.addChoosableFileFilter(ff);
    chooser.setFileFilter(ff);
    if (chooser.showDialog(jplot,"Save As") == 0) {
      scriptFile = chooser.getSelectedFile();
      if (scriptFile == null) return false;
      else if (scriptFile.exists()) {
	int res = JOptionPane.showConfirmDialog(jplot,
						"The file exist: do you want to overwrite the file?","",JOptionPane.YES_NO_OPTION);
	if (res == JOptionPane.NO_OPTION) ret = false;
      }
      if (ret) if (!writeScript(scriptFile)) ret = false;
    }
    return ret;
  }

  /**
   * Writes the script to a file.
   * @param f file to which to save the script
   * @return false if something went wrong, true otherwise.
   */
  private boolean writeScript(File f)
  {
    boolean res = true;
    try {
      PrintWriter pw = new PrintWriter(new FileWriter(f));

      // add first some local settings, such as the look&feel:
      pw.print("lookandfeel = ");
      if (currentLookAndFeel == metal) pw.print("java");
      else if (currentLookAndFeel == metal) pw.print("macintosh");
      else if (currentLookAndFeel == motif) pw.print("motif");
      else if (currentLookAndFeel == windows) pw.print("windows");
      else pw.print("unknown");
      pw.println();
      
      int nFiles = plotPanels.size();
      for (int k=0; k<nFiles; k++) {
	PlotPanel pp = (PlotPanel)plotPanels.get(k);
	pw.println(pp.getSettings());
      }
      pw.println(gp.getSettings());
      pw.close();
    }
    catch (IOException e) {
      Utils.bummer(frame,"It's somehow impossible to write to " + f.toString());
      res = false;
    }    
    return res;
  }

  /**
   * Writes the script to a string. This is useful if JPLOT is
   * used within other software, which asks to save the plot settings.
   * @return the settings in script format, in a string
   */
  public String getScript()
  {
    StringBuffer script = new StringBuffer(1000);
    int nFiles = plotPanels.size();
    for (int k=0; k<nFiles; k++) {
      PlotPanel pp = (PlotPanel)plotPanels.get(k);
      script.append(pp.getSettings());
    }
    script.append(Utils.lf).append(gp.getSettings());
    return script.toString();
  }

  /**
   * Opens a script, popping up a file-chooser to ask for a file
   */
  private void openScript() {
    JFileChooser chooser = new JFileChooser(new File("."));
    FileFilter ff = new InputFileFilter();
    chooser.addChoosableFileFilter(ff);
    chooser.setFileFilter(ff);
    if (chooser.showOpenDialog(jplot) == 0) {
      if (gp.dataChanged()) {
	int result = JOptionPane.showConfirmDialog(jplot,"Settings have changed, save the current project first?");
	if (result == JOptionPane.CANCEL_OPTION) return;
	else if (result == JOptionPane.YES_OPTION) if (!saveScript()) return;
      }
      scriptFile = chooser.getSelectedFile();
      if (scriptFile.exists()) loadScript();
      else scriptFile = null;
    }
  }

  /**
   * Load the script from a file in memory after heaving cleared the 
   * actual data. Should be threaded!
   */
  private void loadScript() {
    if (!scriptFile.exists() || !scriptFile.canRead()) {
      Utils.bummer(frame,"Can't read from " + scriptFile);
      return;
    }

    // read the script file and put all the stuff in a string,
    // which can be parsed by our parseScriptInString method.
    // It is this way since other models (JCHESS...) may pass
    // jplot's script in a string, not in a file.
    //-------------------------------------------------------
    StringBuffer sb = new StringBuffer(1000);
    try {
      BufferedReader in = new BufferedReader(new FileReader(scriptFile));
      String s;
      while ((s=in.readLine()) != null) {
	s = s.trim();
	//System.out.println("looking at " + s);

	// account for comments, which always start with #:
	if (s.length() == 0 || s.startsWith("#")) continue;

	// account for lines broken by a \ and continuing at the next line:
	while (s.endsWith("\\")) {
	  s = s.substring(0,s.length()-1);
	  s += in.readLine().trim();
	}

	if (s.startsWith("lookandfeel")) {
	  StringTokenizer st = new StringTokenizer(s," ,\t=");
	  st.nextToken();
	  String laf = st.nextToken();
	  if (laf.equals("java")) updateLookAndFeel(metal);
	  else if (laf.startsWith("mac")) updateLookAndFeel(mac);
	  else if (laf.startsWith("motif")) updateLookAndFeel(motif);
	  else if (laf.startsWith("windows")) updateLookAndFeel(windows);
	}
	else sb.append(s).append(";");
      }
      in.close();
    }
    catch (IOException e) {
      Utils.oops(frame,"There's a problem with " + scriptFile.toString());
      return;
    }

    // clear the current stuff:
    clear();
        
    // now we have the script in a string, all 'commands' are
    // separated by a ;. Parse this string.
    parseScriptInString(sb.toString());
    gp.setTitleName(scriptFile.toString());
    gp.setDataChanged(false);

    // showing the graph immediately after reading the script leads
    // to weird results, i.e. black screen etc.
    //showGraph(true);
  }

  /**
   * Parse a string which contains parameter settings.
   * This string contains keywords and values, separated by a ';'
   * @param str string containing the settings.
   */
  public void parseScriptInString(String str) {
    if (str == null || str.equals("")) return;
    StringTokenizer st = new StringTokenizer(str,";");
    LinePars lp = null;
    PlotPanel pp = null;
    int lpi = -1;
    boolean inDataFile = false;
    boolean inPlotStyle = false;
    while (st.hasMoreTokens()) {
      StringTokenizer s = new StringTokenizer(st.nextToken()," \t,");
      String t = s.nextToken();

      // parse a closing '}':
      if (t.equals("}")) {
	if (inPlotStyle) {
	  pp.getDataFile().addColumn(lpi,lp);
	  inPlotStyle = false;
	}
	else if (inDataFile) {
	  pp.update();
	  inDataFile = false;
	}
      }

      // parse a datafile block:
      else if (t.startsWith("da")) {
	inDataFile = true;
	if (!insertDatafile(new File(s.nextToken()),false)) break;
	pp = (PlotPanel)plotPanels.get(plotPanels.size()-1);
	pp.clearSelection();
      }

      // selection, i.e. 'selection = 1,2,5,8':
      else if (t.startsWith("se")) {
	s.nextToken();  // eat '='
	pp.getDataFile().setXColumn(Integer.parseInt(s.nextToken()));
      }
      // parse stuff like 'plotstyle of 1 {':
      else if (t.startsWith("pl")) {
	s.nextToken();  // eat 'of'
	lpi = Integer.parseInt(s.nextToken());
	lp = new LinePars();
	inPlotStyle = true;
      }
      // parse stuff like 'style = <graphstyle>'
      else if (t.startsWith("st")) {
	s.nextToken();  // eat '='	
	lp.setGraphStyle(Integer.parseInt(s.nextToken()));
      }
      // parse stuff like 'line = <dashlength>,<penwidth>'
      else if (t.startsWith("li")) {
	s.nextToken();  // eat '='
	int k = Integer.parseInt(s.nextToken());
	lp.setDashLength(gp.dashLengths[k],k);
	lp.setPenWidth(Float.parseFloat(s.nextToken()));
	lp.setDrawLine(Boolean.valueOf(s.nextToken()).booleanValue());
      }
      // parse stuff like 'point = <symbol>,<size>,<plotfreq>,<show>'
      else if (t.startsWith("po")) {
	s.nextToken();  // eat '='
	lp.setSymbol(Integer.parseInt(s.nextToken()));
	lp.setSymbolSize(Float.parseFloat(s.nextToken()));
	lp.setPointFrequency(Integer.parseInt(s.nextToken()));
	lp.setDrawSymbol(Boolean.valueOf(s.nextToken()).booleanValue());
      }
      // parse stuff like 'color = <red>,<green>,<blue>'
      else if (t.startsWith("co")) {
	s.nextToken();  // eat '='
	lp.setColor(new Color(Integer.parseInt(s.nextToken()),
			      Integer.parseInt(s.nextToken()),
			      Integer.parseInt(s.nextToken())));
      }
      // parse stuff like 'legend = <legend>,<show>'
      else if (t.startsWith("le")) {
	s.nextToken();  // eat '='
	String n = s.nextToken();
	if (n.startsWith("\"")) {  // is name between " and "
	  while (!n.endsWith("\"")) n += " " + s.nextToken();
	  n = n.substring(1,n.length()-1);
	}
	lp.setName(n);
	lp.setDrawLegend(Boolean.valueOf(s.nextToken()).booleanValue());
      }
      // parse stuff like 'scaling = <multiplier>,<additioner>'
      else if (t.startsWith("sc")) {
	s.nextToken();  // eat '='
	lp.setMultiplier(Float.parseFloat(s.nextToken()));
	lp.setAdditioner(Float.parseFloat(s.nextToken()));
      }
      else if (t.startsWith("graph")) break;
    }
    gp.updateSettings(str);
  }

  /**
   * Refreshes the actual graph with new data. Updates the graph only
   * if showing. This function first checks whether the file
   * 'filename' is already present in the system. If not, that's
   * simple, the file is added. If it is already present, then we must
   * replace the file, the difficulty is to preserve, as much as
   * possible, the former selection.
   * @param fn name of the file with data for JPlot.  */
  public void update(String fn) {
    if (fn.equals("") || fn == null) {
      clear();
      return;
    }
    boolean found = false;
    for (Enumeration e=plotPanels.elements(); e.hasMoreElements();) {
      PlotPanel pp = (PlotPanel) e.nextElement();
      DataFile df = pp.getDataFile();
      
      if (df.getName().equals(fn) || 
	  df.getFilenameWithoutPath().equals(fn)) {
	
	// datafile exists already. Now check if all the
	// column names are identical:
	//----------------------------------------------
	pp.reload();
	found = true;
	break;
      }
    }
    if (!found) insertDatafile(new File(fn),true);
    updateGraphIfShowing();
  }

  /**
   * Sets the location at which we put the graph
   * @param x x-position in pixels
   * @param y y-position in pixels
   */
  public void setGraphLocation(int x, int y) {
    xPos = x;
    yPos = y;
  }

  /**
   * Refreshes the actual graph with new data. This function calls
   * first 'update' which updates the graph if showing. If the 
   * graph isn't showing, we also popup the graph.
   * @param fn name of the file with data for JPlot.
   */
  public void show(String fn) {
    if (isStandAlone) {
      frame.setState(Frame.NORMAL);  // needed ?!! at least for 1.3.1...
      frame.show();
    }
    update(fn);
    if (plotFrame == null) showGraph(true);
  }

  /**
   * Clears the plot-panel, kills eventual plot frames
   */
  public void clear() {
    if (plotFrame != null) dismiss();

    if (plotPanels.size() > 0) plotPanels.removeAllElements();
    if (tabbedPane.getTabCount() > 0) tabbedPane.removeAll();

    /*
    for (Enumeration e=plotPanels.elements(); e.hasMoreElements();) {
      ((PlotPanel)plotPanels.get(0)).clearSelection();
      //plotPanels.removeElementAt(0);
      if (tabbedPane != null && tabbedPane.getTabCount() > 0) {
	tabbedPane.removeTabAt(0);
      }
    }
    */
    gp.reset();
    repaint();
  }

  /**
   * A utility function that layers on top of the LookAndFeel's
   * isSupportedLookAndFeel() method. Returns true if the LookAndFeel
   * is supported. Returns false if the LookAndFeel is not supported
   * and/or if there is any kind of error checking if the LookAndFeel
   * is supported.
   *
   * The L&F menu will use this method to detemine whether the various
   * L&F options should be active or inactive.
   */
  protected boolean isAvailableLookAndFeel(String laf) {
    try { 
      Class lnfClass = Class.forName(laf);
      LookAndFeel newLAF = (LookAndFeel)(lnfClass.newInstance());
      return newLAF.isSupportedLookAndFeel();
    } catch(Exception e) { // If ANYTHING weird happens, return false
      return false;
    }
  }

  /**
   * Sets the current L&F
   */
  public void updateLookAndFeel(String laf) {
    if (currentLookAndFeel != laf) {
      setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
      currentLookAndFeel = laf;
      try {
	UIManager.setLookAndFeel(currentLookAndFeel);
	SwingUtilities.updateComponentTreeUI(frame);
	Frame [] frames = frame.getFrames();
	for (int i=0; i<frames.length; i++) {
	  SwingUtilities.updateComponentTreeUI(frames[i]);
	}
      }
      catch (Exception ex) {
	String s;
	if (currentLookAndFeel.equals(mac)) s = "Macintosh Look & Feel";
	else if (currentLookAndFeel.equals(metal)) s = "Java Look & Feel";
	else if (currentLookAndFeel.equals(motif)) s = "Motif Look & Feel";
	else s = "Windows Look & Feel";
	Utils.oops(frame,"Couldn't load the " + s + ".\nIt's probably not available for this platform.");
	setCursor(Cursor.getDefaultCursor());
	return;
      }
      mi_mac.setSelected(false);
      mi_java.setSelected(false);
      mi_motif.setSelected(false);
      mi_windows.setSelected(false);
      if (currentLookAndFeel.equals(mac)) mi_mac.setSelected(true);
      else if (currentLookAndFeel.equals(windows)) mi_windows.setSelected(true);
      else if (currentLookAndFeel.equals(motif)) mi_motif.setSelected(true);
      else mi_java.setSelected(true);
      setCursor(Cursor.getDefaultCursor());
    }
  }

  /**
   * Main for using ChPlot as a stand-alone application. Takes care
   * of command line arguments:
   * -i <scriptfile>   : reads a configuration (script) file
   * Note: all arguments without a '-' are considered to be data files.
   */
  static public void main(String[] args) {

    // default look and feel defined by the platform. Avoid motif
    // as the default on unix... it's ugly.
    try {
      String lf = UIManager.getSystemLookAndFeelClassName();
      if (lf.equals(motif)) lf = metal;
      if (!lf.equals(metal)) UIManager.setLookAndFeel(lf);
      currentLookAndFeel = lf;
    }
    catch (Exception e) {
      System.out.println("Error loading L&F " + e);
    }

    File scriptFile=null;
    Vector files = new Vector();
    for (int i=0; i<args.length; i++) {
      if (args[i].startsWith("-")) {
	if (args[i].equals("-i")) {
	  scriptFile = new File(args[++i]);
	}
	else Utils.bummer(null,"Unknown option " + args[i]);
      } 
      else {
	File file = new File(args[i]);
	if (!file.exists()) {
	  Utils.oops(null,"File '" + file.toString() + "' wasn't found anywhere");
	  file = null;
	}
	files.add(file);
      }
    }
    JFrame frame = new JFrame("JPlot");
    jp = new JPlot(frame,files,null,true);
    frame.getContentPane().add(jp);
    frame.addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
	jp.quit();
      }
    });
    frame.setIconImage(jp.getImageIcon("jplot16.png").getImage());
    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    frame.setSize(WIDTH,HEIGHT);
    frame.setLocation(screenSize.width/2-WIDTH/2,screenSize.height/2-HEIGHT/2);
    frame.show();
  }
}
