/*
 * JPLOT  -- Java Plotting Interface for any programme or
 *           as an independent GUI
 * 
 * Copyright (C) 1999 Jan van der Lee
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA.
 *
 * Send bugs, suggestions or queries to <jplot@cig.ensmp.fr>
 * The latest releases are found at 
 * http://www.cig.ensmp.fr/~vanderlee/jplot.html
 *
 * Initally developed for use by the Centre d'Informatique Geologique 
 * Ecole des Mines de Paris, Fontainebleau, France.
 */

package jplot;

import java.awt.*;
import java.util.*;
import javax.swing.*;
import java.awt.event.*;
import javax.swing.border.*;

/**
 * This class pops up a panel with settings used by the labels.
 * Large part of the panel concerns the font and text color, other
 * parts define the exact location (x,y in pixels) and rotation of
 * the label.
 */
public class LabelProperties extends GriddedPanel {
  private JTextField labelText;
  private FontPanel fontPanel;
  private JCheckBox autoPos, manuPos;
  private JRadioButton hide;

  private JTextField xPosField, yPosField, rotField;

  JDialog dialog;
  boolean result;
  private JPlot jplot;
  int type; // type of the label, i.e., TITLE, XLABEL, YLABEL...

  //private Point point;
  //private double rotation;

  /**
   * Principal constructor. Builds a panel with widgets which
   * set the settings for one specific label.
   */
  public LabelProperties(JPlot jp) {

    jplot = jp;
    //point = new Point(0,0);
    //rotation = 0.0;
    
    // make the widgets:
    fontPanel = new FontPanel(jplot,Utils.getDefaultFont(),Color.black);
    
    // layout parameters:
    Dimension longField = new Dimension(180,22);
    Dimension shortField = new Dimension(80,22);
    Dimension cbField = new Dimension(110,20);
    EmptyBorder border = new EmptyBorder(new Insets(5,5,5,5));

    
    GriddedPanel labelPanel = new GriddedPanel();
    labelPanel.setBorder(new EtchedBorder()); 

    // lay out the label text:
    JLabel label = new JLabel("Label:");
    labelText = new JTextField();
    labelText.setPreferredSize(longField);
    label.setBorder(border);
    labelPanel.addComponent(label,1,1);
    labelPanel.addFilledComponent(labelText,1,2,2,1,GridBagConstraints.HORIZONTAL);
    //addFilledComponent(labelText,1,2);

    // rotation:
    label = new JLabel("Rotation:");
    label.setBorder(border);
    labelPanel.addComponent(label,2,1);
    rotField = new JTextField("0.0");
    rotField.setEnabled(true);
    rotField.setPreferredSize(shortField);
    labelPanel.addComponent(rotField,2,2);

    // put location stuff:
    label = new JLabel("Position:");
    label.setBorder(border);
    labelPanel.addComponent(label,3,1);

    ButtonGroup bg = new ButtonGroup();
    autoPos = new JCheckBox("automatic",true);
    autoPos.setPreferredSize(cbField);
    autoPos.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {	
	xPosField.setEnabled(false);
	yPosField.setEnabled(false);
      }
    });
    bg.add(autoPos);
    labelPanel.addComponent(autoPos,3,2);

    manuPos = new JCheckBox("manual",false);
    manuPos.setPreferredSize(cbField);
    manuPos.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {	
	xPosField.setEnabled(true);
	yPosField.setEnabled(true);
      }
    });
    bg.add(manuPos);
    labelPanel.addComponent(manuPos,3,3);

    JPanel posPanel = new JPanel(new BorderLayout());
    posPanel.add(new JLabel("x: "),BorderLayout.WEST);
    xPosField = new JTextField("0.0");
    xPosField.setEnabled(true);
    posPanel.add(xPosField,BorderLayout.CENTER);
    posPanel.setPreferredSize(shortField);
    labelPanel.addComponent(posPanel,4,2);

    posPanel = new JPanel(new BorderLayout());
    posPanel.add(new JLabel("y: "),BorderLayout.WEST);
    yPosField = new JTextField("0.0");
    yPosField.setEnabled(true);
    posPanel.add(yPosField,BorderLayout.CENTER);
    posPanel.setPreferredSize(shortField);
    labelPanel.addComponent(posPanel,4,3);

    hide = new JRadioButton("hide",false);
    hide.setPreferredSize(cbField);
    hide.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {	
      }
    });
    labelPanel.addComponent(hide,5,2);

    addComponent(labelPanel,1,1);

    // and add the font panel:
    addFilledComponent(fontPanel,2,1,3,1,GridBagConstraints.VERTICAL);
  }

  /**
   * @return the current properties of this label in terms of a new
   * graph label object.
   */
  public GraphLabel getGraphLabel() {
    GraphLabel gl = new GraphLabel(type,labelText.getText(),
				   fontPanel.getSelectedFont(),
				   fontPanel.getSelectedColor());
    gl.setRotation(Double.parseDouble(rotField.getText())*Math.PI/180);
    if (manuPos.isSelected()) {
      gl.setLocation(Double.parseDouble(xPosField.getText()),
		     Double.parseDouble(yPosField.getText()));
    }
    gl.setUsePosition(!autoPos.isSelected());
    return gl;
  }  
  
  /**
   * Updates the panel with a graph label object.
   * @param gl graph label object used to refresh this class.
   */
  void refresh(GraphLabel gl) {
    type = gl.getID();
    labelText.setText(gl.getText());
    xPosField.setText(Integer.toString((int)gl.getX()));
    yPosField.setText(Integer.toString((int)gl.getY()));
    rotField.setText(Graph.formatNumber(gl.getRotation()/Math.PI*180,4));
    autoPos.setSelected(!gl.usePosition());
    manuPos.setSelected(gl.usePosition());
    fontPanel.refresh(gl.getFont(),gl.getColor());
  }

  /**
   * Pops up a modal frame including the font panel. Returns the
   * actual font or null if the user pressed the cancel button.
   * @param parent parent frame
   * @param x x-position of the dialog
   * @param y y-position of the dialog
   * @return true if the user validated the current stuff, false otherwise
   */
  boolean show(Frame parent, int x, int y) {
    if (dialog == null) {
      JPanel panel = new JPanel(new BorderLayout());
      dialog = new JDialog(parent,"Textlabel settings",true);
      dialog.addWindowListener(new WindowAdapter() {
	public void windowClosing(WindowEvent e) {
	  result = false;
	  dialog.dispose();
	}
      });
      JPanel p = new JPanel(new FlowLayout());
      p.setBorder(BorderFactory.createEtchedBorder());
      JButton b = new JButton("Apply");
      b.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent e) {
	  result = true;
	  dialog.dispose();
	}
      });
      p.add(b);
      b = new JButton("Cancel");
      b.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent e) {
	  result = false;
	  dialog.dispose();
	}
      });
      p.add(b);
      panel.add(this,BorderLayout.CENTER);
      panel.add(p,BorderLayout.SOUTH);
      dialog.getContentPane().add(panel);
      dialog.setLocation(x,y);
      dialog.pack();
    }
    dialog.show();  // blocks until user brings dialog down.
    return result;
  } 
}
