/*
 * JPLOT  -- Java Plotting Interface for any programme or
 *           as an independent GUI
 * 
 * Copyright (C) 1999 Jan van der Lee
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA.
 *
 * Send bugs, suggestions or queries to <jplot@cig.ensmp.fr>
 * The latest releases are found at 
 * http://www.cig.ensmp.fr/~vanderlee/jplot.html
 *
 * Initally developed for use by the Centre d'Informatique Geologique 
 * Ecole des Mines de Paris, Fontainebleau, France.
 */

package jplot;

import java.awt.*;

/**
 * Contains all the data needed to plot a graph.
 * This data are parameters such as color, dashlength, point type etc.
 */
//public class LinePars extends DataChangeListener {
public class LinePars {

  // name of the dataset which uses these line parameters:
  String name;

  // different graph styles are defined here:
  static final int LINES=0;
  static final int HISTO=1;
  static final int STICKS=2;

  // general:
  protected Color color, startColor, endColor;
  protected int redInc, greenInc, blueInc;
  protected boolean slideColor;

  // points:
  protected int symbol;
  protected int every;
  protected float symbolSize;
  protected boolean useSymbol;
  protected float multiplier;
  protected float additioner;

  // lines:
  protected int colorIndex;
  protected int dashIndex;
  protected float dashLength;
  protected float penWidth;
  protected boolean plotLine;

  // styles:
  protected int style;
  protected boolean showLegend;  
  protected boolean dataModified;

  private final String lf = System.getProperty("line.separator");

  /**
   * Constructor, builds the class from default values.
   * @param s name of this dataset
   */
  public LinePars(String s) {
    colorIndex = 0;
    dashIndex = 0;
    color = GraphPars.color[0];
    dashLength = 0.0f;
    penWidth = 1.0f;
    symbol = GraphPars.NO_SYMBOL;
    symbolSize = 5.0f;
    useSymbol = true;
    plotLine = true;
    every = 1;
    style = LINES;
    showLegend = true;
    name = s;
    multiplier = 1.0f;
    additioner = 0.0f;
    slideColor = false;
    startColor = color;
    endColor = color;
    redInc = greenInc = blueInc = 0;
    dataModified = false;
  }

  /**
   * Constructor, builds the class with a specific color
   * @param colorIndex index pointing to a predefined (in 'ColorPanel') color.
   */
  public LinePars(String s, int colorIndex) {
    this(s);
    this.colorIndex = colorIndex;
    color = GraphPars.color[colorIndex];
  }

  /**
   * Constructor, builds the class from specified values.
   * @param c color
   * @param dl dash width of the line style (1.0f means no dash)
   * @param pw pen width (0 means no line)
   * @param s symbol (-1 means no symbols)
   * @param ss symbol size (0 means no symbols)
   */
  public LinePars(Color c, float dl, float pw, int s, float ss) {
    this("");
    color = c;
    colorIndex = -1;
    dashLength = dl;
    penWidth = pw;
    symbol = s;
    symbolSize = ss;
    style = LINES;
  }

  /**
   * Constructor, builds the class from default values.
   */
  public LinePars() {
    this("");
  }

  /**
   * Constructor, builds the class from specified values.
   * @param lp LinePars class instance
   */
  public LinePars(LinePars lp) {
    copy(lp);
  }

  /**
   * Constructor, builds the class from default values,
   * except for an initial color.
   * @param c initial color.
   */
  public LinePars(String s, Color c) {
    this(s);
    color = c;
  }

  /**
   * Copy operator, builds the class from specified values.
   * @param lp LinePars class instance
   */
  public void copy(LinePars lp) {
    color = lp.color;
    colorIndex = lp.colorIndex;
    dashIndex = lp.dashIndex;
    dashLength = lp.dashLength;
    penWidth = lp.penWidth;
    symbol = lp.symbol;
    symbolSize = lp.symbolSize;
    useSymbol = lp.useSymbol;
    plotLine = lp.plotLine;
    style = lp.style;
    every = lp.every;
    name = lp.name;
    showLegend = lp.showLegend;
    multiplier = lp.multiplier;
    additioner = lp.additioner;
    redInc = lp.redInc;
    greenInc = lp.greenInc;
    blueInc = lp.blueInc;
    slideColor = lp.slideColor;
    startColor = lp.startColor;
    endColor = lp.endColor;
    dataModified = lp.dataModified;
  }

  /**
   * Set the name of this dataset.
   * @param s string which contains the name of the dataset
   */
  void setName(String s) {
    name = s;
  }
  

  /**
   * Returns the name (legend) of this line style
   * @return name
   */
  public String getName() {
    return name;
  }

  /**
   * Returns true if this linestyle should draw the name
   * in the legend of the graph, false otherwise
   * @return true if the name should be shown
   */
  public boolean drawLegend() {
    return showLegend;
  }

  /**
   * Sets whether or not this linestyle should draw the name
   * in the legend of the graph.
   * @param b boolean, true if the name should be shown
   */
  public void setDrawLegend(boolean b) {
    showLegend = b;
  }

  /**
   * Returns the color of the pen.
   * @return color
   */
  public Color getColor() {
    return color;
  }

  /**
   * Returns the start color of the pen. Only useful if
   * the sliding color option is on.
   * @return start color
   */
  public Color getStartColor() {
    return startColor;
  }

  /**
   * Returns the end color of the pen. Only useful if
   * the sliding color option is on.
   * @return end color
   */
  public Color getEndColor() {
    return endColor;
  }

  /**
   * Enabled and initializes the sliding color option. After calling
   * this function, the line color is progressively changed towards
   * the end color through successive calls to 'nextColor'.
   * @param start start color
   * @param end end color
   */
  public void setSlideColor(Color start, Color end) {
    color = start;
    startColor = start;
    endColor = end;
    redInc = end.getRed() - start.getRed();
    greenInc = end.getGreen() - start.getGreen();
    blueInc = end.getBlue() - start.getBlue();
    slideColor = true;
  }

  /**
   * Activates the slide color option.
   * @param b true if we should slide the color
   */
  public void setSlideColor(boolean b) {
    slideColor = b;
  }

  /**
   * @return true if the sliding option is activated
   */
  public boolean slideColor() {
    return slideColor;
  }

  /**
   * Sets the color of the pen. This function is
   * called if successive calls to this class should progressively
   * change the color of the style.
   */
  public void nextColor(int step) {
    color = new Color(color.getRed()+(int)(redInc/step),
		      color.getGreen()+(int)(greenInc/step),
		      color.getBlue()+(int)(blueInc/step));
  }

  /**
   * Returns the index of the color
   * @return color index
   */
  public int getColorIndex() {
    return colorIndex;
  }

  /**
   * Returns the dash length of the pen
   * @return dash length
   */
  public float getDashLength() {
    return dashLength;
  }

  /**
   * Returns the array index of the current dash length
   * @return index to dash length
   */
  public int getDashIndex() {
    return dashIndex;
  }

  /**
   * Return the width of the lines
   * @return pen width
   */
  public float getPenWidth() {
    return penWidth;
  }

  /**
   * Returns the symbol type (symbolic constant)
   * @return the symbol type
   */
  public int getSymbol() {
    return symbol;
  }

  /**
   * Sets the multiplier for this line, i.e. the value which
   * will be used to multiply each value of the dataset before plotting.
   * @param m the multiplier
   */
  public void setMultiplier(float m) {
    multiplier = m;
  }

  /**
   * Returns the multiplier for this line, i.e. the value which
   * will be used to multiply each value of the dataset before plotting.
   * @return the multiplier
   */
  public float getMultiplier() {
    return multiplier;
  }

  /**
   * Sets the additioner for this line, i.e. the value which
   * will be used to increase each value of the dataset before plotting.
   * @param m the multiplier
   */
  public void setAdditioner(float m) {
    additioner = m;
  }

  /**
   * Returns the additioner for this line, i.e. the value which
   * will be used to increase each value of the dataset before plotting.
   * @return the multiplier
   */
  public float getAdditioner() {
    return additioner;
  }

  /**
   * Returns whether or not drawing the symbol
   * @return whether or not drawing the symbol
   */
  public boolean drawSymbol() {
    return useSymbol;
  }

  /**
   * Returns whether or not drawing the lines
   * @return whether or not drawing the lines
   */
  public boolean drawLine() {
    return plotLine;
  }

  /**
   * Returns the size of the symbol in points
   * @return the symbol size in points
   */
  public float getSymbolSize() {
    return symbolSize;
  }

  /**
   * Sets the color of the pen.
   * @param c color
   */
  public void setColor(Color c) {
    color = c;
  }

  /**
   * Sets the color of the pen from the swatch panel.
   * @param i index of the color on the panel.
   */
  public void setColor(int i) {
    color = GraphPars.color[i];
    colorIndex = i;
  }

  /**
   * Sets the dash length of the pen
   * @param dl dash length
   */
  public void setDashLength(float dl) {
    if (dl < 0.0f) {
      dashLength = 0.0f;
      plotLine = false;
    }
    else dashLength = dl;
  }

  /**
   * Sets the dash length of the pen.
   * In addition, set the index of this dash type as presented by the chooser.
   * @param dl dash length
   * @param index index
   */
  public void setDashLength(float dl, int index) {
    setDashLength(dl);
    dashIndex = index;
  }

  /**
   * Sets the width of the lines
   * @param pw pen width
   */
  public void setPenWidth(float pw) {
    penWidth = pw;
  }

  /**
   * Sets the symbol type (symbolic constant)
   * @param s the symbol type
   */
  public void setSymbol(int s) {
    symbol = s;
  }

  /**
   * Sets whether or not drawing the symbol
   * @param b toggle
   */
  public void setDrawSymbol(boolean b) {
    useSymbol = b;
  }

  /**
   * Sets whether or not drawing the line
   * @param b toggle
   */
  public void setDrawLine(boolean b) {
    plotLine = b;
  }

  /**
   * Sets the size of the symbol in points
   * @param ss the symbol size in points
   */
  public void setSymbolSize(float ss) {
    symbolSize = ss;
  }

  /**
   * Sets the frequency used to draw points in terms of an 'every'
   * parameter (every 2 data-point, every 200 data point...). 
   * By default, the program draws a point for every X,Y pair 
   * from the dataset.
   * @param k each point which should be drawn.
   */
  public void setPointFrequency(int k) {
    every = k;
  }

  /**
   * Returns the frequency used to draw points in terms of an 'every'
   * parameter.
   * @return the point frequency (1=every point, 2=every second point...)
   */
  public int getPointFrequency() {
    return every;
  }

  /** 
   * Sets the graph style. The integer is an symbolic constant,
   * must be one of the constants defined in the GraphPars class 
   * (with values such as LINES, HISTO, etc.).
   * @param s line style (symbolic constant)
   */
  void setGraphStyle(int s) {
    style = s;
  }

  /** 
   * Returns the current graph style.
   * @return the current graph style.
   */
  int getGraphStyle() {
    return style;
  }

  /**
   * Returns the style settings in a string.
   * This function is called when writing a script file of the current
   * settings.
   */
  public String getSettings() {
    StringBuffer sb = new StringBuffer(1000);
    sb.append("      style = ").append(style).append(lf).append("      ");
    sb.append("line = ").append(dashIndex).append(",");
    sb.append(penWidth).append(",").append(plotLine);
    sb.append(lf).append("      ");
    sb.append("point = ").append(symbol).append(",").append(symbolSize);
    sb.append(",").append(every).append(",").append(useSymbol);
    sb.append(lf).append("      ");
    sb.append("color = ").append(color.getRed()).append(",");
    sb.append(color.getGreen()).append(",").append(color.getBlue());
    sb.append(lf).append("      ");
    sb.append("legend = \"").append(name).append("\",").append(showLegend);
    sb.append(lf).append("      ");
    sb.append("scaling = ").append(multiplier).append(",");
    sb.append(additioner).append(lf);
    return sb.toString();
  }

  /**
   * @return true if some of the graph data has been modified
   */
  public boolean dataChanged() {
    return dataModified;
  }

  /**
   * Sets to true if the data has been changed by the user.
   * @param b true if the data has been changed
   */
  public void setDataChanged(boolean b) {
    dataModified = b;
  }
}
