/*
 * JPLOT  -- Java Plotting Interface for any programme or
 *           as an independent GUI
 * 
 * Copyright (C) 1999 Jan van der Lee
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA.
 *
 * Send bugs, suggestions or queries to <jplot@cig.ensmp.fr>
 * The latest releases are found at 
 * http://www.cig.ensmp.fr/~vanderlee/jplot.html
 *
 * Initally developed for use by the Centre d'Informatique Geologique 
 * Ecole des Mines de Paris, Fontainebleau, France.
 */

package jplot;

import java.awt.*;
import java.util.*;
import javax.swing.*;
import java.awt.geom.*;
import java.awt.event.*;
import javax.swing.border.*;

/**
 * Here we create a panel which allows a user to modify several
 * parameters specific to piper diagrams.
 * Calling the <code>show</code> method pops up a frame (dialog) which
 * includes the panel.
 */
public class PiperOptions extends JPanel {

  private JDialog dialog;

  private JCheckBox cb_showTds, cb_hideTds;
  private JTextField scalingField;
  private GraphPars gp;
  private JPlot jplot;
  
  /**
   * Principal constructor. Builds a panel with Piper diagram options.
   * @param gp instance of the object containing all graphical parameters
   */
  public PiperOptions (JPlot jp, GraphPars _gp) {
    gp = _gp;
    jplot = jp;
    dialog = null;
    //setLayout(new BoxLayout(this,BoxLayout.Y_AXIS));

    GriddedPanel p = new GriddedPanel();
    p.setBorder(new EmptyBorder(new Insets(5,5,5,5)));

    Dimension cbField = new Dimension(110,20);
    //Dimension shortField = new Dimension(80,22);

    ButtonGroup bg = new ButtonGroup();
    cb_showTds = new JCheckBox("Show");
    bg.add(cb_showTds);
    cb_hideTds = new JCheckBox("Hide");
    bg.add(cb_hideTds);
    scalingField = new JTextField();
    scalingField.setColumns(6);
    //scalingField.setPreferredSize(shortField);
    JPanel scalingPanel = new JPanel(new BorderLayout());
    scalingPanel.add(Utils.makeLabel("scaling"),BorderLayout.WEST);
    scalingPanel.add(scalingField,BorderLayout.CENTER);

    // spacing between labels and fields:
    EmptyBorder border = new EmptyBorder(new Insets(0,0,0,10));

    // place the widgets in the panel:
    JLabel label = new JLabel("Total Dissolved Solids:");
    label.setBorder(border);
    p.addComponent(label,1,1);
    p.addComponent(cb_showTds,1,2);
    p.addComponent(cb_hideTds,1,3);
    p.addComponent(scalingPanel,2,2);
    add(p);
  }

  /**
   * Updates the panel with the current graphical parameters.
   */
  public void refresh() {
    cb_showTds.setSelected(gp.drawTds());
    cb_hideTds.setSelected(!gp.drawTds());
    scalingField.setText(Float.toString(gp.getTdsFactor()));
  }

  public void setValues() {
    gp.setDrawTds(cb_showTds.isSelected());
    gp.setTdsFactor(Float.parseFloat(scalingField.getText()));
  }

 /**
   * Return a modal JDialog.
   * @param parent frame which is the parent of this dialog.
   * @param x x-position of the upper left corner
   * @param y y-position of the upper left corner
   */
  public void show(Frame parent, int x, int y) {
    if (dialog == null) {
      dialog = new JDialog(parent,"Piper diagram options",true);
      dialog.addWindowListener(new WindowAdapter() {
	public void windowClosing(WindowEvent e) {
	  dialog.dispose();
	}
      });

      JPanel p = new JPanel(new FlowLayout());
      p.setBorder(BorderFactory.createEtchedBorder());
      JButton b = new JButton("Apply");
      b.addActionListener(new ActionListener() {
	  public void actionPerformed(ActionEvent e) {
	    setValues();
	    gp.setDataChanged(true);
	    jplot.updateGraphIfShowing();
	    dialog.dispose();
	  }
	});
      p.add(b);
      b = new JButton("Dismiss");
      b.addActionListener(new ActionListener() {
	  public void actionPerformed(ActionEvent e) {
	    dialog.dispose();
	  }
	});
      p.add(b);
      dialog.getContentPane().add(this,BorderLayout.CENTER);
      dialog.getContentPane().add(p,BorderLayout.SOUTH);
      dialog.setLocation(x,y);
      dialog.pack();
    }
    refresh();
    dialog.show();  // blocks until user brings dialog down.
  }
}









