/*
 * JPLOT  -- Java Plotting Interface for any programme or
 *           as an independent GUI
 * 
 * Copyright (C) 1999 Jan van der Lee
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA.
 *
 * Send bugs, suggestions or queries to <jplot@cig.ensmp.fr>
 * The latest releases are found at 
 * http://www.cig.ensmp.fr/~vanderlee/jplot.html
 *
 * Initally developed for use by the Centre d'Informatique Geologique 
 * Ecole des Mines de Paris, Fontainebleau, France.
 */

package jplot;

import javax.swing.*;
import javax.swing.table.*;
import javax.swing.border.*;
import javax.swing.text.*;

import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;

/** 
 * Builds the panel with item lists, to select columns, styles etc.
 * 
 * @author  : Jan van der Lee <vanderlee@cig.ensmp.fr>
 * @created : Fri Aug 14 18:08:08 MET DST 1998 on asterix
 * @version : 0.1 - revision 1
 * @modified: Fri Aug 21 1998
 */

public class PlotPanel extends JPanel {

  private Object[][] itemDataMatrix;
  private JTable itemTable;
  private TableModel itemDataModel;

  private Object[][] selectedDataMatrix;
  private JTable selectedTable;
  private TableModel selectedDataModel;
  //private MyTableModel selectedDataModel;
  private boolean selectedInFocus;

  private Object[][] selectedXDataMatrix;
  private JTable selectedXTable;
  private TableModel selectedXDataModel;
  private boolean selectedXInFocus;

  //private Vector itemVector;
  //private JList itemList;
  private JScrollPane items;
  
  //private Vector selectedVector;
  //private JList selectedList;
  private JScrollPane selected;

  //private int[] allIndices;
  //private String listWidth = "aaaaaa";

  private StyleChooser styleChooser;
  
  private File file;
  private DataFile dataFile;
  private GraphPars gp;
  private JPlot jplot;

  private SimpleEditor dataViewer;

  private JPopupMenu selectedXPopup, selectedYPopup, itemsPopup;
  private final String lf = System.getProperty("line.separator");
  
  /**
   * Principal constructor.
   * Builds the selector panel with plotting choices.
   *
   * @param jp JPlot instance
   * @param df the datafile object, contains the file for the plot
   *
   * @author Jan van der Lee <vanderlee@cig.ensmp.fr>
   */
  public PlotPanel(JPlot jp, DataFile df, GraphPars gp, boolean initialize) {
    dataFile = df;
    jplot = jp;
    this.gp = gp;
    file = dataFile.getFile();
    JPanel p,q,r,main;
    JButton b;
    setLayout(new BorderLayout());
    EmptyBorder eb = new EmptyBorder(4,4,4,4);
    BevelBorder bb = new BevelBorder(BevelBorder.LOWERED);
    setBorder(new CompoundBorder(eb,bb));
    p = new JPanel(new BorderLayout());
    p.setBorder(new EtchedBorder());
    JLabel fn = new JLabel(dataFile.getAbbreviatedFilename());
    fn.setForeground(Color.blue);
    p.add(fn,BorderLayout.WEST);

    JPanel buttons = new JPanel();
    Action action = new AbstractAction("Reload",
				       jplot.getImageIcon("Refresh16.gif")) {
      public void actionPerformed(ActionEvent e) {
	reload();
      }
    };
    SmallButton sb = new SmallButton(action,"Reload this dataset from file");
    buttons.add(sb);
    action = new AbstractAction("Edit datafile",
				jplot.getImageIcon("Open16.gif")) {
      public void actionPerformed(ActionEvent e) {
	viewData();
      }
    };
    sb = new SmallButton(action,"View or edit the current dataset");
    buttons.add(sb);

    p.add(buttons,BorderLayout.EAST);
    add(p,BorderLayout.NORTH);
    selectedInFocus = selectedXInFocus = false;

    main = new JPanel(new GridLayout(1,2));
    main.add(getItemPanel());
    main.add(getSelectedPanel());
    defaultSelection(initialize);
    add(main,BorderLayout.CENTER);
  }

  /*
   * Returns the index of the next color of the vector of
   * predefined colors.
   */
  private int getColorIndex() {
    if (gp.colorIndex == gp.color.length) gp.colorIndex = 0;
    return gp.colorIndex++;
  }

  /*
   * Returns the index of the next point type of the vector of
   * predefined point types. Note that the last point type is not
   * used, since this is only a very small dot.
   */
  private int getPointIndex() {
    if (gp.pointIndex == gp.NO_SYMBOL-1) gp.pointIndex = 0;
    return gp.pointIndex++;
  }

  /*
   * Returns a panel with a table of choices.
   * @return a panel with a table of choices.
   */
  private JPanel getItemPanel() {

    // make a table with the choices:
    final String[] names = {"item"};
    
    // Create the data from the data-class:
    itemDataMatrix = new Object [0][1];
    
    // Create a model of the data.
    itemDataModel = new AbstractTableModel() {
      public int getColumnCount() {return names.length;}
      public int getRowCount() {return itemDataMatrix.length;}
      public Object getValueAt(int row, int col) {return itemDataMatrix[row][col];}
      public String getColumnName(int column) {return names[column];}
    };

    // build a popup menu for the selected X table:
    itemsPopup = new JPopupMenu();
    JMenuItem mi;
    mi = (JMenuItem) itemsPopup.add(new JMenuItem("Set as X"));
    mi.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent e) {
	  int index = itemTable.getSelectedRow();
	  if (index > -1 && index < itemTable.getRowCount()) {
	    if (selectedXDataMatrix.length == 0) {
	      selectedXDataMatrix = new Object[1][1];
	    }
	    selectedXDataMatrix[0][0] = dataFile.getItem(index);
	    dataFile.setXColumn(index);
	    selectedXTable.revalidate();
	    selectedXTable.repaint();
	  }
	}
      });
    mi = (JMenuItem) itemsPopup.add(new JMenuItem("Set as Y"));
    mi.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent e) {
	  int[] index = itemTable.getSelectedRows();
	  for (int j=0; j<index.length; j++) {
	    if (index[j] >= 0) addSelectedItem(index[j]);
	  }
	  selectedTable.revalidate();
	}
      });
        
    // Create the table
    itemTable = new JTable(itemDataModel);
    itemTable.setColumnSelectionAllowed(false);
    itemTable.setSelectionBackground(new Color(248,248,215));
    itemTable.setSelectionForeground(Color.black);
    itemTable.getColumn("item").setPreferredWidth(30);
    itemTable.addMouseListener(new MouseAdapter() {
	public void mousePressed(MouseEvent e) {
	  if (e.isPopupTrigger() || 
	     e.getModifiers() == InputEvent.BUTTON3_MASK) {
	    itemsPopup.show(itemTable,e.getX(),e.getY());
	  }
	}
	public void mouseClicked(MouseEvent e) {
	  if (e.getClickCount() == 2) {
	    int index = itemTable.getSelectedRow();
	    addSelectedItem(index);
	    selectedTable.revalidate();
	  }
	}
      });
    
    // put the stuff in panels and stuff buttons below them to
    // selected items as X or Y columns:
    //--------------------------------------------------------
    items = new JScrollPane(itemTable);
    JPanel p = new JPanel(new BorderLayout());
    p.setBorder(new TitledBorder(new EtchedBorder(),"Choices"));
    p.add(items,BorderLayout.CENTER);
    JPanel q = new JPanel(new FlowLayout());
    JButton b;
    if (gp.getGraphType() != gp.GRAPHTYPE_PIPER) {
      b = new JButton("Set as X");
      b.setToolTipText("Set the current selected column as X-axis");
      b.addActionListener(new ActionListener() {
	  public void actionPerformed(ActionEvent e) {
	    int index = itemTable.getSelectedRow();
	    if (selectedXDataMatrix.length == 0) {
	      selectedXDataMatrix = new Object[1][1];
	    }
	    selectedXDataMatrix[0][0] = dataFile.getItem(index);
	    dataFile.setXColumn(index);
	    selectedXTable.revalidate();
	    selectedXTable.repaint();
	  }
	});
      q.add(b);
      b = new JButton("Set as Y");
    }
    else b = new JButton("Select");

    b.setToolTipText("Set the current selected column as Y-axis");
    b.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	int[] index = itemTable.getSelectedRows();
	for (int j=0; j<index.length; j++) {
	  if (index[j] >= 0) addSelectedItem(index[j]);
	}
	selectedTable.revalidate();
      }
    });
    q.add(b);
    p.add(q,BorderLayout.SOUTH);
    return p;
  }

  /**
   * Adds a new item to the table of selected items.
   * @param item item which will be added to the selection.
   */
  private void addSelectedItem(int index) {
    int len = selectedDataModel.getRowCount();
    Object[][] newMatrix = new Object [len+1][2];
    for (int i=0; i<len; i++) {
      for (int j=0; j<2; j++) newMatrix[i][j] = selectedDataMatrix[i][j];
    }
    newMatrix[len][0] = dataFile.getItem(index);
    LinePars lp = new LinePars(dataFile.getItem(index),getColorIndex());
    if (gp.getGraphType() == gp.GRAPHTYPE_PIPER) {
      lp.setDrawLine(false);
      lp.setSymbol(getPointIndex());
    }
    newMatrix[len][1] = new LineStyle(lp);
    selectedDataMatrix = null;
    selectedDataMatrix = newMatrix;
    dataFile.addColumn(index,lp);
    if (JPlot.debug) {
      System.out.println("column " + index + 
			 " (" + dataFile.getItem(index) + ") added" +
			 ", there are now " + dataFile.getNumberOfColumns() +
			 " columns");
    }

  }

  /**
   * Removes an item from the table of selected items.
   * @param index index of the item which will be removed
   */
  private void removeSelectedItem(int index) {
    int len = selectedDataModel.getRowCount();
    if (len >= 1) {
      Object[][] newMatrix = new Object [len-1][2];
      int k=0;
      for (int i=0; i<len; i++) {
	if (i != index) {
	  newMatrix[k][0] = selectedDataMatrix[i][0];
	  newMatrix[k][1] = selectedDataMatrix[i][1];
	  k++;
	}
      }
      selectedDataMatrix = null;   // hoping for some garbage collecting ...
      selectedDataMatrix = newMatrix;
    }
    else selectedDataMatrix = new Object[0][2];
    dataFile.removeColumn(index);
  }

  /**
   * Returns a panel with a table of selected X and Y items.
   * @return a panel with a table of selected items.
   */
  private JPanel getSelectedPanel() {
    JPanel p = new JPanel(new BorderLayout());
    p.setBorder(new TitledBorder(new EtchedBorder(),"Selection"));
    p.add(getSelectedYPanel(),BorderLayout.CENTER);
    if (gp.getGraphType() != gp.GRAPHTYPE_PIPER) {
      p.add(getSelectedXPanel(),BorderLayout.NORTH);
    }
    return p;
  }

  /**
   * Returns a panel with a table of selected X item.
   * @return a panel with a table of selected items.
   */
  private JPanel getSelectedXPanel() {

    final String[] names = {"X-item"};

    selectedXDataMatrix = new Object [0][1];

    // Create a model of the data.
    selectedXDataModel = new AbstractTableModel() {
      public int getColumnCount() { return names.length; }
      public int getRowCount() { return selectedXDataMatrix.length;}
      public Object getValueAt(int row, int col) {return selectedXDataMatrix[row][col];}
      public String getColumnName(int column) {return names[column];}
      public Class getColumnClass(int c) {return getValueAt(0,c).getClass();}
      public boolean isCellEditable(int row, int col) {return false;}
      public void setValueAt(Object aValue, int row, int column) { 
	selectedXDataMatrix[row][column] = aValue; 
      }
    };
    
    // build a popup menu for the selected X table:
    selectedXPopup = new JPopupMenu();
    JMenuItem mi;
    mi = (JMenuItem) selectedXPopup.add(new JMenuItem("Options..."));
    mi.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent e) {
	  // To do
	}
      });
    mi = (JMenuItem) selectedXPopup.add(new JMenuItem("Remove"));
    mi.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent e) {
	  selectedXDataMatrix = new Object[0][0];
	  dataFile.setXColumn(-1);     // -1 unsets the X column index
	  selectedXTable.revalidate();
	  selectedXInFocus = false;
	}
      });
    
    // Create the table
    selectedXTable = new JTable(selectedXDataModel);
    selectedXTable.setSelectionBackground(new Color(248,248,215));
    selectedXTable.setSelectionForeground(Color.black);
    selectedXTable.setColumnSelectionAllowed(false);
    selectedXTable.setRowSelectionAllowed(false);
    selectedXTable.addMouseListener(new MouseAdapter() {
	public void mousePressed(MouseEvent e) {
	  if (e.isPopupTrigger() || 
	     e.getModifiers() == InputEvent.BUTTON3_MASK) {
	    selectedXPopup.show(selectedXTable,e.getX(),e.getY());
	  }
	}
	public void mouseClicked(MouseEvent e) {
	  selectedInFocus = false;
	  selectedXInFocus = true;
	}
      });
    Dimension d = new Dimension(220,selectedXTable.getRowHeight()+1);
    selectedXTable.setPreferredScrollableViewportSize(d);
    JScrollPane selectedX = new JScrollPane(selectedXTable);
    JPanel p = new JPanel();
    p.add(selectedX);
    return p;
  }

  /**
   * Fill the selected table with a default selection.
   */
  private void defaultSelection(boolean initialize) {
    if (JPlot.debug) System.out.print("setting default selection...");
    gp.colorIndex = 0;
    gp.pointIndex = 0;
    int N = dataFile.getNumberOfItems();
    itemDataMatrix = new Object [N][1];
    for (int i=0; i<N; i++) itemDataMatrix[i][0] = dataFile.getItem(i);

    //if (initialize) {
      if (gp.getGraphType() != gp.GRAPHTYPE_PIPER) {
	if (N > 0) {
	  selectedXDataMatrix = new Object[1][1];
	  selectedXDataMatrix[0][0] = dataFile.getItem(0);
	  dataFile.setXColumn(0);      
	}
	else selectedXDataMatrix = new Object [0][1];
	selectedXTable.revalidate();
      }
      
      int k=(gp.getGraphType()==gp.GRAPHTYPE_PIPER)? 0 : 1;
      if (N > k) {
	selectedDataMatrix = new Object [N-k][2];
	for (int i=k; i<N; i++) {
	  selectedDataMatrix[i-k][0] = dataFile.getItem(i);
	  LinePars lp = new LinePars(dataFile.getItem(i),getColorIndex());
	  if (k==0) {
	    lp.setDrawLine(false);
	    lp.setSymbol(getPointIndex());
	  }
	  selectedDataMatrix[i-k][1] = new LineStyle(lp);
	  dataFile.addColumn(i,lp);
	}
      }
      else selectedDataMatrix = new Object [0][2];
      selectedTable.revalidate();
      //}
    itemTable.revalidate();
    repaint();
    if (JPlot.debug) System.out.println("OK");
  }

  /**
   * Update the selected table with new data
   */  
  public void update() {
    if (JPlot.debug) System.out.print("updating in PlotPanel...");
    if (gp.getGraphType() != gp.GRAPHTYPE_PIPER) {
      if (dataFile.getNumberOfItems() > 0) {
	selectedXDataMatrix = new Object[1][1];
	if (dataFile.getXColumn() < dataFile.getNumberOfItems()) {
	  selectedXDataMatrix[0][0] = dataFile.getItem(dataFile.getXColumn());
	}
	else selectedXDataMatrix[0][0] = dataFile.getItem(0);
      }
      else selectedXDataMatrix = new Object [0][1];
      selectedXTable.revalidate();
    }
    int k = dataFile.getNumberOfColumns();
    if (k > 0) {
      selectedDataMatrix = new Object [k][2];
      for (int i=0; i<k; i++) {
	int m = dataFile.getYColumn(i);
	if (m >= dataFile.getNumberOfItems()) break;
	selectedDataMatrix[i][0] = dataFile.getItem(m);
	selectedDataMatrix[i][1] = new LineStyle(dataFile.getLinePars(i));
      }
    }
    else selectedDataMatrix = new Object [0][2];
    selectedTable.revalidate();
    if (JPlot.debug) System.out.println("done");
    repaint();
  }
  
  /**
   * Returns a panel with a table of selected Y items.
   * @return a panel with a table of selected Y items.
   */
  private JPanel getSelectedYPanel() {

    // initialize a StyleChooser instance, we'll need it later:
    styleChooser = new StyleChooser(jplot,"Style chooser",
				    new LinePars());
    selectedDataMatrix = new Object [0][2];
    selectedDataModel = new AbstractTableModel() {
      final String[] names = {"Y-items","style"};
      public int getColumnCount() { return names.length; }
      public int getRowCount() { return selectedDataMatrix.length;}
      public Object getValueAt(int row, int col) {
	return selectedDataMatrix[row][col];
      }
      public String getColumnName(int column) {return names[column];}
      public Class getColumnClass(int c) {return getValueAt(0,c).getClass();}
      public boolean isCellEditable(int row, int col) {return false;}
      public void setValueAt(Object aValue, int row, int column) {
	selectedDataMatrix[row][column] = aValue;
	fireTableCellUpdated(row,column);
      }
    };

    // build a popup menu for the table:
    selectedYPopup = new JPopupMenu();
    JMenuItem mi;
    mi = (JMenuItem) selectedYPopup.add(new JMenuItem("Plot style..."));
    mi.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent e) {
	  int row = selectedTable.getSelectedRow();
	  if (row < 0 || row >= selectedTable.getRowCount()) return;
	  LineStyle ls = (LineStyle)selectedDataModel.getValueAt(row,1);
	  LinePars lp = styleChooser.show(350,100,ls.getLinePars());
	  if (lp != null) {
	    gp.setDataChanged(lp.dataChanged());
	    dataFile.setLinePars(row,lp);
	    ls.setLinePars(lp);
	    selectedTable.repaint();
	    if (jplot.plotFrame != null) jplot.showGraph(true);
	  }
	}
      });
    mi = (JMenuItem) selectedYPopup.add(new JMenuItem("Remove"));
    mi.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent e) {
	  if (selectedInFocus) {
	    int[] index = selectedTable.getSelectedRows();
	    for (int j=0; j<index.length; j++) removeSelectedItem(index[0]);
	    selectedTable.revalidate();
	    selectedInFocus = false;
	  }
	}
      });

    // Create the table
    selectedTable = new JTable(selectedDataModel);    
    selectedTable.setColumnSelectionAllowed(false);
    selectedTable.setSelectionBackground(new Color(248,248,215));
    selectedTable.setSelectionForeground(Color.black);
    selectedTable.setCellSelectionEnabled(true);

    //Set up renderer for the linestyle
    selectedTable.setDefaultRenderer(LineStyle.class, new LineStyleRenderer());
    selectedTable.getColumn("style").setPreferredWidth((int)selectedTable.getColumn("style").getPreferredWidth()/3);
    selectedTable.getColumn("Y-items").setPreferredWidth(selectedTable.getColumn("Y-items").getPreferredWidth()*2);
    selectedTable.addMouseListener(new MouseAdapter() {
	public void mousePressed(MouseEvent e) {
	  if (e.isPopupTrigger() || 
	     e.getModifiers() == InputEvent.BUTTON3_MASK) {
	    selectedYPopup.show(selectedTable,e.getX(),e.getY());
	  }
	}
	
	public void mouseClicked(MouseEvent e) {
	  selectedInFocus = true;
	  selectedXInFocus = false;
	  if (selectedTable.getSelectedColumn() == 1) {
	    if (e.getClickCount() == 1) {
	      int row = selectedTable.getSelectedRow();
	      LineStyle ls = (LineStyle)selectedDataModel.getValueAt(row,1);
	      LinePars lp = styleChooser.show(350,100,ls.getLinePars());
	      if (lp != null) {
		gp.setDataChanged(lp.dataChanged());
		dataFile.setLinePars(row,lp);
		ls.setLinePars(lp);
		selectedTable.repaint();
		if (jplot.plotFrame != null) jplot.showGraph(true);
	      }
	    }
	  }
	}
      });
    
    selected = new JScrollPane(selectedTable);
    JPanel p = new JPanel(new BorderLayout());
    p.add(selected,BorderLayout.CENTER);
    JPanel q = new JPanel(new FlowLayout());
    ImageIcon icon = jplot.getImageIcon("Remove16.gif");
    JButton b = new JButton("Remove",icon);
    b.setToolTipText("Remove the selected item(s)");
    b.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	if (selectedInFocus) {
	  int[] index = selectedTable.getSelectedRows();
	  for (int j=0; j<index.length; j++) removeSelectedItem(index[0]);
	  selectedTable.revalidate();
	  selectedInFocus = false;
	}
	else if (selectedXInFocus) {
	  selectedXDataMatrix = new Object[0][0];
	  dataFile.setXColumn(-1);     // -1 unsets the X column index
	  //gp.getDataFile().setXColumn(-1);  
	  selectedXTable.revalidate();
	  selectedXInFocus = false;
	}
      }
    });
    q.add(b);
    icon = jplot.getImageIcon("New16.gif");
    b = new JButton("Clear",icon);
    b.setToolTipText("Clear the list with selected items");
    b.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	clearSelection();
      }
    });
    q.add(b);
    p.add(q,BorderLayout.SOUTH);
    return p;
  }

  /**
   * Clear all the selected stuff. This method is called when the
   * user clears the selection by pressing the clear button.
   */
  public void clearSelection() {
    int n = selectedDataModel.getRowCount();
    for (int j=0; j<n; j++) removeSelectedItem(0);
    selectedTable.revalidate();
    if (gp.getGraphType() != gp.GRAPHTYPE_PIPER) {
      selectedXDataMatrix = new Object[0][0];
      dataFile.setXColumn(-1);   // -1 unsets the X column index    
      selectedXTable.revalidate();
    }
    gp.colorIndex = 0;
    gp.pointIndex = 0;    
  }

  /**
   * Reloads the datafile and resets the selection to default.
   */
  public void reload() {
    if (JPlot.debug) System.out.println("reloading " + file.toString());
    if (!dataFile.loadColumnNames(gp.getLabels())) defaultSelection(true);
    jplot.updateGraphIfShowing();
  }

  /**
   * Reloads the datafile and resets the selection to default.
   */
  public void viewData() {
    if (dataViewer == null) {
      dataViewer = new SimpleEditor(jplot,file.toString(),true); 
    }
    dataViewer.show(100,20,file);
  }

  /**
   * Sets the index of the current datafile.
   * @param index index of the current datafile.
   */
  public void setDataFileIndex(int index) {
    dataFile.setIndex(index);
  }

  /**
   * @return the current datafile used by this panel
   */
  public DataFile getDataFile() {
    return dataFile;
  }

  /**
   * Returns the settings of this panel in a string.
   * This string is used to save the settings of the current
   * graph in a script file.
   * @return string with settings
   */
  public String getSettings() {
    StringBuffer sb = new StringBuffer(1000);
    sb.append("  datafile ").append(dataFile.getFile().toString());
    sb.append(" {").append(lf).append("    selection = ").append(dataFile.getXColumn());
    int N = dataFile.getNumberOfColumns();
    for (int i=0; i<N; i++) {
      sb.append(",").append(dataFile.getYColumn(i));
    }
    for (int i=0; i<N; i++) {
      sb.append(lf).append("    plotstyle of ").append(dataFile.getYColumn(i)).append(" {").append(lf);
      sb.append(dataFile.getLinePars(i).getSettings());
      sb.append("    }");
    }
    sb.append(lf).append("  }").append(lf);
    return sb.toString();
  }
  

  /**
   * Renders a cell of a JLabel as a panel with a linestyle demo.
   */
  private class LineStyleRenderer extends JPanel 
    implements TableCellRenderer {
    
    public LineStyleRenderer() {
      super();
      setBackground(Color.white);
    }
    
    public Component getTableCellRendererComponent(JTable table, 
						   Object value, 
						   boolean isSelected, 
						   boolean hasFocus,
						   int row, int column) {
      return (LineStyle)value;
    }
  }
}
