/*
 * JPLOT  -- Java Plotting Interface for any programme or
 *           as an independent GUI
 * 
 * Copyright (C) 1999 Jan van der Lee
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA.
 *
 * Send bugs, suggestions or queries to <jplot@cig.ensmp.fr>
 * The latest releases are found at 
 * http://www.cig.ensmp.fr/~vanderlee/jplot.html
 *
 * Initally developed for use by the Centre d'Informatique Geologique 
 * Ecole des Mines de Paris, Fontainebleau, France.
 */

package jplot;

import java.awt.*;
import java.util.*;
import javax.swing.*;
import java.awt.geom.*;
import java.awt.event.*;
import javax.swing.border.*;
import javax.swing.colorchooser.*;

import java.io.*;
import com.sun.image.codec.jpeg.*;
import java.awt.image.BufferedImage;


/**
 * SavePanel build a panel used to save the graph to some format (JPEG
 * for the time being, waiting for a decent EPS/PS decoder).
 *
 * Extends JPanel and not JDialog since this allows to include, if
 * needed later, the panel within another widget.
 * Calling the <code>show</code> method pops up a frame (dialog) which
 * includes the panel.
 */
public class SavePanel extends GriddedPanel {

  private JDialog dialog;

  private JComboBox format;
  private JTextField saveTo;
  private JTextField imageWidth;
  private JTextField imageHeight;

  private final String[] formats = {"jpeg"};

  private final Dimension mediumField = new Dimension(160,26);
  private final Dimension shortField = new Dimension(70,26);

  private Graph graph;

  /**
   * Principal constructor, builds the panel.
   */
  public SavePanel (Graph g) {
    dialog = null;
    graph = g;
    //setLayout(new BoxLayout(this,BoxLayout.Y_AXIS));

    // build the widgets:
    EmptyBorder border = new EmptyBorder(new Insets(0,0,0,10));
    JLabel label = new JLabel("Format:");
    label.setBorder(border);
    format = new JComboBox(formats);
    format.setPreferredSize(mediumField);
    format.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	// TODO: add action
      }
    });
    addComponent(label,1,1);
    addFilledComponent(format,1,2,2,1,GridBagConstraints.HORIZONTAL);

    label = new JLabel("Save to:");
    label.setBorder(border);
    saveTo = new JTextField("out.jpg");
    saveTo.setPreferredSize(mediumField);
    addComponent(label,2,1);
    addFilledComponent(saveTo,2,2,2,1,GridBagConstraints.HORIZONTAL);
    //addComponent(saveTo,2,2);

    label = new JLabel("Image size:");
    label.setBorder(border);
    JPanel p = new JPanel(new BorderLayout());
    p.add(Utils.makeLabel("width"),BorderLayout.WEST);
    imageWidth = new JTextField("570");
    p.add(imageWidth,BorderLayout.CENTER);
    p.setPreferredSize(shortField);
    addComponent(label,3,1);
    addComponent(p,3,2);

    p = new JPanel(new BorderLayout());
    p.add(Utils.makeLabel("height"),BorderLayout.WEST);
    imageHeight = new JTextField("410");
    p.add(imageHeight,BorderLayout.CENTER);
    p.setPreferredSize(shortField);
    addComponent(p,3,3);
  }

  /**
   * Builds a panel with the scaling options for one of the axes.
   * @param axis axis for which this panel is built.
   */
  /*
  private JPanel getJpegPanel() {
    GriddedPanel p = new GriddedPanel();
    p.setBorder(new EmptyBorder(new Insets(5,5,5,5)));

    return p;
  }
  */

  /**
   * Saves the current image to a JPG. Draws the image off-screen
   * using the specfied width/height.
   */
  public void saveJPG() {
    if (saveTo.getText().equals("")) return;
    int width = Integer.parseInt(imageWidth.getText());
    int height = Integer.parseInt(imageHeight.getText());
    Dimension size = new Dimension(width,height);
    
    try {
      File file = new File(saveTo.getText());
      FileOutputStream out = new FileOutputStream(file);
      JPEGImageEncoder encoder = JPEGCodec.createJPEGEncoder(out);

      BufferedImage bi = graph.getBufferedImage(size);
      JPEGEncodeParam param = encoder.getDefaultJPEGEncodeParam(bi);
      param.setQuality(1.0f,false);
      encoder.setJPEGEncodeParam(param);
      encoder.encode(bi);
    } catch (Exception ex) {
      ex.printStackTrace();
    }
  }

  /**
   * Updates the panel with the current graphical parameters.
   */
  public void refresh() {
  }

  public void setValues() {
  }

 /**
   * Return a modal JDialog.
   * @param parent frame which is the parent of this dialog.
   * @param x x-position of the upper left corner
   * @param y y-position of the upper left corner
   */
  public void show(Frame parent, int x, int y) {
    if (dialog == null) {
      dialog = new JDialog(parent,"Save graph",false);
      dialog.addWindowListener(new WindowAdapter() {
	public void windowClosing(WindowEvent e) {
	  dialog.dispose();
	}
      });

      JPanel p = new JPanel(new FlowLayout());
      p.setBorder(BorderFactory.createEtchedBorder());
      JButton b = new JButton("Apply");
      b.addActionListener(new ActionListener() {
	  public void actionPerformed(ActionEvent e) {
	    saveJPG();
	    dialog.dispose();
	  }
	});
      p.add(b);
      b = new JButton("Dismiss");
      b.addActionListener(new ActionListener() {
	  public void actionPerformed(ActionEvent e) {
	    dialog.dispose();
	  }
	});
      p.add(b);
      dialog.getContentPane().add(this,BorderLayout.CENTER);
      dialog.getContentPane().add(p,BorderLayout.SOUTH);
      dialog.setLocation(x,y);
      dialog.pack();
    }
    refresh();
    dialog.show();  // blocks until user brings dialog down.
  }
}









