/*
 * JPLOT  -- Java Plotting Interface for any programme or
 *           as an independent GUI
 * 
 * Copyright (C) 1999 Jan van der Lee
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA.
 *
 * Send bugs, suggestions or queries to <jplot@cig.ensmp.fr>
 * The latest releases are found at 
 * http://www.cig.ensmp.fr/~vanderlee/jplot.html
 *
 * Initally developed for use by the Centre d'Informatique Geologique 
 * Ecole des Mines de Paris, Fontainebleau, France.
 */

package jplot;

import java.awt.*;
import java.util.*;
import javax.swing.*;
import java.awt.geom.*;
import java.awt.event.*;
import javax.swing.border.*;
import javax.swing.colorchooser.*;

/**
 * ScalingPanel creates a panel which allows a user to modify the
 * way the graph is scaled on the axes. For example, by setting the
 * maximum and minimum values of the X-axis instead of using an
 * automatic scaling.
 * Calling the <code>show</code> method pops up a frame (dialog) which
 * includes the panel.
 */
public class ScalingPanel extends JPanel {

  private JDialog dialog;

  private JCheckBox[] cb_auto = new JCheckBox [GraphPars.NAXES];
  private JCheckBox[] cb_manu = new JCheckBox [GraphPars.NAXES];
  private JCheckBox[] cb_log = new JCheckBox [GraphPars.NAXES];
  private JCheckBox[] cb_lin = new JCheckBox [GraphPars.NAXES];
  private JTextField[] divide = new JTextField [GraphPars.NAXES];
  private JTextField[] offset = new JTextField [GraphPars.NAXES];
  private JTextField[] to = new JTextField [GraphPars.NAXES];
  private JTextField[] from = new JTextField [GraphPars.NAXES];
  private GraphPars gp;
  private JPlot jplot;
  
  /**
   * Principal constructor. Builds a panel with X and Y options
   * @param gp instance of the object containing all graphical parameters
   */
  public ScalingPanel (JPlot jp, GraphPars _gp) {
    gp = _gp;
    jplot = jp;
    dialog = null;
    setLayout(new BoxLayout(this,BoxLayout.Y_AXIS));
    for (int k=0; k<gp.NAXES; k++) add(getPanel(k));
  }

  /**
   * Builds a panel with the scaling options for one of the axes.
   * @param axis axis for which this panel is built.
   */
  private GriddedPanel getPanel(final int axis) {
    String labelText = (axis == gp.X_AXIS)? "X-scaling:" : "Y-scaling:";

    GriddedPanel p = new GriddedPanel();
    p.setBorder(new EmptyBorder(new Insets(5,5,5,5)));
    
    Dimension cbField = new Dimension(110,20);
    Dimension shortField = new Dimension(80,22);

    // build the widgets:
    ButtonGroup bg = new ButtonGroup();
    cb_lin[axis] = new JCheckBox("linear",!gp.useLogScale(axis));
    cb_lin[axis].setPreferredSize(cbField);
    bg.add(cb_lin[axis]);
    cb_log[axis] = new JCheckBox("logarithmic",gp.useLogScale(axis));
    cb_log[axis].setPreferredSize(cbField);
    bg.add(cb_log[axis]);

    divide[axis] = new JTextField();
    offset[axis] = new JTextField();

    bg = new ButtonGroup();
    cb_auto[axis] = new JCheckBox("automatic",gp.useAutoRange(axis));
    cb_auto[axis].setPreferredSize(cbField);
    cb_auto[axis].addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {	
	from[axis].setEnabled(false);
	to[axis].setEnabled(false);
      }
    });
    bg.add(cb_auto[axis]);
    cb_manu[axis] = new JCheckBox("manual",!gp.useAutoRange(axis));
    cb_manu[axis].setPreferredSize(cbField);
    cb_manu[axis].addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {	
	from[axis].setEnabled(true);
	to[axis].setEnabled(true);
      }
    });
    bg.add(cb_manu[axis]);

    JPanel fromPanel = new JPanel(new BorderLayout());
    fromPanel.add(Utils.makeLabel("from"),BorderLayout.WEST);
    from[axis] = new JTextField((new Float(gp.getMinValue(axis))).toString());
    from[axis].setEnabled(cb_manu[axis].isSelected());
    fromPanel.add(from[axis],BorderLayout.CENTER);
    fromPanel.setPreferredSize(shortField);

    JPanel toPanel = new JPanel(new BorderLayout());
    toPanel.add(Utils.makeLabel("to"),BorderLayout.WEST);
    to[axis] = new JTextField((new Float(gp.getMaxValue(axis))).toString());
    to[axis].setEnabled(cb_manu[axis].isSelected());
    toPanel.add(to[axis],BorderLayout.CENTER);
    toPanel.setPreferredSize(shortField);

    // spacing between labels and fields:
    EmptyBorder border = new EmptyBorder(new Insets(0,0,0,10));

    // place the widgets in the panel:
    JLabel label = new JLabel(labelText);
    label.setBorder(border);
    p.addComponent(label,1,1);
    p.addComponent(cb_lin[axis],1,2);
    p.addComponent(cb_log[axis],1,3);
    label = new JLabel("divide by");
    label.setBorder(border);
    p.addComponent(label,2,2);
    p.addFilledComponent(divide[axis],2,3);
    label = new JLabel("offset");
    label.setBorder(border);
    p.addComponent(label,3,2);
    p.addFilledComponent(offset[axis],3,3);
    p.addComponent(cb_auto[axis],4,2);
    p.addComponent(cb_manu[axis],4,3);
    p.addFilledComponent(fromPanel,5,2);
    p.addFilledComponent(toPanel,5,3);
    return p;
  }

  /**
   * Updates the panel with the current graphical parameters.
   */
  public void refresh() {
    for (int axis=0; axis<gp.NAXES; axis++) {
      cb_auto[axis].setSelected(gp.useAutoRange(axis));
      cb_manu[axis].setSelected(!gp.useAutoRange(axis));
      from[axis].setText(Float.toString((float)gp.getMinValue(axis)));
      from[axis].setEnabled(!cb_auto[axis].isSelected());
      to[axis].setText(Float.toString((float)gp.getMaxValue(axis)));
      to[axis].setEnabled(!cb_auto[axis].isSelected());
      cb_log[axis].setSelected(gp.useLogScale(axis));
      cb_lin[axis].setSelected(!gp.useLogScale(axis));
      divide[axis].setText(Float.toString(gp.getGlobalDivider(axis)));
      offset[axis].setText(Float.toString(gp.getGlobalOffset(axis)));
    }
  }

  public void setValues() {
    for (int k=0; k<gp.NAXES; k++) {
      gp.setAutoRange(k,cb_auto[k].isSelected());
      gp.setLogScale(k,cb_log[k].isSelected());
      gp.setGlobalDivider(k,Float.parseFloat(divide[k].getText()));
      gp.setGlobalOffset(k,Float.parseFloat(offset[k].getText()));
      if (!cb_auto[k].isSelected()) {
	if (!from[k].getText().equals("")) {
	  gp.setMinValue(k,(new Double(from[k].getText())).doubleValue());
	}
	if (!to[k].getText().equals("")) {
	  gp.setMaxValue(k,(new Double(to[k].getText())).doubleValue());
	}
	if (cb_log[k].isSelected() && (gp.getMinValue(k) <= 0.0 ||
				       gp.getMaxValue(k) <= 0.0)) {
	  Utils.bummer(jplot.frame,"Don't! Logarithmic axes require positive data.\n Switching log-scale off...\n");
	  gp.setLogScale(k,false);
	}
      }
    }
  }

 /**
   * Return a modal JDialog.
   * @param parent frame which is the parent of this dialog.
   * @param x x-position of the upper left corner
   * @param y y-position of the upper left corner
   */
  public void show(Frame parent, int x, int y) {
    if (dialog == null) {
      dialog = new JDialog(parent,"Scaling options",false);
      dialog.addWindowListener(new WindowAdapter() {
	public void windowClosing(WindowEvent e) {
	  dialog.dispose();
	}
      });

      JPanel p = new JPanel(new FlowLayout());
      p.setBorder(BorderFactory.createEtchedBorder());
      JButton b = new JButton("Apply");
      b.addActionListener(new ActionListener() {
	  public void actionPerformed(ActionEvent e) {
	    setValues();
	    gp.setDataChanged(true);
	    jplot.updateGraphIfShowing();
	  }
	});
      p.add(b);
      b = new JButton("Dismiss");
      b.addActionListener(new ActionListener() {
	  public void actionPerformed(ActionEvent e) {
	    dialog.dispose();
	  }
	});
      p.add(b);
      dialog.getContentPane().add(this,BorderLayout.CENTER);
      dialog.getContentPane().add(p,BorderLayout.SOUTH);
      dialog.setLocation(x,y);
      dialog.pack();
    }
    refresh();
    dialog.show();  // blocks until user brings dialog down.
  }
}









