/*
 * JPLOT  -- Java Plotting Interface for any programme or
 *           as an independent GUI
 * 
 * Copyright (C) 1999 Jan van der Lee
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA.
 *
 * Send bugs, suggestions or queries to <jplot@cig.ensmp.fr>
 * The latest releases are found at 
 * http://www.cig.ensmp.fr/~vanderlee/jplot.html
 *
 * Initally developed for use by the Centre d'Informatique Geologique 
 * Ecole des Mines de Paris, Fontainebleau, France.
 */

package jplot;

import java.awt.*;
import java.util.*;
import javax.swing.*;
import java.awt.geom.*;
import java.awt.event.*;
import javax.swing.border.*;
import javax.swing.colorchooser.*;

/**
 * StyleChooser is a class which enables the user to set the drawing style.
 * It must be called with a DataArray object, which contains the initial
 * drawing style. This class also provides the data drawing style 
 * chooser, an advanced specific purpose chooser which allows to select
 * a line- and/or point drawing style, a plotting style, colors etc.
 */
public class StyleChooser {

  private LinePars lp;
  private LineStyle lineStyle;
  private JDialog chooser;
  private JButton b1,b2,b3;
  private ImageIcon no_icon1,no_icon2,no_icon3;
  private ImageIcon yes_icon1,yes_icon2,yes_icon3;
  private DashPanel dashPanel;
  private PointPanel pointPanel;
  private ColorPanel colorPanel, popupColorPanel;
  private JPanel chooserPanel;

  private SmallColorPreview startColorPreview;
  private SmallColorPreview endColorPreview;
  private JButton b_startColor, b_endColor;
  private JRadioButton rb_slideColor;

  private JCheckBox cb_show, cb_hide;
  private JTextField legendName,divider,additioner;
  private JPlot jplot;
  private String title;

  private boolean isGlobal;
  
  /**
   * Principal constructor.
   * @param jp parent jplot instance to which this dialog is attached
   * @param title current title of the dialog
   * @param lp line parameter settings (pointtype, color, linestyle...)
   * @param global true if built as a global chooser
   */
  public StyleChooser(JPlot jp, String title, LinePars lp, 
		      boolean global) {
    jplot = jp;
    this.title = title;
    isGlobal = global;
    if (lp != null) this.lp = lp;
    else this.lp = new LinePars();
    chooser = null;
  }

  /**
   * Constructor.
   * @param jp parent jplot instance to which this dialog is attached
   * @param title current title of the dialog
   * @param lp line parameter settings (pointtype, color, linestyle...)
   */
  public StyleChooser(JPlot jp, String title, LinePars lp) {
    this(jp,title,lp,false);
  }

  /**
   * Principal constructor. The argument is a DataArray object, which
   * contains the drawing styles. Always built as a local chooser.
   * @param jp parent jplot instance to which this dialog is attached
   * @param title current title of the dialog
   * @param global true if built as a global chooser
   */
  public StyleChooser(JPlot jp, String title, boolean global) {
    this(jp,title,null,global);
  }

  /*
   * Returns a chooser panel for selecting a new drawing style.  The
   * panel contains five fields, with the <b>style</b> (e.g., error
   * boxes, lines, points, histograms etc.), the <b>line-type</b>
   * (solid, dashed, etc.), the <b>point-type</b> (none, dots,
   * triangles etc.), the </b>color</b> and a window showing the
   * actual choice.
   * 
   * @param isGlobal true if the style chooser is used for all the 
   * datapoints
   */
  private void getChooserPanel() {
    chooserPanel = new JPanel(new BorderLayout());

    JPanel mainPanel = new JPanel(new BorderLayout());
    yes_icon1 = jplot.getImageIcon("plot1.jpg");
    yes_icon2 = jplot.getImageIcon("plot2.jpg");
    yes_icon3 = jplot.getImageIcon("plot3.jpg");
    no_icon1 = jplot.getImageIcon("pplot1.jpg");
    no_icon2 = jplot.getImageIcon("pplot2.jpg");
    no_icon3 = jplot.getImageIcon("pplot3.jpg");
    
    // the northern part contains the minigraphs
    // which define the drawing styles:
    //------------------------------------------
    JPanel north = new JPanel();
    //north.setLayout(new BoxLayout(north,BoxLayout.X_AXIS));
    north.setBorder(new TitledBorder(new EtchedBorder(),"Styles"));
    b1 = new JButton(no_icon1);
    b1.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	lp.setGraphStyle(lp.LINES);
	setGraphStyles();
      }
    });
    b1.setMargin(new Insets(0,0,0,0));
    north.add(b1);
    b2 = new JButton(no_icon2);
    b2.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	lp.setGraphStyle(lp.HISTO);
	setGraphStyles();
      }
    });
    b2.setMargin(new Insets(0,0,0,0));  
    north.add(b2);
    b3 = new JButton(no_icon3);
    b3.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	lp.setGraphStyle(lp.STICKS);
	setGraphStyles();
      }
    });
    b3.setMargin(new Insets(0,0,0,0));  
    north.add(b3);
    mainPanel.add(north,BorderLayout.NORTH);

    // the western part is about linestyle (dash-types):
    //--------------------------------------------------
    dashPanel = new DashPanel(lp);
    mainPanel.add(dashPanel,BorderLayout.WEST);

    // center panel contains the plotting points:
    //-------------------------------------------
    pointPanel = new PointPanel(lp);
    mainPanel.add(pointPanel,BorderLayout.CENTER);

    // right panel with the colors:
    //-----------------------------
    colorPanel = new ColorPanel(jplot.frame,lp.getColor());
    mainPanel.add(colorPanel,BorderLayout.EAST);

    // a small panel with the legend label and a checkbox:
    //----------------------------------------------------
    GriddedPanel legend=null;
    GriddedPanel global=null;
    if (!isGlobal) {
      legend = new GriddedPanel();
      legend.setBorder(new TitledBorder(new EtchedBorder(),"Legend label"));
      legendName = new JTextField(lp.getName());
      legend.addFilledComponent(legendName,1,1,2,1,GridBagConstraints.HORIZONTAL);
      ButtonGroup bg = new ButtonGroup();
      cb_show = createCheckBox("show ",lp.drawLegend());
      bg.add(cb_show);
      legend.addComponent(cb_show,1,3);
      cb_hide = createCheckBox("hide ",!lp.drawLegend());
      bg.add(cb_hide);
      legend.addComponent(cb_hide,1,4);
    }

    // button to reset the plot style to default:
    else {
      global = new GriddedPanel();
      global.setBorder(new TitledBorder(new EtchedBorder(),"Global options"));

      rb_slideColor = new JRadioButton("slide color");
      rb_slideColor.setSelected(false);
      rb_slideColor.addActionListener(new ActionListener() {
	  public void actionPerformed(ActionEvent e) {
	    b_startColor.setEnabled(rb_slideColor.isSelected());
	    b_endColor.setEnabled(rb_slideColor.isSelected());
	  }
	});
      global.addComponent(rb_slideColor,1,1);

      b_startColor = new JButton(jplot.getImageIcon("color.jpg"));
      b_startColor.addActionListener(new ActionListener() {
	  public void actionPerformed(ActionEvent e) {
	    if (popupColorPanel == null) {
	      popupColorPanel = new ColorPanel(jplot.frame);
	    }
	    Color color = popupColorPanel.show(jplot.frame,
					       lp.getStartColor(),100,100);
	    if (color != null) startColorPreview.setColor(color);
	  }
	});
      b_startColor.setMargin(new Insets(0,0,0,0));
      b_startColor.setToolTipText("Change the start color");
      startColorPreview = new SmallColorPreview(lp.getStartColor(),40,18);
      JPanel p = new JPanel(new FlowLayout());
      p.add(startColorPreview);
      p.add(b_startColor);

      JLabel label = new JLabel("Start:");
      //label.setBorder(border);
      global.addComponent(label,1,2);
      global.addComponent(p,1,3);      

      b_endColor = new JButton(jplot.getImageIcon("color.jpg"));
      b_endColor.addActionListener(new ActionListener() {
	  public void actionPerformed(ActionEvent e) {
	    if (popupColorPanel == null) {
	      popupColorPanel = new ColorPanel(jplot.frame);
	    }
	    Color color = popupColorPanel.show(jplot.frame,
					       lp.getEndColor(),100,100);
	    if (color != null) endColorPreview.setColor(color);
	  }
	});
      b_endColor.setMargin(new Insets(0,0,0,0));
      b_endColor.setToolTipText("Change the end color");
      endColorPreview = new SmallColorPreview(lp.getEndColor(),40,18);
      p = new JPanel(new FlowLayout());
      p.add(endColorPreview);
      p.add(b_endColor);

      label = new JLabel("End:");
      //label.setBorder(border);
      global.addComponent(label,1,4);
      global.addComponent(p,1,5);      
    }
      
    JPanel scaling = new JPanel(new FlowLayout());
    scaling.setBorder(new TitledBorder(new EtchedBorder(),"Scaling"));
    divider = new JTextField(Float.toString(1.0f/lp.getMultiplier()));
    divider.setPreferredSize(new Dimension(65,24));
    scaling.add(new JLabel("divide by:"));
    scaling.add(divider);
    additioner = new JTextField(Float.toString(lp.getAdditioner()));
    additioner.setPreferredSize(new Dimension(50,24));
    scaling.add(new JLabel("offset:"));
    scaling.add(additioner);

    JPanel p = new JPanel();
    p.setLayout(new BoxLayout(p,BoxLayout.Y_AXIS));
    if (!isGlobal) p.add(legend);
    else p.add(global);
    p.add(scaling);
    mainPanel.add(p,BorderLayout.SOUTH);
    chooserPanel.add(mainPanel,BorderLayout.CENTER);

    // south contains buttons OK or Cancel
    //------------------------------------
    p = new JPanel();
    p.setBorder(new EtchedBorder());
    JPanel buttons = new JPanel(new GridLayout(1,2));
    JButton b = new JButton("Apply");
    b.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	lp.setColor(colorPanel.getSelectedColor());
	if (!isGlobal) {
	  lp.setName(legendName.getText());
	  if (cb_show.isSelected()) lp.setDrawLegend(true);
	  else lp.setDrawLegend(false);
	}
	else if (rb_slideColor.isSelected()) {
	  lp.setSlideColor(startColorPreview.getColor(),
			   endColorPreview.getColor());
	}
	if (!divider.getText().equals("")) {
	  lp.setMultiplier(1.0f/Float.parseFloat(divider.getText()));
	}
	else lp.setMultiplier(1.0f);
	if (!additioner.getText().equals("")) {
	  lp.setAdditioner(Float.parseFloat(additioner.getText()));
	}
	else lp.setAdditioner(0.0f);
	lp.setDataChanged(true);
	chooser.dispose();
      }
    });
    buttons.add(b);
    b = new JButton("Cancel");
    b.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	lp = null;
	chooser.dispose();
      }
    });
    buttons.add(b);
    p.add(buttons);
    chooserPanel.add(p,BorderLayout.SOUTH);
  }

  /*
   * Make a label specifically used by the tic-panel.
   * @return a label with specific font and alignments.
   */
  private JLabel makeLabel(String txt)
  {
    JLabel label = new JLabel(txt + " ");
    label.setVerticalAlignment(SwingConstants.BOTTOM);
    label.setHorizontalAlignment(SwingConstants.CENTER);
    label.setFont(new Font("SansSerif",Font.ITALIC,11));
    label.setForeground(Color.black);
    return label;
  }

  private JCheckBox createCheckBox(String s, boolean b) {
    JCheckBox cb = new JCheckBox(" " + s + " ", b);
    cb.setFont(new Font("serif", Font.PLAIN, 12));
    cb.setHorizontalAlignment(JCheckBox.LEFT);
    return cb;
  }

  /*
   * updates the panel with the current data
   */
  private void setGraphStyles() {
    if (lp.getGraphStyle() == lp.LINES) b1.setIcon(yes_icon1);
    else b1.setIcon(no_icon1);
    if (lp.getGraphStyle() == lp.HISTO) b2.setIcon(yes_icon2);
    else b2.setIcon(no_icon2);
    if (lp.getGraphStyle() == lp.STICKS) b3.setIcon(yes_icon3);
    else b3.setIcon(no_icon3);
    chooserPanel.repaint();
  }

  /*
   * updates the panel with the current data
   */
  private void refresh() {
    setGraphStyles();
    dashPanel.refresh(lp);
    pointPanel.refresh(lp);
    colorPanel.refresh(lp.getColor());
    divider.setText(Double.toString(1.0/lp.getMultiplier()));
    additioner.setText(Double.toString(lp.getAdditioner()));
    if (!isGlobal) {
      cb_show.setSelected(lp.drawLegend());
      cb_hide.setSelected(!lp.drawLegend());
      legendName.setText(lp.getName());
    }
    else {
      startColorPreview.setColor(lp.getStartColor());
      endColorPreview.setColor(lp.getEndColor());
      rb_slideColor.setSelected(lp.slideColor());
      b_startColor.setEnabled(rb_slideColor.isSelected());
      b_endColor.setEnabled(rb_slideColor.isSelected());
    }
  }

  /**
   * Return a modal JDialog with a customized linestyle chooser.
   * The panel includes choices for dashes, pointstyles and colors.
   * @param x coordinate of the pin used to fix this frame
   * @param y coordinate of the pin used to fix this frame
   * @param lp initial line parameters
   * @return new line style parameters or null if canceled
   */
  public LinePars show(int x, int y, LinePars initLinePars) {
    if (initLinePars == null) lp = new LinePars();
    else lp = new LinePars(initLinePars);

    if (chooser == null) {
      chooser = new JDialog(jplot.frame,title,true);
      chooser.addWindowListener(new WindowAdapter() {
	public void windowClosing(WindowEvent e) {
	  lp = null;
	  chooser.dispose();
	}
      });
      getChooserPanel();
      chooser.setLocation(x,y);
      chooser.getContentPane().add(chooserPanel);
      chooser.pack();
    }
    refresh();
    chooser.pack();
    chooser.show();  // blocks until user brings dialog down.
    return lp;
  }
}

