/*
 * JPLOT  -- Java Plotting Interface for any programme or
 *           as an independent GUI
 * 
 * Copyright (C) 1999 Jan van der Lee
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA.
 *
 * Send bugs, suggestions or queries to <jplot@cig.ensmp.fr>
 * The latest releases are found at 
 * http://www.cig.ensmp.fr/~vanderlee/jplot.html
 *
 * Initally developed for use by the Centre d'Informatique Geologique 
 * Ecole des Mines de Paris, Fontainebleau, France.
 */

package jplot;

import java.awt.*;
import javax.swing.*;

/**
 * SwatchPanel builds a chooser panel from a series of rectangular
 * chooseable 'buttons'... Note that this class must be derived in
 * order to be used properly.
 */
class SwatchPanel extends JPanel {

  protected Dimension swatchSize;
  protected Dimension numSwatches;
  protected Dimension gap;
  protected int Ns;
  Color[] lowerLeftLineColor;
  Color[] upperRightLineColor;
  protected int width, height;

  /**
   * Constructs the swatch panel
   * @param selectedIndex index of the current selected index
   */
  public SwatchPanel(int selectedIndex) {
    initValues();
    Ns = numSwatches.width*numSwatches.height;
    setToolTipText("");
    setOpaque(true);
    gap = new Dimension(2,2);
    lowerLeftLineColor = new Color [Ns];
    upperRightLineColor = new Color [Ns];
    setRequestFocusEnabled(false);
    resetBackgrounds();
    if (selectedIndex >= 0) {
      lowerLeftLineColor[selectedIndex] = Color.white;
      upperRightLineColor[selectedIndex] = Color.black;
    }
    width = numSwatches.width*(swatchSize.width + gap.width) -1;
    height = numSwatches.height*(swatchSize.height + gap.height) -1;
  }

  /**
   * Constructs the swatch panel
   */
  public SwatchPanel() {
    this(0);
  }  

  protected void initValues() {
    swatchSize = new Dimension(1,1);
    numSwatches = new Dimension(1,1);
  }

  protected void resetBackgrounds() {
    for (int i=0; i<Ns; i++) {
      lowerLeftLineColor[i] = Color.black;
      upperRightLineColor[i] = Color.white;
    }
  }

  /**
   * Presses the swatch with the specified index, which correspond
   * to the actually selected swatch.
   * @param index index of the selected swatch.
   */
  protected void setSelectedSwatch(int index) {
    for (int i=0; i<Ns; i++) {
      lowerLeftLineColor[i] = Color.black;
      upperRightLineColor[i] = Color.white;
    }
    if (index >= 0) {
      lowerLeftLineColor[index] = Color.white;
      upperRightLineColor[index] = Color.black;
    }
    repaint();
  }
  
  protected void paintIt(Graphics2D g2, int column, int row, int x, int y) {}

  public void paintComponent(Graphics g) {
    if (Ns == 0) return;
    Graphics2D g2 = (Graphics2D)g;
    g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, 
			RenderingHints.VALUE_ANTIALIAS_ON);
    g2.setColor(getBackground());
    g2.fillRect(0,0,getWidth(), getHeight());
    for (int row=0; row<numSwatches.height; row++) {
      for (int column=0; column<numSwatches.width; column++) {
	int x = column*(swatchSize.width+gap.width);
	int y = row*(swatchSize.height+gap.height);
	int index = row*numSwatches.width+column;
	paintIt(g2,column,row,x,y);
	g2.setColor(lowerLeftLineColor[index]);
	g2.drawLine(x+swatchSize.width-1,y,x+swatchSize.width-1,y+swatchSize.height-1);
	g2.drawLine(x,y+swatchSize.height-1,x+swatchSize.width-1,y+swatchSize.height-1);
	g2.setColor(upperRightLineColor[index]);
	g2.drawLine(x,y,x+swatchSize.width-1,y);
	g2.drawLine(x,y,x,y+swatchSize.height-1);
      }
    }
  }
  
  public Dimension getPreferredSize() {
    return new Dimension(width,height);
  }

  public Dimension getMaximumSize() {
    return getPreferredSize();
  }
  
  public Dimension getMinimumSize() {
    return getPreferredSize();
  }
  
  public int getIndex(int x, int y) {
    return (getRow(y)*numSwatches.width)+getColumn(x);
  }

  public int getColumn(int x) {
    return x/(swatchSize.width+gap.width);
  }

  public int getRow(int y) {
    return y/(swatchSize.height+gap.height);
  }
}
