/*
 * JPLOT  -- Java Plotting Interface for any programme or
 *           as an independent GUI
 * 
 * Copyright (C) 1999 Jan van der Lee
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA.
 *
 * Send bugs, suggestions or queries to <jplot@cig.ensmp.fr>
 * The latest releases are found at 
 * http://www.cig.ensmp.fr/~vanderlee/jplot.html
 *
 * Initally developed for use by the Centre d'Informatique Geologique 
 * Ecole des Mines de Paris, Fontainebleau, France.
 */

package jplot;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import javax.swing.text.*;

import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;

/** 
 * General utilities, marco classes
 * 
 * @author  : Jan van der Lee <vanderlee@cig.ensmp.fr>
 * @created : Fri Aug 14 18:08:08 MET DST 1998 on asterix
 * @modified: Fri Aug 21 1998
 */

public class Utils {

  static final String empty = "            ";
  private static JColorChooser colorChooser;

  public static final String lf = System.getProperty("line.separator");
  public static final String fs = System.getProperty("file.separator");
  

  /**
   * Returns a color chooser on request. Very usefull if the
   * color chooser is often needed, since we only allocate the
   * chooser once
   */
  public static JColorChooser getColorChooser() {
    if (colorChooser == null) colorChooser = new JColorChooser();
    return colorChooser;
  }

  /**
   * Says whether we're running on a UNIX platform or not.
   * This is simply checked by the file-separator variable.
   * @return true if you're lucky
   */
  public static boolean isUnix()
  {
    return fs.equals("/");
  }

  /**
   * Oops message.
   * Shows a message dialog box with a specified message.
   * @param s the message
   */
  static void oops(Frame parent, String s) {
    JOptionPane.showMessageDialog(parent,s,"Oops",JOptionPane.INFORMATION_MESSAGE);
  }

  /**
   * Bummer message.
   * Shows an error message dialog box with a specified message.
   * @param s the message
   */
  static void bummer(Frame parent, String s) {
    JOptionPane.showMessageDialog(parent,s,"Bummer",JOptionPane.ERROR_MESSAGE);
  }

  /**
   * Returns a new instance of a default font. Currently, 'Default'
   * doesn't seem too nice on e.g. unix platforms. 
   * @return a new instance of a font.
   */
  static Font getDefaultFont() {
    return new Font("Helvetica",Font.PLAIN,11);
  }
  
  /**
   * Make a label specifically used by the tic-panel.
   * @return a label with specific font and alignments.
   */
  static public JLabel makeLabel(String txt)
  {
    JLabel label = new JLabel(txt + " ");
    label.setVerticalAlignment(SwingConstants.BOTTOM);
    label.setHorizontalAlignment(SwingConstants.CENTER);
    //label.setFont(new Font("SansSerif",Font.ITALIC,11));
    label.setFont(new Font("SansSerif",Font.PLAIN,11));
    label.setForeground(Color.black);
    return label;
  }

  static public JCheckBox createCheckBox(String s, boolean b) {
    JCheckBox cb = new JCheckBox(" " + s + " ", b);
    cb.setFont(new Font("serif", Font.PLAIN, 12));
    cb.setHorizontalAlignment(JCheckBox.LEFT);
    return cb;
  }

  public static class LabeledComboBox extends JPanel {

    public JComboBox combobox;
    private boolean action;

    /**
     * Principal constructor.
     * Builds a combox with a label to the left of it.
     *
     * @param label the label
     * @param v a vector with the combobox content
     * @oaram editable a flag which is true if the field is editable
     */
    LabeledComboBox(String label, boolean editable)
    {
      combobox = new JComboBox();
      combobox.setEditable(editable);
      action = true;
      setLayout(new FlowLayout());
      JLabel l = new JLabel(label);
      l.setHorizontalTextPosition(l.RIGHT);
      //l.setDisplayedMnemonic(label.charAt(0));
      l.setLabelFor(combobox); 
      add(l);
      add(combobox);
    }   

    LabeledComboBox(String label, boolean editable, Vector v)
    {
      this(label,editable);
      replaceContent(v);
    }

    LabeledComboBox(String label, boolean editable, String s[])
    {
      this(label,editable);
      replaceContent(s);
    }

    public void replaceContent(Vector v)
    {
      action = false;
      if (combobox.getItemCount() > 0) combobox.removeAllItems();
      if (v.size() == 0) {
	combobox.setEnabled(false);
	combobox.addItem(empty);
      }
      else {
	combobox.setEnabled(true);
	for (Enumeration e=v.elements(); e.hasMoreElements();) {
	  combobox.addItem(e.nextElement());
	}
      }
      action = true;
    }

    public void replaceContent(String s[])
    {
      action = false;
      if (combobox.getItemCount() > 0) combobox.removeAllItems();
      if (s.length == 0) {
	combobox.setEnabled(false);
	combobox.addItem(empty);
      }
      else {
	combobox.setEnabled(true);
	for (int i=0; i<s.length; i++) combobox.addItem(s[i]);
      }
      action = true;
    }
    
    public boolean actionPerformed()
    {
      return action;
    }

    public void setSelectedItem(Object o)
    {
      combobox.setSelectedItem(o);
    }

    public String getSelectedItem()
    {
      if (combobox.isEnabled()) return (String) combobox.getSelectedItem();
      return "";
    }
  }

  public static class LabeledTextField extends JPanel {

    public JTextField textfield;
    public JCheckBox checkbox;
    public JComboBox units;
    private boolean action;
    private String label;

    private boolean CheckBoxSet;
    private boolean UnitsSet;
    
    /**-------------------------------------------------------------------
     * Principal constructor,
     *
     * @param label the label
     * @param text initial text displayed in the editor
     * @param withCheckBox true if the textfield is checkable
     *--------------------------------------------------------------------*/
    LabeledTextField(String lb, String text, boolean withUnits,
		     boolean withCheckBox)
    {
      label = lb;
      if (!withCheckBox) setLayout(new BoxLayout(this,BoxLayout.X_AXIS));
      else setLayout(new FlowLayout());
      JLabel l = new JLabel(label);
      l.setHorizontalTextPosition(l.RIGHT);
      //l.setDisplayedMnemonic(label.charAt(0));
      textfield = new JTextField(text);
      l.setLabelFor(textfield);
      CheckBoxSet = withCheckBox;
      UnitsSet = withUnits;
      if (withCheckBox) {
	checkbox = new JCheckBox();
	add(checkbox);
      }
      add(l);
      add(textfield);
      if (withUnits) {
	units = new JComboBox();
	add(units);
      }
      action = true;
    }

    /**-------------------------------------------------------------------
     * another constructor
     *--------------------------------------------------------------------*/
    LabeledTextField(String label, String text, boolean withUnits)
    {
      this(label,text,withUnits,false);
    }


    /**-------------------------------------------------------------------
     * another constructor
     *--------------------------------------------------------------------*/
    LabeledTextField(String label, String text)
    {
      this(label,text,false);
    }

    /**-------------------------------------------------------------------
     * and another constructor
     *--------------------------------------------------------------------*/
    LabeledTextField(String label, String text, int cols)
    {
      this(label,text);
      textfield.setColumns(cols);
      //Dimension d = textfield.getPreferredSize();
      //d.setWidth(textfield.getColumnWidth()*cols);
      //textfield.setPreferredSize(new Dimension(textfield.getColumnWidth*cols,));
    }

    /**-------------------------------------------------------------------
     * and again another constructor
     *--------------------------------------------------------------------*/
    LabeledTextField(String label, String text, boolean withUnits, 
		     int cols)
    {
      this(label,text,withUnits);
      textfield.setColumns(cols);
      //Dimension d = textfield.getPreferredSize();
      //d.setWidth(textfield.getColumnWidth()*cols);
    }

    public void setUnits(Vector v)
    {
      if (UnitsSet) {
	action = false;
	if (units.getItemCount() > 0) units.removeAllItems();
	if (v.size() == 0) {
	  units.setEnabled(false);
	  units.addItem(empty);
	}
	else {
	  units.setEnabled(true);
	  for (Enumeration e=v.elements(); e.hasMoreElements();) {
	    units.addItem(e.nextElement());
	  }
	}
	action = true;
      }
    }

    public void setUnits(String s[])
    {
      if (UnitsSet) {
	action = false;
	if (units.getItemCount() > 0) units.removeAllItems();
	if (s.length == 0) {
	  units.setEnabled(false);
	  units.addItem(empty);
	}
	else {
	  units.setEnabled(true);
	  for (int i=0; i<s.length; i++) {
	    units.addItem(s[i]);
	  }
	}
	action = true;
      }
    }

    public void setColumns(int cols) 
    {
      textfield.setColumns(cols);
    }

    public String getUnit() 
    {
      if (UnitsSet) {
	return (String) units.getSelectedItem();
      }
      return "";
    }

    public void setUnit(String u) 
    {
      if (UnitsSet) {
	units.setSelectedItem(u);
      }
    }
    
    public String getText() 
    {
      return textfield.getText();
    }

    public String getLabel() 
    {
      return label;
    }

    public void setText(String s) 
    {
      textfield.setText(s);
    }

    public void setEditable(boolean b) 
    {
      textfield.setEditable(b);
    }

    public void setEnabled(boolean b) 
    {
      checkbox.setEnabled(b);
    }

    public void setSelected(boolean b) 
    {
      textfield.setEnabled(b);
      if (CheckBoxSet) checkbox.setSelected(b);
      if (UnitsSet) units.setEnabled(b);
    }

    public boolean isSelected() 
    {
      return checkbox.isSelected();
    }

    public boolean actionPerformed()
    {
      return action;
    }
  }
  
  /**-------------------------------------------------------------------
   * A checkbox with a line of text added to it as a label
   *--------------------------------------------------------------------*/
  public static class CheckBoxWithText extends JPanel {
    
    JCheckBox checkbox;
    JLabel text;
    
    /**-------------------------------------------------------------------
     * Principal constructor,
     *
     * @param label the label on the button
     * @param s text to be added just at the right of the checkbox label
     *--------------------------------------------------------------------*/
    CheckBoxWithText(String label, String s)
    {
      checkbox = new JCheckBox(label);
      text = new JLabel(s);
      //checkbox.setSelected(false);
      add(checkbox);
      add(text);
    }   

    /**-------------------------------------------------------------------
     * another constructor,
     *
     * @param label the label on the button
     *--------------------------------------------------------------------*/
    CheckBoxWithText(String label)
    {
      this(label,"");
    }   

    public void setSelected(boolean b) 
    {
      checkbox.setSelected(b);
    }
    
    public void setText(String s)
    {
      text.setText(s);
    }

    public String getText()
    {
      return text.getText();
    }

  }

}



